#include "TPhSensorType.h"
#include <stdio.h>

ClassImp(TPhSensorType)

  //////////////////////////////////////////////////////////////////////////
  //                                                                      //
  // TPhSensorType Class                                                  //
  //                                                                      //
  // Implements the Mapping of                                            //
  //   (Bonding Pattern, Pad)  <-> (Row, Col) on the Sensor               //
  //                                                                      //
  //                                                                      //
  // Helpful Member Functions are :                                       //
  //                                                                      //
  //  1) GetRowCol(Int_t &Row, Int_t &Col, Int_t BondPat, Int_t Pad)      //
  //             ....Pass Bonding Pattern and Pad to get back Row and Col //
  //                                                                      //
  //  2) GetBondPatPad(Int_t Row, Int_t Col, Int_t &BondPat, Int_t &Pad)  //
  //             ....Pass Row and Col to get back Bonding Pattern and Pad //
  //                                                                      //
  //
  // $Id: TPhSensorType.cxx,v 1.3 1999/01/13 14:56:59 decowski Exp $ 
  //
  //////////////////////////////////////////////////////////////////////////

TPhSensorType::TPhSensorType()
{
}


TPhSensorType::~TPhSensorType()
{
  delete[] fData;
}

void TPhSensorType::SetSensorType(Int_t Type)
{
  Char_t Name[20];
  sprintf(Name, "SensorType_%d", Type);
  fType = Type;
  SetName(Name);
  SetTitle(Name);

}

// Screen Dump of ASCII Map ... mostly for Debugging
void TPhSensorType::Dump()
{
  Int_t i;
  printf("-----------------------------------------------------------\n");
  printf(" TYPE :: %d\t NumBondPat :: %d\t NumPadsPerBondPat :: %d\n", 
	 fType, fNumBondPat,fNumPadsPerBondPat);
  printf("-----------------------------------------------------------\n");
  for(i=0; i<fNumPadsPerBondPat*fNumBondPat; i++){
    printf("%d\t%d\t%d\t%d\n", fData[i].fRow, fData[i].fCol, 
	   fData[i].fBondPat, fData[i].fPad);
  }
  printf("-----------------------------------------------------------\n");
}

void TPhSensorType::GetRowCol(Int_t &Row, Int_t &Col, Int_t BondPat, 
			      Int_t Pad)
{
  Int_t i;

  for(i=0; i<fNumPadsPerBondPat*fNumBondPat; i++){
    if(fData[i].fBondPat == BondPat && fData[i].fPad == Pad){
      Row = fData[i].fRow;
      Col = fData[i].fCol;
    }
  }
#ifdef DEBUG
  printf("%d\t%d\t\t%d\t%d\n", Row,Col, BondPat, Pad);
#endif
}

void TPhSensorType::GetBondPatPad(Int_t Row, Int_t Col, 
				  Int_t &BondPat, Int_t &Pad)
{
  Int_t i;

  for(i=0; i<fNumPadsPerBondPat*fNumBondPat; i++){
    if(fData[i].fRow == Row && fData[i].fCol == Col){
      BondPat = fData[i].fBondPat;
      Pad = fData[i].fPad;
    }
  }
#ifdef DEBUG
  printf("%d\t%d\t\t%d\t%d\n", Row,Col, BondPat, Pad);
#endif
}



void TPhSensorType::Initialise(Int_t Type)
{
  Char_t str[80];
  Char_t Name[256];
  Int_t T;
  Int_t Row;
  Int_t Col;
  Int_t BondPat;
  Int_t Pad;
  Int_t i;
  
  FILE *handle;

  // Expect ASCII Data File to be called SensorT().txt
  sprintf(Name, "SensorT%d.txt", Type);
  handle = fopen(Name, "r");

  //Set Sensor Type Name explicitly
  sprintf(Name, "SensorType_%d", Type);
  fType = Type;
  SetName(Name);
  SetTitle(Name);
  
  //Based on Sensor Type, Decide fNumPadsPerBondPat and fNumBondPat:
  switch(Type)
    {
    case 1 : 
      fNumPadsPerBondPat = 128;
      fNumBondPat        = 12;
      break;
    case 2 :
      fNumPadsPerBondPat = 64 ;
      fNumBondPat        = 8;
      break;
    case 3 :
      fNumPadsPerBondPat = 64 ;
      fNumBondPat        = 8;
      break;
    case 4 :
      fNumPadsPerBondPat = 64 ;
      fNumBondPat        = 4;
      break;
    case 5 :
      fNumPadsPerBondPat = 64;
      fNumBondPat        = 4;
      break;
    default :
      printf("Invalid Sensor Type!\n");
      return;
      break;
    }

  fData = new DataStruct_t[fNumPadsPerBondPat*fNumBondPat];  
  for (i=0; i< fNumPadsPerBondPat*fNumBondPat; i++){
    fgets(str, 80, handle);
    sscanf(str, "%d %d %d %d %d", &T, &Row, &Col, &BondPat, &Pad);
    fData[i].fRow     = Row;
    fData[i].fCol     = Col;
    fData[i].fBondPat = BondPat;
    fData[i].fPad     = Pad;   
  }

}


void TPhSensorType::Streamer(TBuffer &R__b)
{
  // Stream an object of class TPhSensorType.
  //
  // Hand Modified!
  //  
  Int_t DataLength,i;
  Int_t *DataArray;

  if (R__b.IsReading()) {
    Version_t R__v = R__b.ReadVersion(); if (R__v) { }
    TNamed::Streamer(R__b);
    R__b >> fNumPadsPerBondPat;
    R__b >> fNumBondPat;
    R__b >> fType;
    DataLength = fNumPadsPerBondPat * fNumBondPat * 4;
    fData = new DataStruct_t[fNumPadsPerBondPat*fNumBondPat];  
    // Streaming Array to substitute fDataStruct
    DataArray = new Int_t[DataLength];
    R__b.ReadFastArray(DataArray, DataLength);
    for(i=0;i<DataLength/4;i++){
      fData[i].fRow    =DataArray[4*i];
      fData[i].fCol    =DataArray[4*i+1];
      fData[i].fBondPat=DataArray[4*i+2];
      fData[i].fPad    =DataArray[4*i+3];
    }
    delete[] DataArray;
  } else {
    R__b.WriteVersion(TPhSensorType::IsA());
    TNamed::Streamer(R__b);
    R__b << fNumPadsPerBondPat;
    R__b << fNumBondPat;
    R__b << fType;
    DataLength = fNumPadsPerBondPat * fNumBondPat * 4;
    // Streaming Array to substitute fDataStruct
    DataArray = new Int_t[DataLength];
    for(i=0;i<DataLength/4;i++){
      DataArray[4*i]  =fData[i].fRow    ;
      DataArray[4*i+1]=fData[i].fCol    ;
      DataArray[4*i+2]=fData[i].fBondPat;
      DataArray[4*i+3]=fData[i].fPad    ;
    }
    R__b.WriteFastArray(DataArray, DataLength);
    delete[] DataArray;
  }
}

















