// Made by A.O. on 4.11.1997
// Converts root file with MCTracks data container in the events to
// HIJET output compatible ascii file, appropriate to use as an input to Phobos MC.
//main()
{

//-----------------------------------------------+
// CopyRootToHijet.C                             |
// Read the ROOT file with Phobos events         |
// and convert it to Hijet Ascii compatible file.|
//                                               |
// ----------------------------------------------+

// Reset the ROOT environment
// do not reset unless you gave up defining input_name from root command line
   gROOT.Reset();

    Char_t *input_name = "pmc100ev89.hijet.aou.root";

    const Int_t   maxPlanes = 146; 
    const Float_t kPi = 3.14159;

    Int_t num=0; 
    Int_t numtrk=0;
    Int_t i; 
    Int_t fVerbose=0;
    Double_t theta,phi,pxy;

// Declare pointers to a TPhEvent and TPhTrack object
    TPhEvent *event;
    TPhTrack *track;

    Char_t FileName[256];
    const Char_t *phathome = gSystem->Getenv("PHATHOME");
    printf("Found PHAT home at %s\n",phathome);
    sprintf(FileName,"%s/data/%s",phathome,input_name);

//  Create a TPhDST object for accessing the ROOT data file
    TPhDST *dst = new TPhDST();
    dst->Open(FileName);

// Open the Ascii output file.
    sprintf(FileName,"%s/data/%s.hijet",phathome,input_name);

    FILE *out;
    out = fopen(FileName,"w");

   TObjArray        *CList;
   TPhDataContainer *DContainer;

   fprintf(out,"C------ \n");
   while(event = dst->GetNextEvent())
   {
   //  Write event header
       event->ls();

       // Assume tracks in MCTracks data container
       DContainer = (TPhDataContainer*) event->GetObject("MCTracks");
       CList = DContainer->GetObjectList();
       if( CList ) {
          printf("\nMCTracks data container: %d tracks in event\n", CList->GetEntries());

          TIter TrackIter(CList);
          while(track = (TPhMCTrack*) TrackIter()) {
            numtrk++;
            pxy = sqrt((Double_t) (track->GetPx()*track->GetPx()+track->GetPy()*track->GetPy())); 
            theta = 
              atan(pxy/(Double_t)((fabs(track->GetPz()) < 1.0e-7) ? 1.0e-7  :  track->GetPz()));
	    phi = acos(((Double_t)track->GetPx())/((pxy < 1.0e-7) ? 1.0e-7  :  pxy));
            if(track->GetPy() < 0) phi = 2.0*kPI-phi;
            fprintf(out,"P %5d %10.5g %10.5g %10.5g \n",
                    track->GetGPid(),
                    track->GetP(),theta,phi);
            if (fVerbose) track->Print();
          }
       }
// Don't forget to free the memory before going to next event
       delete event;
       fprintf(out,"E------ \n");
   }
   fclose(out);
}

