//  macro that reads Test Beam 98 data and produces TPhEvents with 
//  RawHitFECn0 objects, and more
//  version where only conversion occures
//  and even writing is canceled at the moment
main (Char_t *inputfilename=0)
{
  gROOT->Reset();

//Char_t *inputfilename = "ped005.dat";
  const kMaxEvents = 1000000;

  const Int_t kTB98Channels = 768;         // channels per string  (2 byte words)
  const Int_t kTB98StrRep = 2;             // strings per repeater 
  const Int_t kTrailerLength = 20;         // trailer length       (2 byte words)
  const Int_t kTB98BufferSize = ((kTB98Channels*kTB98StrRep)+kTrailerLength)*2;  // 3112 bytes

  // sequencing and resequencing FEC and string numbers
  const Int_t kFECNo[16]      = {8,0,8,0,8,0,8,0,9,1,9,1,9,1,9,1}; 
  const Int_t kStringRem[16]  = {5,0,4,1,5,0,4,1,5,0,4,1,5,0,4,1};
  const Int_t kStringIn[16]   = {0,5,1,4,0,5,1,4,0,5,1,4,0,5,1,4};
  const Int_t kStringOut[16]  = {0,1,1,0,2,3,3,2,0,1,1,0,2,3,3,2}; 

  if(!inputfilename) {
     printf("Call main(filename) with the input file name as a parameter\n");
     return;
  }
  Char_t outputfilename[100];
  sprintf(outputfilename,"%s.root",inputfilename);
  printf("Converting TB98 file %s to %s ROOT format\n",inputfilename,outputfilename);

  // input file
  FILE *inputfile;
  if(!(inputfile = fopen(inputfilename,"rb"))) {
     printf("Error: Input file %s could not be found\n",inputfilename);
     return;
  }      

  // output file
  TFile *outputfile;
  if(!(outputfile = new TFile(outputfilename,"RECREATE","Test Beam 98 data"))) {
     printf("Error: Output file %s could not be opened.\n",outputfilename);
     return;
  }

  // Get TB98 detector
  const Char_t *phathome = gSystem->Getenv("PHATHOME");
  Char_t *detpath[100];
  sprintf(detpath,"%s/init/Detector_TB98.root",phathome);
  TFile *detfile = new TFile(detpath);
  TPhDetector *detector = detfile->Get("PMC_TB98_V4.30");
  
  // prepare for conversion and read the raw byte stream from TB98 data file
  Int_t run=0,counter=0,string=0,eventcounter=1,repeater=0,eventnumber=0;
  Char_t eventname[30];

  Int_t xpix=0,ypix=0,adc=0;
  TPhSiSensor *plane=NULL;
  TPhHitArray *ha=NULL;
  TPhEvent *event = NULL;
  TPhObjectContainer *rawcont = NULL;
  TPhRawHitFECn0TB98 *rawhit = NULL, *rawhitclone = NULL;
  UChar_t *readbuffer = new UChar_t[kTB98BufferSize];
  
  outputfile->cd();
  while( !feof(inputfile) ) {
    fread(readbuffer,kTB98BufferSize,1,inputfile);
    repeater = counter%8;
    counter++;
    
    if(!feof(inputfile)) {
      // fill FEC with 1 repeater info (2 strings)
      // - from 1 FEC with wrong string positions make 
      //   2 FECs each with one string at the correct position
      rawhit = new TPhRawHitFECn0TB98(readbuffer, kTB98BufferSize);
      // make a clone object
      rawhitclone = (TPhRawHitFECn0TB98 *) rawhit->Clone();    
      
      // set the proper FEC number for this string
      rawhit->SetFECNo(kFECNo[string]);
      // remove 2nd string from one FEC
      if(!rawhit->RemoveString(kStringRem[string])) return;
      // move 1st to the proper position
      if(!rawhit->MoveString(kStringIn[string],kStringOut[string])) return;
      string++; string = string%16;
      
      // set the proper FEC number for this string
      rawhitclone->SetFECNo(kFECNo[string]);
      // remove 1st string from 2nd FEC
      if(!rawhitclone->RemoveString(kStringRem[string])) return;
      // move 2nd to the proper position
      if(!rawhitclone->MoveString(kStringIn[string],kStringOut[string])) return;
      string++; string = string%16;
    }

    // create new TPhEvent if repeater number = 0
    if(!repeater) {
      if(counter>1) {
	 // actions on the event_by_event basis
	 //---------------------------------------
         detector->SetEvent(event);
	 // make hit arrays outof raw hits
         detector->MakeHitArrays("RawHitFECn0s","HitArraysTB98");
	 // attach them to the sensors
         detector->FillWithEventHitArrays("HitArraysTB98");
         plane = (TPhSiSensor *) detector->GetSensor("NA00");
         ha = plane->GetHitArray();
	 //NA00 for example
         for(xpix=0;xpix<70;xpix++) {
	   for(ypix=0;ypix<22;ypix++) {
	     // do with them what you need
	     // ypix - rows - first, 
             adc = ha->GetADC(ypix,xpix);
	   }
	 }
         //---------------------------------------
         outputfile->cd();
	 event->Write();
 	 delete event;
         if((eventcounter >= kMaxEvents) || (feof(inputfile)) ) break;
      }
       eventnumber = rawhit->GetEventNumber();
       sprintf(eventname,"Event_%d",eventnumber);
       event = new TPhEvent(run,eventnumber);
       event->SetName(eventname);
       event->SetTitle("Test Beam 98");

       printf("Event no: %d\n",eventcounter);
       eventcounter++;

       // create and add a container with raw hits to the event object
       rawcont = new TPhObjectContainer("RawHitFECn0s", "Test Beam 98");
       event->AddObject(rawcont);
    }
    rawcont->Add(rawhit);
    rawcont->Add(rawhitclone);
  }
  delete [] readbuffer;

  if(ferror(inputfile)) printf("Error occured while reading file %s!\n",inputfilename);
  fclose(inputfile); 
  outputfile->Close();
}
