// Read events from a Root file and displays energy deposition in one of the
// Sensors.
//
//Begin_Html <A HREF="mailto:decowski@mit.edu">decowski@mit.edu</A>
//End_Html
{
    gROOT->Reset();

    Int_t      i;

    TPhMCHit    *hit;
    TPhEvent  *event;
    TPhObjectContainer *HitContainer;

    // We will histogram the energy deposition in one sensor
    TCanvas c1("Show energy deposition");
    TH1F *EDep = new TH1F("EDep"," Energy Deposition in VBA0", 
			  100, 0, 0.0006);

    // Open the data file. File can be generated by the write_events.C macro
    // As with all Phat stuff, PHATHOME envrionment has to be defined and 
    // pointing to the main Phat directory
    Char_t FileName[200];
    sprintf(FileName, "%s/data/cv0_00000e1.apx.root", 
	    gSystem->Getenv("PHATHOME"));
    TPhDST *dst = new TPhDST(FileName);

    // Get the detector object. This is only needed by TPhSwim, to trace the 
    // tracks through the detector
    Char_t FileName[256];
    sprintf(FileName,"%s/init/test/Detector.root", 
	    gSystem->Getenv("PHATHOME"));
    TFile f(FileName);
    TPhDetector *det = f.Get("PMC_V4.20");
    f.Close();

    // Get a spectrometer sensor.
    TPhVTXSensor *sensor = det->GetSensor("VBA0");
    if(!sensor) {
      printf("No such sensor!\n");
      return;
    }

    // This loop loops over all events. 
    while(event = dst->GetNextEvent()) {
      printf("Found %d %d \n",event->GetNRun(), event->GetNEvent());

      // Get the container with the hits
      HitContainer = (TPhObjectContainer*) event->GetObject("MCHits");

      // iterate over all hits in the container and see if the label 
      // of the hits is the same as the label of the sensor
      TIter Hits(HitContainer->GetIterator());
      while(hit = (TPhMCHit*) Hits()) {
	if (hit->GetSensorLabel() == sensor->GetLabel())
	  EDep->Fill(hit->GetDe());       
      }
      delete event;
    }
    EDep->Draw();
    EDep->GetXaxis()->SetTitle("dE (GeV)");
}
