//
//
{
   gROOT->Reset();
  
   const Char_t *PhatDirName = gSystem->Getenv("PHATHOME");
   const Char_t *PMCDirName = gSystem->Getenv("PMCHOME");
   const Int_t runno = 1;

   Char_t gfilename[100];
   TPhPMC *pmc;

   if((pmc = TPhPMC::GetInstance()) == NULL) {
     pmc = TPhPMC::CreateInstance();
     if( !pmc->Init() ) {
       printf("PMC initialization failed - aborting !\n");
       exit(0);
     }
   }

   //   pmc->SetVerbose(2);
   TStopwatch timer;

   // Open output data file in root format.
   TFile *file = new TFile("data.root","RECREATE");
   if( !file->IsOpen() ) {
      printf("Output file %s could not be opened!\n", outputfile);
      exit(0);
   }

   // Make a new run object 
   // Handling of run and event objects is done now outside of the TPhPMC class
   TPhRun *run;
   run= new TPhRun(runno);
   run->SetTitle("Run generated by TPhPMC");
   // Set link to run object (temporary solution)
   pmc->SetRunLink(run);

   TPhEvent *event, *testeventp;
   TPhMCTrack *testtrackp;

   TPhDetector *det;

   // Detector has been created after Init() only
   // if SMEM parameter is turned on in the control file.
   det = TPhDetector::GetInstance();
   printf("\n(PHAT)PMC run with:\n GEANT ver.%6.3f\n PMC ver. %6.3f\n Geometry ver. %6.3f\n",
           pmc->GetGeantVer(),pmc->GetPmcVer(),pmc->GetPmcDetVer());

   Char_t parname[10];
   Int_t number = 0; Float_t values[5] = {0,0,0,0,0};
   // An "event at a time" mode (other modes available)
   // can only be set in a RHDATA control-file by GEN=0 
   // Some parameters can be changed interactively e.g.:

   // make sure to read tracks from memory (set by SwimPhatTrack() function) 
   strcpy(parname,"GEN "); number = 1; values[0] = 10;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }

   /*
   // Turn-off multiple scattering
   strcpy(parname,"MULS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off decays
   strcpy(parname,"DCAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off hadronic interactions
   strcpy(parname,"HADR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off energy loss
   strcpy(parname,"LOSS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off pair production
   strcpy(parname,"PAIR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off bremstrahlung
   strcpy(parname,"BREM"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off compton scattering
   strcpy(parname,"COMP"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off photoelectric effect
   strcpy(parname,"PHOT"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off delta-rays
   strcpy(parname,"DRAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off annihilation
   strcpy(parname,"ANNI"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }

   // Other parameters available to change:
   // CUTS MULS DCAY HADR LOSS PAIR BREM COMP PHOT DRAY ANNI PIXL 
   // SECT RBXX RBYY RBZZ RMOM RPID RETA RPHI RARM RMUL 
   // GEN TRIG SAOU SMEM
   // their meaning is described inside the control-file, 
   // current values are printed by a call to pmc->PrintPMCPars().
   */
   pmc->PrintPMCPars();

   Float_t mom = 1.0;
   for(Int_t evc = 0;  evc < 1; evc++) {
     // TPhEvent event object is created by a user and then filled by 
     // PMC with pmc->Run() call this outputs (one or more (daughter)) 
     // particles together with their hits to the event
     event = new TPhEvent(runno,evc);


     timer.Start();
     mom = 1;
     for(Int_t counter = 0; counter < 1000; counter++) {
       testtrackp = pmc->SwimPhatTrack(8, gRandom->Rndm() * 14 - 7, 
				       gRandom->Rndm() * 2 - 1, 
				       100, 0, 0.0, mom,
				       event, "PMCTracks", "PMCHits");
	 if(!testtrackp) exit(0);
       }
     file->cd();
     event->Print("1");
     event->Write();
     run->AddEvent(event);

     //     delete event;

     // after every event built one can clear the base of track numbering
     // to let them start numbering from zero again
     pmc->SetTrackBaseNo();
   }

   TPhObjectContainer *tracks = 
     (TPhObjectContainer*)event->GetObject("PMCTracks");
   TCanvas c("EventDisplay");
   det->Draw();
   tracks->Draw();

   timer.Stop();
   timer.Print();

   run->Write();
   if(det) det->Write("LatestGeometry");
   file->Close();
}
