// A.O. 18.06.1998
// example macro to run one particle through Phobos setup with the PMC code.
// Warning: TPhPMC singleton object is allowed to be initialized by Init() function
//          only once so the macro (as a whole) can not be repeatedly executed.
// Changed: 03.12.1998 A.O.
//          To add creation of TPhRun
//
// $Id: single_particle.C,v 1.3 1998/12/09 10:49:17 olszewsk Exp $
runpmc(Int_t runno=1)
{
   gROOT->Reset();
  
   const Char_t *PhatDirName = gSystem->Getenv("PHATHOME");
   const Char_t *PMCDirName = gSystem->Getenv("PMCHOME");
   Char_t gfilename[100];

   TPhPMC *pmc = TPhPMC::CreateInstance();
   //   pmc->SetVerbose(2);
   TStopwatch timer;

   // Open output data file in root format.
   TFile *file = new TFile("data.root","RECREATE");
   if( !file->IsOpen() ) {
      printf("Output file %s could not be opened!\n", outputfile);
      exit(0);
   }

   // Make a new run object 
   // Handling of run and event objects is done now outside of the TPhPMC class
   TPhRun *run;
   run= new TPhRun(runno);
   run->SetTitle("Run generated by TPhPMC");
   // Set link to run object (temporary solution)
   pmc->SetRunLink(run);

   TPhEvent *event, *testeventp;
   TPhMCTrack *testtrackp;

   TPhDetector *det;
   // The detector is created automatically from the UGCDATA file 
   // when TPhPMC object gets initialized,
   // and SMEM - "writing to memory" option is turned on in the control-file.

   // For a correct initialization standard PMC I/O files must be set via soft links
   // They can be set under system shell or by calls to 
   // TPhPMC::SetPathToFile(Char_t *filesymbol, Char_t *filename) before
   // Init() has been called.
   // "filesymbols" are: 
   // -----------------------------------------------------
   // FLDASC;           //  path to magnetic field map file
   // GENEVS;           //  path to HIJET input data file
   //    the links to the above two file types can be
   //    changed even after Init() has been called.
   // -----------------------------------------------------
   // GENEVS;           //  path to HIJET input data file
   // RHAOU;            //  path to output ascii AOU file
   // RHDATA;           //  path to control file with PMC parameters
   // RHASC;            //  path to console output file
   // RHHST;            //  path to output histogram file
   // RHNTU;            //  path to ntuple output file for tracks with hits
   // MCNTU;            //  path to ntuple output file for all tracks
   // RHZEB;            //  path to RZ-format Zebra bank output file
   //
   // If file names are not specified an attempt to connect "standard" files at locations
   // PMCHOME, PHATHOME or . (current directory) is made.
   //   sprintf(gfilename,"%s","physics_control");
   //   pmc->SetPathToFile("RHDATA", gfilename);

   // False for pmc->Init() means file names are not set correctly! then exit safely
   if( !pmc->Init() ) {
     printf("PMC initialization failed - aborting !\n");
     exit(0);
   }
   // Detector has been created after Init() only
   // if SMEM parameter is turned on in the control file.
   det = TPhDetector::GetInstance();
   printf("\n(PHAT)PMC run with:\n GEANT ver.%6.3f\n PMC ver. %6.3f\n Geometry ver. %6.3f\n",
           pmc->GetGeantVer(),pmc->GetPmcVer(),pmc->GetPmcDetVer());

   Char_t parname[10];
   Int_t number = 0; Float_t values[5] = {0,0,0,0,0};
   // An "event at a time" mode (other modes available)
   // can only be set in a RHDATA control-file by GEN=0 
   // Some parameters can be changed interactively e.g.:

   // make sure to read tracks from memory (set by SwimPhatTrack() function) 
   strcpy(parname,"GEN "); number = 1; values[0] = 10;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }

   /*
   // Turn-off multiple scattering
   strcpy(parname,"MULS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off decays
   strcpy(parname,"DCAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off hadronic interactions
   strcpy(parname,"HADR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off energy loss
   strcpy(parname,"LOSS"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off pair production
   strcpy(parname,"PAIR"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off bremstrahlung
   strcpy(parname,"BREM"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off compton scattering
   strcpy(parname,"COMP"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off photoelectric effect
   strcpy(parname,"PHOT"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off delta-rays
   strcpy(parname,"DRAY"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }
   // Turn-off annihilation
   strcpy(parname,"ANNI"); number = 1; values[0] = 0;
   if( !pmc->SetPMCParValue(parname, &number, values) ) {
      printf("PMC unable to change value of %s parameter: aborting!\n",parname);
      exit(0);
   }

   // Other parameters available to change:
   // CUTS MULS DCAY HADR LOSS PAIR BREM COMP PHOT DRAY ANNI PIXL 
   // SECT RBXX RBYY RBZZ RMOM RPID RETA RPHI RARM RMUL 
   // GEN TRIG SAOU SMEM
   // their meaning is described inside the control-file, 
   // current values are printed by a call to pmc->PrintPMCPars().
   */
   pmc->PrintPMCPars();

   Float_t mom = 1.0;
   for(Int_t evc=0; evc<2; evc++) {
     // TPhEvent event object is created by a user and then filled by PMC with pmc->Run() call
     // this outputs (one or more (daughter)) particles together with their hits to the event
     event = new TPhEvent(runno,evc);

     mom = 1.0;
     timer.Start();
     while ( (mom-=0.001) > 0.95 ) 
       {
	 // the command bellow can be repeated as many times as necessary
	 // to accumulate particles in an TPhEvent object.
	 // set the (input) particle characteristics (id,vx,vy,vz,px,py,pz).
	 testtrackp = pmc->SwimPhatTrack(8,0,0,0,mom+0.001,0.0,mom,
					 event, "PMCTracks", "PMCHits");
	 // creation of Phat TPhTracks and TPhHits occurs only 
	 // if SMEM value is set > 0 in the RHDATA file.
	 // SMEM value cannot be increased to > 0 after a call to Init() to prevent
	 // from use of different geometries in PMC simulations and hit labeling.
	 // abort if for some reason SwimPhatTrack is not producing events
	 if(!testtrackp) exit(0);
       }
     file->cd();
     event->Print("1");
     event->Write();
     run->AddEvent(event);

     delete event;
     // after every event built one can clear the base of track numbering
     // to let them start numbering from zero again
     pmc->SetTrackBaseNo();
   }
   timer.Stop();
   timer.Print();

   run->Write();
   if(det) det->Write("LatestGeometry");
   file->Close();
}
