{
// ReconEvents.C
//
// Reconstruct PHOBOS events using the template matching algorithm

// Clean the ROOT environment 

    gROOT->Reset();
    gROOT->Time(); 
   

   Int_t i;

// Define a couple of pointers to be used later

    TPhEvent *event;
    TPhHit   *hit;  
    TPhTrack *track;

    Char_t FileName[256];

// For some of the later stuff we will need to know the 
// location of the $PHATHOME directory. Use the TSystem 
// class (thanks, Fons) for getting this variable:

   const Char_t *phathome = gSystem->Getenv("PHATHOME");
   printf("Found PHAT home at %s\n",phathome);

// Create some histograms

   TH1F nhits("nhits","nhits",20,0,20);
   TH1F chi2("chi2","chi^2 per dof",100,0,5);
   TH1F prob("prob","chi^2 probability",250,0,1);

   TH1F mom("mom","mom",100,0.1,0.2);
   TH1F delta_p("delta_p","delta_p",100,-0.025,0.025);
   TH2F pty_in("pty_in","pt vs rapidity, Input",30,-6,6,20,0,2);
   TH2F pty_acc("pty_acc","pt vs rapidity, acc",20,0,2,20,0,0.5);
   TH2F pty_found("pty_found","pt vs rapidity, found",20,0,2,20,0,0.5);
   TH1F p_acc("p_acc","p (GeV), acc tracks\n",20,0.1,0.4);
   TH1F p_found("p_found","p (GeV), found tracks\n",20,0.1,0.4);

// Also create a canvas into which the histogram will be 
// drawn

    TCanvas c("c");

// Create a few important objects

    TPhTemplateManager *tman  = new TPhTemplateManager();
    TPhTrackSelector   *tsel  = new TPhTrackSelector();
    TPhStraightHough   *shough= new TPhStraightHough();
    TPhSwim            *swim  = new TPhSwim();
    TPhStuff           *stuff = new TPhStuff();
    TPhQPair           *pair  = new TPhQPair();

// TPhIdealMagnet and TPhDetector are needed as interfaces to 
// the magnetic field and the geometry

    TPhIdealMagnet   *magnet = new TPhIdealMagnet();
    TPhDetector *det = new TPhDetector();

// Fill the sensor list from the data files in the 
// $PHATHOME/init/test directory

 
    sprintf(FileName,"%s/init/test.2d",phathome);
    det->ReadASCIIGeometry(FileName);

    printf("Finished Geometry\n");
    swim->SetDetector(det);
    swim->SetMagnet(magnet);
    swim->SetVerbose(0);

    tman->SetDetector(det);
    tman->SetHough(shough);
    tman->SetMaxCompDist(3.5);
    tman->SetMinCompEnergy(0.);

    sprintf(FileName, "%s/macros/templates.root", phathome);
    tman->ReadTemplateList(FileName);

    TObjArray *list = tman->GetTemplateList();
    TObjArray *TrackList;

    shough->SetDetector(det);
    
// Reset the event counter

    Int_t num = 0;
    Float_t x;
    Float_t px;
    Float_t pz;
    Float_t max_prob;
    Float_t best_p;   

shough->SetVerbose(88);

// Now read events from the file until we either hit the 
// end-of-file or have read 100 events, whichever comes 
// first

// Open the input file with the events. So far these events
// contain only track momentum information as generated by an
// event generator (in this case the TPhRandomEvent class).

// Again, use phathome to find the data file

    sprintf(FileName,"%s/data/test.root",phathome);

// Now create a TPhDST object to access the data file at the
// path given by FileName

    TPhDST *dst = new TPhDST(FileName);

// Reset the event counter
    Int_t num = 0;

// This is the container where all the tracks are stored;
    TPhObjectContainer *AllTracks;

// Now read events from the file until we either hit the
// end-of-file or have read 100 events, whichever comes
// first

    while((event = dst->GetNextEvent()) && num < 1000)
    {

        printf("Now at event %d %d\n",event->GetNRun(),event->GetNEvent());
	AllTracks = (TPhObjectContainer*) event->GetObject("MCTracks");
        printf("Number of total tracks is %d\n",
	       AllTracks->GetObjectList()->GetEntries());

// Here we use the TPhSwim object to propagate each track
// in the event through the PHOBOS setup (actually only the
// right arm of the spectrometer for the present implementation
// of TPhSwim.

        swim->SwimEvent(event);

	//mpd TPhDetector must also know about the event!
	det->SetEvent(event);

// Use the GetNextTrack() iterator from TPhEvent to
// loop over all tracks in the event. For those with at
// least one hit in the spectrometer, fill the number of
// hits into a histogram that was defined above.

	TObjArray *AllTracksList = AllTracks->GetObjectList();
	TIter NextTrack(AllTracksList);

        while(track = (TPhTrack *) NextTrack())
        {
            pty_in.Fill(track->GetRap(0.139),track->GetPt());
            if(track->GetNHits() > 0)
            {
                nhits->Fill(track->GetNHits());
                if(track->GetNHits() > 12 &&
                   track->GetPx() > 0.1   &&
                   track->GetPx() < 0.25  &&
                   track->GetPz() > 0.1   &&
                   track->GetPz() < 0.25  &&
                   track->GetCharge() == 1)
                {
                     p_acc.Fill(track->GetP());
                     printf("Search for track %g %g %g %d\n",
                              track->GetPx(),
                              track->GetPy(),
                              track->GetPz(),
                              track->GetNHits());
                }
            }
        }
        printf("hough transform\n");
        shough->TransformHits();
        shough->SetVerbose(1);

        printf("template matching\n");
        tman->SetVerbose(2);
        tman->CreateTrackCandidates();

        printf("track selection\n");

        TObjArray *CandList = tman->GetCandidateList();
        tsel->SetVerbose(1);
        tsel->SelectTracks(CandList);

        TrackList = tsel->GetTrackList();
        TPhTrack *FoundTrack;
        printf("TrackList %x\n",TrackList);
        printf("TrackList %d entries\n",TrackList->GetEntries());
        for(i=0;i<TrackList->GetEntries();i++)
        {
           FoundTrack = (TPhTrack *) TrackList->At(i);
           printf("Selected track with p = %g,%g,%g  prob %g, %d hits\n",
                         FoundTrack->GetPx(),
                         FoundTrack->GetPy(),
                         FoundTrack->GetPz(),
                         FoundTrack->GetProb(),
                         FoundTrack->GetNHits());
            pty_found.Fill(FoundTrack->GetRap(0.139),FoundTrack->GetPt());
            p_found.Fill(FoundTrack->GetP());
        }

// Draw the histogram

        if(!(num%100))
        {
            p_acc.Draw("");
            p_found.Draw("samee1p");
            c.Update();
        }

// Remove hits from sensor hit buffers

        TrackList->Clear();
        CandList->Delete();
        det->DeleteHits();
        delete event;
        num++;
    }
}
