// Macro to show how to use the Hough Transform. 
// It uses the simulated tracks that belong to the event to 
// test against the Hough Transformed hits. 
//
// $Id: recon_hough.C,v 1.2 1998/09/18 03:14:50 decowski Exp $
{
  gROOT->Reset();

  // Grab a detector
  TPhDetector *det = TPhDetector::ReadLatestGeometry();
  // Instantiate the Hough Transformer
  TPhStraightHough   *shough = new TPhStraightHough();

  // We will get MUCH verbose output, disable if you don't want it
  shough->SetVerbose(1);
  
  // Grab the event file 
  TPhDST data("$PHATHOME/data/cv0_00000e1.apx.root");
  TPhEvent *event;

  event = data.GetNextEvent();
  //  while(event = data.GetNextEvent()) {
    det->SetEvent(event);

    // The Hough transform collects the hits from the spectrometer 
    // detector
    det->FillWithEventHits("MCHits");

    // Make the Hough transform
    shough->TransformHits();

    // We are now ready to use the Hough transform to test the 
    // various tracks (in this case the simulated ones, lacking anything
    // better, but these will obviously be the track templates like it is 
    // done in TPhTemplateManager::CreateTrackCandidates())
    TObjArray HoughTracks;

    TPhObjectContainer *Tracks = (TPhObjectContainer*) 
      event->GetObject("MCTracks");

    // Loop over all tracks and compare the first 4 planes for 
    // compatibility with the Hough transform.
    TIter NextTrack(Tracks->GetIterator());
    TPhMCTrack *Track;
    while(Track = (TPhMCTrack*) NextTrack()) {
      if(shough->CheckVector(Track)) {
	printf("Track: (%f, %f, %f) was ACCEPTED\n", 
	       Track->GetPxz(), Track->GetTheta(), Track->GetPy());
	HoughTracks.Add(Track);
      }
    }

    // Make a cute display of the accepted tracks
    TCanvas c("DetectorView");
    det->GetSubDetector("Vertex")->TurnOff();
    det->GetSubDetector("RingMult")->TurnOff();
    det->GetSubDetector("Trigger")->TurnOff();
    det->GetSubDetector("Octagon")->TurnOff();
    det->GetSubDetector("TOFN")->TurnOff();
    det->GetSubDetector("TOFP")->TurnOff();

    det->Draw();
    HoughTracks.Draw();
//}
}

