// Reconstruct PHOBOS events using the template matching algorithm
{
// Clean the ROOT environment 
    gROOT->Reset();
    gROOT->Time(); 
   
// Get the directory where we sit in
   const Char_t *phathome = gSystem->Getenv("PHATHOME");
   printf("Found PHAT home at %s\n",phathome);

// Create some histograms
   TH1F nhits("nhits","nhits",20,0,20);
   TH1F chi2("chi2","chi^2 per dof",100,0,5);
   TH1F prob("prob","chi^2 probability",250,0,1);

   TH1F mom("mom","mom",100,0.1,0.2);
   TH1F delta_p("delta_p","delta_p",100,-0.025,0.025);
   TH2F pty_in("pty_in","pt vs rapidity, Input",30,-6,6,20,0,2);
   TH2F pty_acc("pty_acc","pt vs rapidity, acc",20,0,2,20,0,0.5);
   TH2F pty_found("pty_found","pt vs rapidity, found",20,0,2,20,0,0.8);
   TH1F p_acc("p_acc","p (GeV), acc tracks\n",20,0.1,0.4);
   TH1F p_found("p_found","p (GeV), found tracks\n",20,0.1,0.4);

// Get detector object from ROOT file 
    Char_t FileName[512];
    sprintf(FileName, "%s/data/pi300_1000x100.api.root", phathome);

    TFile f(FileName);

    TPhDetector *det = f.Get("TPhDetector");
    f.Close();

// This needs to be fixed!! have to call MakeTranslateObject explicitly!
det->MakeTranslateObject();

// Open a TCanvas and draw the detector

/*
    TCanvas c2("view");
    TView v(1);
    v.SetRange(-5, -5, -5, 80, 20, 80);

    det->Draw();
*/


// Create a few important objects

    TPhStraightHough   *shough= new TPhStraightHough();
    TPhTemplateManager *tman  = new TPhTemplateManager();
    TPhTrackSelector   *tsel  = new TPhTrackSelector();
    TPhStuff           *stuff = new TPhStuff();
    TPhQPair           *pair  = new TPhQPair();


// Read the 100 templates from file

    tman->ReadTemplateList("templates_exact.root");

// Reset the event counter

    Int_t num = 0;
    Float_t x;
    Float_t px;
    Float_t pz;
    Float_t max_prob;
    Float_t best_p;   

// Now open the event data base using TPhDST and reconstruct
// the straight line part of the tracks.

    TPhDST dst(FileName);
    TPhEvent   *event;
    TObjArray  *tracklist;
    TPhMCTrack *mctrack;

// Switch on debugging output for straight line finder
    shough->SetVerbose(2);
    tman->SetVerbose(1);
    tsel->SetVerbose(1);

    num = 0;
    Int_t i;
    Int_t j;
    TObjArray *list;
    TObjArray *TrackList;
    TPhMCHit    *hit;
    while((event = dst->GetNextEvent()) && (num < 100))
    {
        printf("Search straight tracks in event %d %d\n",event->GetNRun(),event->GetNEvent());
        det->SetEvent(event);
        TPhObjectContainer *dat = (TPhObjectContainer *) event->GetObject("MCTracks");
        TrackList = dat->GetObjectList();
        for(i=0;i<TrackList->GetEntries();i++)
        {
            mctrack = (TPhMCTrack *) TrackList->At(i);
            list = mctrack->GetHitList();
            if(list && mctrack->GetPx() > 0)
            {
                printf("%d: Search track %g %g %d: %d hits\n",i,
                        mctrack->GetPx(),
                        mctrack->GetPz(),
                        mctrack->GetCharge(),list->GetEntries());
		pty_in.Fill(mctrack->GetRap(0.14), mctrack->GetPt());

                for(j=0;j<list->GetEntries();j++)
                {
                    hit = (TPhMCHit *) list->At(j);
                    printf("hit %d: %g %g %g %d %s\n",j,
                           hit->GetX(),
                           hit->GetY(),
                           hit->GetZ(),
                           hit->GetLayer(),
			   hit->GetSensorName());
                }
            }
        }
        list = det->GetSensorList();
        printf("Make hitlists for %d sensors\n",list->GetEntries());
        for(i=0;i<list->GetEntries();i++)
        {
            sensor = (TPhSensor *) list->At(i);
            if(strstr(sensor->GetChLabel(),"S"))
            {
	      sensor->SetVerbose(1);
	      sensor->MakeHitArrays("MCRawHitBuffers", "TempHitArrays");
	      sensor->MakeHitList("MCRawHitBuffers", "TempHitArrays");
            }
        }
        shough->TransformHits();
        tman->CreateTrackCandidates();
        tsel->SelectTracks(tman->GetCandidateList());
        list = tsel->GetTrackList();
        printf("Found %d tracks\n",list->GetEntries());
        for(i=0;i<list->GetEntries();i++)
        {
            track = (TPhTrack *) list->At(i);
            pty_found->Fill(track->GetRap(0.14),track->GetPt());
            printf("Found track %g %g %d\n\n\n\n",track->GetPx(),track->GetPz(),track->GetCharge());
        }
        det->DeleteHits();
        TrackList->Clear();
        num++;
	delete event;
    }
    delete det;
}




