#ifndef _matrix_h
#define _matrix_h

////////////////////////////////////////////////////////////////////
//
// matrix class. This class is used for the covariance matrix. 
// This class is in the public domain and was originally written 
// by Kyriakos Georgiou <kdgrc@rc.rit.edu>. It was adopted to Root
// by Gunther Roland. 
// 
// $Id: matrix.h,v 1.2 1998/05/30 02:30:25 decowski Exp $
//
////////////////////////////////////////////////////////////////////

#include <iostream.h> 
#include <fstream.h>

// Root libs
#include "TObject.h"

enum mat_type { identity };

#ifdef __RUN_TIME_CHECKING__
#include <stdlib.h>
#endif

#undef minor

class matrix : public TObject {
// mpd Note: for compatibility reasons on the different processors,
// mpd this must be an UInt_t == 4 bytes unsigned integer. avoid 
// mpd using "unsigned long" == 8 bytes on alpha, 4 on others. 

        UInt_t row;                                        // row size
	UInt_t col;                                        // column size
	Double_t *mat;                                     //!matrix data 
	Int_t error;

	matrix lu(Int_t& i, matrix& p) const;               // lu decomposition
	matrix backsub(matrix& B, matrix& X, 
		       const matrix& P, 
		       const UInt_t xcol) const;	    // back-substitution

public:
        matrix(UInt_t x, UInt_t y);                         // regular constructor
	matrix(UInt_t x, UInt_t y, 
	                    enum mat_type type );           // unity constructor
	matrix(UInt_t x, UInt_t y, Double_t init);          // init constructor
	matrix(void);                                       // null constructor
	matrix(const matrix& m);                            // copy constractor
	~matrix(void);                                      // destructor

	Double_t *matval() const { return this->mat; }     // read mat priv. part

	UInt_t rowlen() const { return this->row; }        // read row priv. part
	UInt_t collen() const { return this->col; }        // read col priv. part
	void poke(UInt_t x, UInt_t y, Double_t a);          // poke an element
	Double_t peek(UInt_t x, UInt_t y) const;            // peek an element

	Double_t det() const;                               // determinant
	Double_t minor(UInt_t x, UInt_t y);                 // minor
	Double_t cofactor(UInt_t x, UInt_t y);              // cofactor

	matrix& operator = (const matrix& m);               // assign
	matrix& operator += (const matrix& m);              // add & assign
	matrix& operator -= (const matrix& m);              // subtract & assign
	matrix& operator *= (const double a);               // mul. scalar & assign
	matrix& operator *= (const matrix& m);              // multiply & assign
	matrix operator ~ () const;		            // inverse
	matrix operator % (const matrix& m) const;	    // linear eq. solver

	Int_t err() const { return this->error; }	    // read error priv. part

        ClassDef(matrix,1)
};

matrix operator + (const matrix& m, const matrix& n);   // add
matrix operator * (const matrix& m, const matrix& n);   // multiply
matrix operator * (const Double_t a, const matrix& n);	// multiply by a const
matrix operator * (const matrix& n, const Double_t a);	// multiply by a const
matrix operator - (const matrix& m, const matrix& n);   // subtract
matrix operator - (const matrix& m);                    // unary minus
matrix operator ! (const matrix& m);                    // transpose

#endif /* matrix.h */
