#ifndef __TPhAbsRawHitFEC__
#define __TPhAbsRawHitFEC__


/////////////////////////////////////////////////////////////////////
// 
// Class describing the Abstract Interface to the FEC storage. This 
// class does not store any data, but describes how the data from the
// FEC is accessible to the user. 
//
// The TPhRawHitFECn0 (non 0-suppressed) and TPhRawHitFECs0 (0-supp.)
// classes implement the data storage.
// 
// Created: March 23, 1998 by Patrick Decowski
// 
// $Id: TPhAbsRawHitFEC.h,v 1.7 1999/03/12 16:03:17 olszewsk Exp $
// 
/////////////////////////////////////////////////////////////////////


// Root classes
#include "TObjArray.h"

// Phobos classes
#include "TPhDataObject.h"
#include "TPhString.h"

// Definitions regarding raw buffers
enum ERunMode {
  kSetupFEC    = 0x0,   // FEC Setup Mode
  kCalibration = 0x1,   // Calibration Mode
  kTestRun     = 0x2,   // Test Run (bit patterns etc.)
  kRun         = 0x3    // Normal Run
};

enum EDescriptionTrailer {
  kTrailerLength = 20,  // The length of the trailer in 16-bit words

// This part describes where the various control words are positioned
// in the trailer
  kFECNumPos        = 16,  // The FEC number
  kRunModePos       = 16,  // Run mode
  kFIFOErrorPos     = 17,  // FIFO Error
  kShiftOutErrorPos = 18,  // Shift Out Error
  kTotalWordsPos    = 19   // Totalwords Position
};

typedef struct FECRawHit_t {
  UInt_t Channel;
  Int_t ADC;
} FECRawHit_t;

enum ERangeADC {
  kADCMax = 4095, 
  kADCMaxNegative = 512
};

class TPhAbsRawHitFEC : public TPhDataObject {

protected:
  void                 CalcTotalWords();
  Char_t*              Conv2Base(UChar_t Byte, Char_t *Str) const;

public:
  TPhAbsRawHitFEC();
  TPhAbsRawHitFEC(Int_t FECNum);
  virtual ~TPhAbsRawHitFEC();

  virtual UChar_t       GetMonitorVal(Int_t Index);
  virtual ERunMode      GetMode();
  virtual FECRawHit_t*  GetRawHitArray(Int_t string);
  virtual UShort_t      GetRawHitArraySize(Int_t String)
    { return fRawHitBufSize[String]; }
  virtual Int_t         GetError();

  virtual void          Print(Option_t *option = "");

  virtual Int_t         GetFECNum() const { return fFECNumber; }
  static  Char_t*       MakeFECName(Int_t FECNum, Char_t *String);

// Functions used for FAKE DATA generation
  virtual void          Add(Int_t String, FECRawHit_t *hit);
  virtual void          SetMonitorVal(Int_t Index, UChar_t Val);
  virtual void          SetMode(ERunMode Mode);
  virtual void          SetFIFOError(Int_t String);
  virtual void          SetShiftOutError(Int_t String);

// For debugging
  virtual void          Peek(Int_t indx)
  { printf("Trailer[%d] = %x\n", indx, fTrailer[indx]); }

protected:
  Int_t        fFECNumber;                  //The FEC number of this raw data container
  UShort_t     fTrailer[kTrailerLength];    // The FEC trailer
  FECRawHit_t* fRawHits[kNumStrings];       // One Buffer per FEC string
  UShort_t     fRawHitBufSize[kNumStrings]; // The size of the string buffer
  UShort_t     fTotalWords;                 // Total number of words from FEC

public:
  ClassDef(TPhAbsRawHitFEC,1) // Abstraction of the FEC Raw Hit buffer

};


#endif // __TPhAbsRawHitFEC__
