#ifndef _TPhDST_INCLUDED_
#define _TPhDST_INCLUDED_

#include "TObject.h"
#include "TFile.h"
#include "TH1.h"

#include "TPhEvent.h"
#include "TPhDetector.h"

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhDST                                                         //
//                                                                   //
//    PHOBOS service class to access data on the Phobos DST files    //
//                                                                   //
//    TPhDST allows for an easy access of TPhEvent and TPhDetector   //
//    objects written to files in a ROOT TFile format                //
//                                                                   //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)              //
//    Created : 1996                                                 //
//    Version : 1.0                                                  //
//    Changed : 01.05.1998 Andrzej Olszewski                         //
//              TPhDetector access added                             //
//                                                                   //
// $Id                                                               //
//                                                                   //
///////////////////////////////////////////////////////////////////////


class TPhDST : public TObject {
    
  public: 
    TPhDST();
    TPhDST(Char_t *FileName);
    virtual ~TPhDST();

    Bool_t        Open(Char_t *FileName);
    Bool_t        Close();
    void          Reset();
    TFile        *GetFile() { return fRootFile; };
    void          ls() 
      { if(fRootFile) fRootFile->ls(); }

    TPhEvent     *GetNextEvent(Int_t skip=0);
    TPhDetector  *GetDetector();
    TPhDetector  *GetDetector(Char_t *name);
    void          SetVerbose(Int_t flag) { fVerbose = flag; }
    Int_t         GetVerbose() { return fVerbose; }

    ClassDef(TPhDST,1)  // PHOBOS DST interface class    

  protected:
 
    Int_t       fNumEvent;    // Event counter for event selection
    Int_t       fVerbose;     //! Control debugging output
    TFile      *fRootFile;    // DST File with Example events
    TList      *fListOfKeys;  // List of keys in DST      
    TIter      *fNextKey;     //! Key to next event in DST
};

#endif

