#ifndef _TPhParManager_INCLUDED_
#define _TPhParManager_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhParManager                                                  //
//                                                                   //
//    Phobos class storing and managing event/run parameters         //
//                                                                   //
//    TPhParManager keeps a list of TPhParKey parameter objects      //
//    and provides user interface for reading their values.          //
//    The basic set of reading methods can be extended in inheriting //
//    classes.                                                       //
//                                                                   //
//    Author  : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl)     //
//    Created : 02/12/98                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
// $Id: TPhParManager.h,v 1.2 1998/12/15 22:57:54 decowski Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes
#include "TNamed.h"
#include "TList.h"

// Phat classes
#include "TPhParKey.h"

class TPhParManager : public TNamed {

public:

    TPhParManager();
    TPhParManager(Char_t *name, Char_t *title);

    virtual ~TPhParManager();

    TList     *GetList() { return fParameters;    }
    void       AddParameter(TPhParKey *par);
    TPhParKey *GetParameter(Int_t pos);

    Int_t     GetNPars();
    Int_t     GetParPos(Char_t *name, Char_t *source = NULL);
    Char_t    GetParType(Char_t *name, Char_t *source = NULL);
    Bool_t    GetParPosType(Int_t *pos, Char_t *type, Char_t *name, Char_t *source = NULL);
    Bool_t    GetParTextValue(Char_t *value, Char_t *name, Char_t *source = NULL);
    Bool_t    GetParNumValue(Float_t *value, Int_t *numval, Char_t *name, Char_t *source = NULL);
    Bool_t    GetParTextValue(Char_t *value, Int_t pos);
    Bool_t    GetParNumValue(Float_t *value, Int_t *numval, Int_t pos);

    // Run and event parameters can be stored here.
    // Entries may store parameter values, parameter definitions 
    // (run DEFR and event DEFE keywords), and comments (C keyword)
    // These entries can be listed using specialized method, and also by a keyword
    void      ls();
    void      PrintAll();
    void      PrintPars();                       // parameter values only
    void      PrintDefs();                       // DEFR and DEFE keywords
    void      PrintRunDefs();                    // DEFR keywords only
    void      PrintEvDefs();                     // DEFE keywords only
    void      PrintByKey(Char_t *keyword);       // "keyword" param. values only

private:

    TList      *fParameters;      //  list of TPhParKey parameter objects
    
public:

    ClassDef(TPhParManager,1)    // Run/Event parameter manager

};

#endif




