#ifndef _TPhRun_INCLUDED_
#define _TPhRun_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhRun                                                         //
//                                                                   //
//    Phobos event class                                             //
//                                                                   //
//    TPhRun containing information on the run type and features     //
//                                                                   //
//    Author  : Andrzej Olszewski (andrzej.olszewski@ifj.edu.pl      //
//    Created : 12/03/98                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
// $Id: TPhRun.h,v 1.2 1998/12/15 22:56:29 decowski Exp $
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes
#include "TDatime.h"

// Phat classes
#include "TPhEvent.h"
#include "TPhParManager.h"
#include "TPhNoteManager.h"

class TPhRun : public TPhDataObject {

public:

    TPhRun();
    TPhRun(Int_t run);

    virtual ~TPhRun();

    Int_t      GetNRun()          { return fNRun;          }
    Int_t      GetDate()          { return fDate;          }
    Int_t      GetTime()          { return fTime;          }
    Int_t      GetFEvent()        { return fFEvent;        }
    Int_t      GetLEvent()        { return fLEvent;        }

    void       SetNRun(Int_t NRun){ fNRun = NRun;          }

    void       AddEvent(TPhEvent *event);

    TPhParManager  *GetParManager()            { return fParMgr;   }
    void            AddParameter(TPhParKey *par);
    Int_t           GetNPars();
    TPhParKey      *GetParameter(Int_t pos);
    TPhNoteManager *GetNoteManager()           { return fNoteMgr;  }
    void            AddNotes(TPhNotes *not);
    Int_t           GetNNotes();
    TPhNotes       *GetNotes(Int_t pos);

    void Print();
    void PrintPars();
    void PrintNotes();
    void PrintParDefs();

private:

    Int_t       fNRun;            //  run number
    Int_t       fFEvent;          //  first event number
    Int_t       fLEvent;          //  last event number
    Int_t       fDate;            //  date when run was started
    Int_t       fTime;            //  time when run was started
    TPhParManager  *fParMgr;      //  run parameter manager 
    TPhNoteManager *fNoteMgr;     //  run notes manager

    TDatime    *fdatime;          //! day/time run time object

public:

    ClassDef(TPhRun,1)    // Run info storage class

};

#endif




