#include "TPhDataObject.h"
#include <iostream.h>

ClassImp(TPhDataObject)


////////////////////////////////////////////////////////////
//
// TPhDataObject is the base class for all PHOBOS classes 
// containing data, either as raw data from the DAQ or 
// calibrated data from the calibration procedures or the 
// results from the reconstruction or analysis processors.
//
// The main purpose of the TPhDataObject class is to provide
// a uniform interface for bookkeeping, information exchange 
// between procedural classes and persistent I/O
//
// The individual data objects referring to one event are 
// kept with the TPhEvent object, where they are stored in
// a hash table using a unique name for retrieval.
//
////////////////////////////////////////////////////////////

TPhDataObject::TPhDataObject()
{
// Default constructor. Does nothing.
// Don't use this constructor unless you have to and know
// what you are doing
// Use TPhDataObject(Char_t *name) instead.

   fCreationID = NULL;
   fIsPersistent = kTRUE;
}

TPhDataObject::TPhDataObject(Char_t *Name, Char_t *Title)
{
// Constructor. Create the data container setting the name
// (and title, if supplied)


   SetName(Name);
   if(Title)
       SetTitle(Title);
   else
       SetTitle(Name);

   fCreationID = NULL;
   fIsPersistent = kTRUE;
}

TPhDataObject::~TPhDataObject()
{
// Destructor. Delete TPhDataObject and all the data objects
// currently owned by TPhDataObject

   delete fCreationID;
}

void TPhDataObject::SetCreationID(TObject *Creator)
{
// Store the details on when, by which calling class and by
// which user the data object was added to the event.

   fCreationID = new TPhCreationID(Creator);
}

void TPhDataObject::ls(Option_t *option)
{
// List Object. option is still unused.

  IndentLevel();
  cout << IsA()->GetName() << " Name:" << GetName() << endl;
}

void TPhDataObject::Print(Option_t *option)
{
// List Object. 
// Use "number" as an option to indicate how many sublevels should be listed

  Int_t level = atoi(option);
  if(level>=0)   printf(">>%s<<  Name: %s\n", IsA()->GetName(), GetName());
  return;
}

