#include "TPhParManager.h"

ClassImp(TPhParManager)


/////////////////////////////////////////////////////////////////////
//                                                                 //
// TPhParManager is a PHOBOS class for storing and reading         //
// various parameters saved in TPhParKey objects                   //
//                                                                 //
// $Id: TPhParManager.cxx,v 1.4 1998/12/15 21:50:02 decowski Exp $
//                                                                 //
/////////////////////////////////////////////////////////////////////

TPhParManager::TPhParManager()
{
// Default onstructor. 
// Use TPhParManager(Int_t run, Int_t event) instead

  fParameters = NULL;
}

TPhParManager::TPhParManager(Char_t *name, Char_t *title)
{
// User constructor. 

  SetName(name);
  SetTitle(title);
  fParameters = NULL;
}

TPhParManager::~TPhParManager()
{
// Destructor. 

   SafeDelete(fParameters);
}

Int_t TPhParManager::GetNPars()
{
  // Get number of parameters in the list

  if(!fParameters) return 0;
  return fParameters->GetSize();
}

Bool_t TPhParManager::GetParPosType(Int_t *pos, Char_t *type, 
                                     Char_t *name, Char_t *source)
{
  // Find position of the specified parameter in the list
  // return FALSE if parameter not found
  Int_t currpos, listsize;
  TPhParKey *parkey;

  if(!name || !pos || !type) {
    Warning("GetParPos",
            "NULL pointers present in the call not allowed! \n");
    return kFALSE;
  }

  if(!fParameters) return kFALSE;
  listsize = fParameters->GetSize();
  if(source) {
    for(currpos=0;currpos<listsize;currpos++) {
      parkey = (TPhParKey *) fParameters->At(currpos);
      if(parkey->HasName(name) && parkey->IsFrom(source)) {
	*pos = currpos;
        *type = parkey->GetParValueType();
	return kTRUE;
      }
    }
  } else {
    for(currpos=0;currpos<listsize;currpos++) {
      parkey = (TPhParKey *) fParameters->At(currpos);
      if(parkey->HasName(name)) {
	*pos = currpos;
        *type = parkey->GetParValueType();
	return kTRUE;
      }
    }
  }
  return kFALSE;
} 

Int_t TPhParManager::GetParPos(Char_t *name, Char_t *source)
{
  // Find position of the specified parameter in the list
  // return unreasonable value (-1) if parameter not found
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!name) return -1;

  if(!fParameters) return -1;
  listsize = fParameters->GetSize();
  if(source) {
    for(pos=0;pos<listsize;pos++) {
      parkey = (TPhParKey *) fParameters->At(pos);
      if(parkey->HasName(name) && parkey->IsFrom(source)) return pos;
    }
  } else {
    for(pos=0;pos<listsize;pos++) {
      parkey = (TPhParKey *) fParameters->At(pos);
      if(parkey->HasName(name)) return pos;
    }
  }
  return -1;
} 

Char_t TPhParManager::GetParType(Char_t *name, Char_t *source)
{
  // Get parameter type (if set)
  // Returns -1 if parameter not found
  Int_t pos;

  if( (pos = GetParPos(name, source)) > -1) 
    return ((TPhParKey *) fParameters->At(pos))->GetParValueType();
    
  return -1;
}

Bool_t TPhParManager::GetParTextValue(Char_t *value, Char_t *name, 
				      Char_t *source)
{
  // Get parameter value in the text form
  // Use GetParText(Int_t pos) method for faster access
  // declare "value" buffer before calling this function
  Int_t pos;

  if((pos = GetParPos(name, source)) > -1) 
    return ((TPhParKey *) fParameters->At(pos))->GetParTextValue(value);
  return kFALSE;
}

Bool_t TPhParManager::GetParNumValue(Float_t *value, Int_t *numval, 
                                     Char_t *name, Char_t *source)
{
  // Get parameter value as a float
  // Use GetParNum(Int_t pos) method for faster access
  Int_t pos;
  TPhParKey *parkey = NULL;

  if((pos = GetParPos(name, source)) > -1) 
    return ((TPhParKey *) fParameters->At(pos))->GetParNumValue(value,numval);
  return kFALSE;
}

Bool_t TPhParManager::GetParTextValue(Char_t *value, Int_t pos)
{
  // Get parameter value in the text form
  // First find parameter position with GetParPos()
  // declare "value" buffer before calling this function

  if(!fParameters) return kFALSE;

  if((pos>fParameters->GetSize()-1) || (pos<0)) {
    Warning("GetParText","Position %d out of range\n",pos);
    return kFALSE;
  }
  return ((TPhParKey *) fParameters->At(pos))->GetParTextValue(value);
}

Bool_t TPhParManager::GetParNumValue(Float_t *value, Int_t *numval, Int_t pos)
{
  // Get parameter value as a float
  // First find parameter position with GetParPos()

  if(!fParameters) return kFALSE;

  if((pos>fParameters->GetSize()-1) || (pos<0)) {
    Warning("GetParNum","Position %d out of range\n",pos);
    return kFALSE;
  }
  return ((TPhParKey *) fParameters->At(pos))->GetParNumValue(value, numval);
}


void TPhParManager::AddParameter(TPhParKey *par)
{
  // Add TPhParKey object to the list

  if(!fParameters) fParameters = new TList();
  fParameters->Add(par);
}

TPhParKey *TPhParManager::GetParameter(Int_t pos)
{
  // Get TPhParKey parameter object 

  if(!fParameters) return NULL;
  if((pos>fParameters->GetSize()-1) || (pos<0)) {
    Warning("GetParameter","Position %d out of range\n",pos);
    return NULL;
  }

  return (TPhParKey *) fParameters->At(pos);
}

void TPhParManager::ls()
{
  // Print all parameters, their definitions and comments

  if(!fParameters) return;

  fParameters->ls();
  return;
}

void TPhParManager::PrintAll()
{
  // Print all parameters, their definitions and comments
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName("DEFR") || parkey->HasName("DEFE")) 
         parkey->PrintNoKey();
    else parkey->Print();
  }
  return;
}
  
void TPhParManager::PrintPars()
{
  // Print values of all parameters
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName("DEFR") || parkey->HasName("DEFE")) 
         continue;
    else parkey->Print();
  }
  return;
}
  
void TPhParManager::PrintDefs()
{
  // Print all parameters, their definitions and comments
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName("DEFR") || parkey->HasName("DEFE")) parkey->PrintNoKey();
  }
  return;
}

void TPhParManager::PrintRunDefs()
{
  // Print all parameters, their definitions and comments
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName("DEFR")) parkey->PrintNoKey();
  }
  return;
}

void TPhParManager::PrintEvDefs()
{
  // Print all parameters, their definitions and comments
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName("DEFE")) parkey->PrintNoKey();
  }
  return;
}

void TPhParManager::PrintByKey(Char_t *keyword)
{
  // Print all parameters, their definitions and comments
  Int_t pos, listsize;
  TPhParKey *parkey;

  if(!fParameters) return;

  listsize = fParameters->GetSize();
  for(pos=0;pos<listsize;pos++) {
    parkey = (TPhParKey *) fParameters->At(pos);
    if(parkey->HasName(keyword)) parkey->PrintNoKey();
  }
  return;
}

