#include "TPhRun.h"

ClassImp(TPhRun)


//////////////////////////////////////////////////////////////////
//                                                              //
// TPhRun is a PHOBOS data object storing information           //
// on the run features.                                         //
//                                                              //
// $Id: TPhRun.cxx,v 1.2 1998/12/15 21:50:59 decowski Exp $
//                                                              //
//////////////////////////////////////////////////////////////////

TPhRun::TPhRun()
{
// Don't use this constructor unless you have to and know
// Use TPhRun(Int_t run, Int_t firsteventno) instead

    fNRun   = 0;
    fFEvent = -1;        
    fLEvent = -1;        
    fDate   = 0;
    fTime   = 0;
    fParMgr = NULL;     
    fNoteMgr= NULL;
}

TPhRun::TPhRun(Int_t run)
{
// Constructor for the user.

    fNRun   = run;
    fFEvent = -1;   
    fLEvent = -1;        
    fdatime = new TDatime();
    fDate   = fdatime->GetDate();
    fTime   = fdatime->GetTime();
    fParMgr = NULL;     
    fNoteMgr= NULL;

    Char_t txt[100];
    sprintf(txt,"RUN_%d",run);
    SetName(txt);
}

TPhRun::~TPhRun()
{
// Destructor. 

   SafeDelete(fParMgr);
   SafeDelete(fNoteMgr);
}

void TPhRun::AddEvent(TPhEvent *event)
{
  // Add event names to the run list, 
  // Set the first and last event number for this run object

  Int_t evnum = event->GetNEvent();
  fLEvent = evnum;
  if(fFEvent<0) fFEvent = evnum;
  if(fdatime) {
    fdatime->Set();
    event->GetEventHeader()->SetDate(fdatime->GetDate());
    event->GetEventHeader()->SetTime(fdatime->GetTime());
  }
  return;
}

Int_t TPhRun::GetNPars()
{
  // Get number of parameters in the list

  if(!fParMgr) return 0;
  return fParMgr->GetNPars();
}

Int_t TPhRun::GetNNotes()
{
  // Get number of parameters in the list

  if(!fNoteMgr) return 0;
  return fNoteMgr->GetNNotes();
}

void TPhRun::AddParameter(TPhParKey *par)
{
  // Add TPhParKey object to the list
  Char_t name[20];

  if(!fParMgr) {
    sprintf(name,"PARS_RUN_%d",fNRun);
    fParMgr = new TPhParManager(name,"Run parameters");
  }
  fParMgr->AddParameter(par);
}

void TPhRun::AddNotes(TPhNotes *not)
{
  // Add TPhNotes object to the list
  Char_t name[20];

  if(!fNoteMgr) {
    sprintf(name,"NOTES_RUN_%d",fNRun);
    fNoteMgr = new TPhNoteManager(name,"Run notes");
  }
  fNoteMgr->AddNotes(not);
}

TPhParKey *TPhRun::GetParameter(Int_t pos)
{
  // Get TPhParKey parameter object at pos position

  if(!fParMgr) return NULL;
  return fParMgr->GetParameter(pos);
}

TPhNotes *TPhRun::GetNotes(Int_t pos)
{
  // Get TPhNotes notes object at pos position

  if(!fNoteMgr) return NULL;
  return fNoteMgr->GetNotes(pos);
}

void TPhRun::Print()
{
  // Print run header

   cout << IsA()->GetName() << " Run: " << GetNRun() << " First event: " << GetFEvent() 
        << " Last event " << GetLEvent() << " Date: " << GetDate() << " " << GetTime() 
        << endl;
   cout << "Parameters:" << endl; 
   if(fParMgr) fParMgr->PrintAll();
   cout << "Notes:" << endl; 
   if(fNoteMgr) fNoteMgr->Print();
   return;
}

void TPhRun::PrintPars()
{
  // Print all parameters

   cout << "Parameters:" << endl; 
   if(fParMgr) fParMgr->PrintPars();
   return;
}

void TPhRun::PrintNotes()
{
  // Print all notes

   cout << "Notes:" << endl; 
   if(fNoteMgr) fNoteMgr->Print();
   return;
}

void TPhRun::PrintParDefs()
{
  // Print parameter definitions

   cout << "Parameters:" << endl; 
   if(fParMgr) fParMgr->PrintDefs();
   return;
}
