#ifndef _TPhBoxSensor_INCLUDED_
#define _TPhBoxSensor_INCLUDED_

////////////////////////////////////////////////////////////////////////////////
// 
//  TPhBoxSensor 
// 
// TPhBoxSensor is the base class for all non-Si Phobos BOX shaped sensors 
//
// Created 971219 by P. Decowski
//
// Changed 15.07.98  A. Olszewski
//                   Use rotation matrices to calculate sensor outer points 
//
// $Id
//
////////////////////////////////////////////////////////////////////////////////

// Root classes
#include "TBRIK.h"
#include "TRotMatrix.h"

// Phobos classes
#include "TPhSensor.h"

class TPhBoxSensor : public TPhSensor {

public:

  TPhBoxSensor();
  TPhBoxSensor(const Char_t* Name, Int_t Label, Int_t Layer, 
	       Double_t X0, Double_t X1,
	       Double_t Y0, Double_t Y1, Double_t Z0, Double_t Z1);

  virtual ~TPhBoxSensor();

  virtual void       Paint(Option_t *option) { TNode::Paint(option); }
  virtual void       CalcAlpha();
  virtual void       Print();   // *MENU*

  virtual Double_t     GetX0()          { return fX0;         }
  virtual Double_t     GetX1()          { return fX1;         }
  virtual Double_t     GetY0()          { return fY0;         }
  virtual Double_t     GetY1()          { return fY1;         }
  virtual Double_t     GetZ0()          { return fZ0;         }
  virtual Double_t     GetZ1()          { return fZ1;         }

  virtual Float_t      GetDx()          { return fDz;         }
  virtual Float_t      GetDy()          { return fDy;         }
  virtual Float_t      GetDz()          { return fDx;         }
  
  virtual Double_t     GetAlpha()       { return fAlpha;      }

  virtual void       SetX0(Double_t x)        { fX0    = x;       }
  virtual void       SetX1(Double_t x)        { fX1    = x;       }
  virtual void       SetY0(Double_t x)        { fY0    = x;       }
  virtual void       SetY1(Double_t x)        { fY1    = x;       }
  virtual void       SetZ0(Double_t x)        { fZ0    = x;       }
  virtual void       SetZ1(Double_t x)        { fZ1    = x;       }
  virtual void       SetAlpha(Double_t x)     { fAlpha     = x;   }

  void SetDim(Float_t dx, Float_t dy, Float_t dz);
  void CalcBoxPoints();

protected:
  Double_t      fX0;            //  Sensor position X0
  Double_t      fX1;            //  Sensor position X1
  Double_t      fY0;            //  Sensor position Y0 
  Double_t      fY1;            //  Sensor position Y1
  Double_t      fZ0;            //  Sensor position Z0
  Double_t      fZ1;            //  Sensor position Z1
  Double_t      fAlpha;         //  Sensor angle with respect to beam axis (in rad)

  Float_t       fDx;            // Sensor half-length
  Float_t       fDy;            // Sensor half-length
  Float_t       fDz;            // Sensor half-length

public:
  ClassDef(TPhBoxSensor,1)      // Base class for BOX shaped sensors other than Si.

};

#endif /* _TPhBoxSensor_INCLUDED_ */
