#ifndef _TPhSpectSensor_INCLUDED_
#define _TPhSpectSensor_INCLUDED_

/////////////////////////////////////////////////////////////////////////////////////
//                                                                                 //
//    TPhSpectSensor                                                               //
//                                                                                 //
//    Phobos Spectrometer Sensor                                                   //
//                                                                                 //
//    TPhSpectSensor manages access to all the information and                     //
//    functionality for reconstructing events for the spectrometer                 //
//    in the Phobos environment                                                    //
//                                                                                 //
//    Author  : Gunther Roland (rolandg@vxcern.cern.ch)                            //
//    Created : 01/12/95                                                           //
//    Version : 2.0                                                                //
//    Changed : 26/5/97          Add ChLabel (A.O.)                                //
//              30.09.97         Add Print() (A.O.)                                //
//              15 Oct. 97       Added GetCoords,                                  //
//                               MakeHitArrays (mpd)                               //
//                               MakeRawHitBuffers (A.O.)                          //
//    Changed : 20 Dec. 97       Changed TPhSensor to TPhSpectSensor               //
//                               now inherits from TPhBoxSensor and TPhSiSensor    // 
//                                                                                 //
/////////////////////////////////////////////////////////////////////////////////////

// Root Classes

// Phobos Classes
#include "TPhBoxSiSensor.h"

class TPhHit;
class TPhMCHit;

class TPhSpectSensor : public TPhBoxSiSensor {

public:
  TPhSpectSensor();
    
  virtual ~TPhSpectSensor();

  Double_t    DistToSensor(Double_t *x);
  virtual Double_t     GetSensorHeight(){ return GetPadWidthY()*GetRows(); }

  void       Print();
  void       Draw(Option_t *option="");
  void       CalcDistVar();
  void       MoveHitToSensor(TPhMCHit *hit); 
  void       DisplayDialog(); // *MENU*

private:
  Double_t      fA;             //! for DistToSensor()
  Double_t      fB;             //! for DistToSensor()
  Double_t      fC;             //! for DistToSensor()
  Double_t      fD;             //! for DistToSensor()
  Double_t      fLen2;          //! sensor length squared, for DistToSensor()

public:

  ClassDef(TPhSpectSensor,1)  // Phobos Spectrometer sensor element class

};

#endif


