#include "TMath.h"
#include "TPhChannelCalTB98.h"

ClassImp(TPhChannelCalTB98)

/////////////////////////////////////////////////////////////////////////////
//                                                                         //
// Methods to calibrate ADC signals in each separate channel.              //
// Test Beam 98 specific calibration                                       //
//                                                                         //
// $Id: TPhChannelCalTB98.cxx,v 1.1 1999/02/24 22:48:06 olszewsk Exp $
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

TPhChannelCalTB98::TPhChannelCalTB98() : TPhChannelCalAbs() 
{
  // Default constructor
}  

TPhChannelCalTB98::TPhChannelCalTB98(Int_t parno, Int_t parpointsno, Float_t boardlinearity,
				     Float_t *pars, Float_t *calpoints, 
				     Float_t *parerrors, Float_t *relerrors) :
                   TPhChannelCalAbs(parno, parpointsno, boardlinearity, pars, calpoints, 
				    parerrors, relerrors) 
{
  // Normal constructor
  if(parno != 4) {
    Error("TPhChannelCalTB98","Wrong number (%d) of calibration params\n", parno); 
    fParNo = 0;                    
    fParPointsNo = 0;
    fCalLinearity = 0;
    fPars.Reset();
    fParErrors.Reset(); 
    fCalPoints.Reset();
    fRelErrors.Reset();
  }
}  

TPhChannelCalTB98::~TPhChannelCalTB98()
{
  // Default destructor
}  
  
Bool_t TPhChannelCalTB98::Calibrate(Float_t &FloatADC)
{
  // Calibrate (change) ADC value according to the given formula and 
  // current parameters

//990303 mpd&hp: CHANGED ADC Short_t into Float_t!!!
  Double_t a,b,c,delta,r,r1,r2;
  Char_t sign = 1;
  
  if(!FloatADC) return kTRUE;

  // Convert negative entries to positive for calibration
  // and assign previous sign before leaving
  if(FloatADC<0) sign = -1;
  FloatADC *= sign;

  if( FloatADC >= 
      (fPars[0] + fPars[1]*fPars[3] + fPars[2]*fPars[3]*fPars[3]) ) {
    FloatADC = (Short_t) fPars[3];
    return kTRUE;
  }

  // find solution for the 2nd order poly
  a = fPars[2]; 
  b = fPars[1]; 
  c = fPars[0] - FloatADC;

  // 990316 HP check for model: use linear or quadratic
  if (a==0) {
  // use linear
    r= -c/b;
  } else { 
  //use quadratic
    delta = b*b - 4.0*a*c;
    if(delta<0) {
      FloatADC = 0;
      Warning("Calibrate","bad parameters, Delta<0, ADC set to 0.\n");
      return kFALSE;
    }
    r1 = (-b + TMath::Sqrt(delta))/(2*a);
    r2 = (-b - TMath::Sqrt(delta))/(2*a);
    if(r1>=0 && r1<4100) r = r1;
    else 
      if(r2>=0 && r2<4100) r = r2;
    else {
       FloatADC = 0;
       Warning("Calibrate","signal out of range, set to 0.\n");
       return kFALSE;
    }
  }
  FloatADC = r;

  // transform DAC setting to charge (fC x 10 - to keep accuracy)
  // 2.0pF test capacitor, 21.0 voltage divider on input of preamplifier
  // fCalLinearity - calibration board linearity, 10 - scaling factor
  //990303 HP: use floatADC with 1 FloatADC=0.01fC
  //990318 HP use TEMPORARILY a uniform cal board gain of measured 0.9707
  fCalLinearity = 0.9707;
  FloatADC = (FloatADC * 2.0 * fCalLinearity * 100.0 / 21.0); 
  FloatADC *= sign;
  
  return kTRUE;
}


