
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "TPhGufldMagnet.h"

ClassImp(TPhGufldMagnet)

//////////////////////////////////////////////////////////////////////////////////////////
//
// TPhGufldMagnet is another way of getting to the Phobos Magnet Simulated field map
//
//////////////////////////////////////////////////////////////////////////////////////////

TPhGufldMagnet::TPhGufldMagnet()
{
// Constructor
// PHOBOS Magnet class. Provides access to the magnetic field object.

   bfld = 0;
   magx = 0;
   magz = 0;  
   moff[0] = 0;  
   moff[1] = 0;  
   moff[2] = 0;  
   rfld = 0;    
   magt = 0;
   magv = 0;     

   gufldinit = 0;
   gufldread = 0;
   isaou = 0;
}


TPhGufldMagnet::TPhGufldMagnet(Char_t *filename, Int_t *type, Double_t *bfldmax, 
         Double_t *zmagx, Double_t *zmagz, Double_t *zmoff, Double_t *zrfld, 
         Double_t *version)
{
// Constructor
// Like TPhGufldMagnet(), but also sets field parameters
// and reads/constructs the magnetic field map.

   InitFieldMap(filename, type, bfldmax, zmagx, zmagz, zmoff, zrfld, version);

   if( !ReadFieldMap(filename, type, version) ) {
     printf("Field map not initialized!");
     abort();
   }

   isaou = 0;
}


TPhGufldMagnet::~TPhGufldMagnet()
{
// Default destructor. Don't forget to release space
// for field map if it has been allocated. 
}
    
FILE *fusrmap;
FILE *fusraou;


Bool_t TPhGufldMagnet::InitFieldMap(Char_t *filename, Int_t *type, Double_t *bfldmax, 
         Double_t *zmagx, Double_t *zmagz, Double_t *zmoff, Double_t *zrfld, 
         Double_t *version)
{
// initialization of the magnetic field parameters.
// for type = 1: zmagx, zmagz, zrfld and bfldmax must be set,
//               zmoff may define additional offset of the center
// for type = 2: zmagx, zmagz, zrfld have no meaning,
//               zmoff and bfldmax may redefine the field
// for type = 3: zrfld has no meaning,
//               zmagx, zmagz, zmoff and bfldmax may redefine the field
//               they must be set if not defined inside the map

  if(gufldinit != 0)
  {   printf(" InitFieldMap: field parameters are already initialized. \n");
      return kFALSE;
  }

  strcpy(FileName,filename);
  magt = *type;
  bfld = *bfldmax;
  magx = *zmagx;
  magz = *zmagz;
  moff[0] = zmoff[0];
  moff[1] = zmoff[1];
  moff[2] = zmoff[2];
  rfld = *zrfld;
  magv = *version;

  fFieldType = magt;              //  parameters in the TPhMagnet class
  fFieldVersion = (Float_t) magv;
  fFieldMax = (Float_t) bfld;        
  fFieldCenter[0] = (Float_t) magx;   
  fFieldCenter[1] = 0;   
  fFieldCenter[2] = (Float_t) magz;   
  fFieldOffset[0] = (Float_t) moff[0];   
  fFieldOffset[1] = (Float_t) moff[1];  
  fFieldOffset[2] = (Float_t) moff[2];  

  gufldinit = 1;
  return kTRUE;
}


Bool_t TPhGufldMagnet::ReadFieldMap(Char_t *filename, Int_t *type, Float_t *version)
{
// read type= 2 or 3, or calculate type=1 magnetic field map
// but first InitFieldMap() must be used to initialize field parameters

  Double_t x[3], b[3];

  if(gufldinit == 0)
  {   printf(" ReadFieldMap: the field parameters must be first initialized\n");
      return kFALSE;
  }
  if((Int_t)magt != *type)
  {   printf(" ReadFieldMap: can not read type=%2d field \n", *type);
      printf(" since gufld is initialized with type=%2d parameters\n", magt);
      return kFALSE;
  }

  *version = 0;
  if(magt > 1) 
  {  
      x[0] = x[1] = x[2] = 0.0;
      gufld(x, b);
  }
 *version = magv;
  strcpy(filename, MapFileName);
  return kTRUE;;
}

Bool_t TPhGufldMagnet::ReadFieldMap(Char_t *filename, Int_t *type, Double_t *version)
{
// read type= 2 or 3, or calculate type=1 magnetic field map
// but first InitFieldMap() must be used to initialize field parameters

  Double_t x[3], b[3];

  if(gufldinit == 0)
  {   printf(" ReadFieldMap: the field parameters must be first initialized\n");
      return kFALSE;
  }
  if((Int_t)magt != *type)
  {   printf(" ReadFieldMap: can not read type=%2d field \n", *type);
      printf(" since gufld is initialized with type=%2d parameters\n", magt);
      return kFALSE;
  }

  *version = 0;
  if(magt > 1)
  {
      x[0] = x[1] = x[2] = 0.0;
      gufld(x, b);
  }
 *version = magv;
  strcpy(filename, MapFileName);
  return kTRUE;;
}

void TPhGufldMagnet::GetFieldParameters(Char_t *filename, Int_t *type, Double_t *bfldmax, 
            Double_t *zmagx, Double_t *zmagz, Double_t *zmoff, Double_t *zrfld, 
            Double_t *version)
{
   strcpy(filename,FileName);
  *type = magt;
  *bfldmax = bfld;
  *zmagx = magx;
  *zmagz = magz;
   zmoff[0] = moff[0];
   zmoff[1] = moff[1];
   zmoff[2] = moff[2];
  *zrfld = rfld;
  *version = magv;
  return; 
}

Bool_t TPhGufldMagnet::GetField(Float_t* Coord, Float_t* B)
{
//
  Double_t vect[3],b[3];
  Bool_t   gotit;

//  printf("Warning: Double to Float conversion - precision may be lost!\n");
  vect[0] = Coord[0];
  vect[1] = Coord[1];
  vect[2] = Coord[2];

  gotit = GetField(vect, b);

  B[0] = b[0];
  B[1] = b[1];
  B[2] = b[2];

  return gotit;
}


Bool_t  TPhGufldMagnet::GetField(Double_t *vect, Double_t *b)
{
  Bool_t   gotit;

  gotit = gufld(vect, b);

//ao  factor 0.1 applied to convert kGauss -> Tesla
  b[0] *= 0.1;
  b[1] *= 0.1;
  b[2] *= 0.1;

  return gotit;
}


Bool_t TPhGufldMagnet::gufld(Double_t *vect, Double_t *b)
{

/*DocumentationBlock                                                          */
/************************************************************************     */
/*                                                                            */
/* Description:                                                               */
/*  The Phobos-specific version of GUFLD which returns the magnetic           */
/*  field B at any point VECT.  It has 3 modes, controlled by variables       */
/*  BFLD, RFLD, MAGT, MAGX, MAGZ and MOFF, in common RHSYS, which             */
/*  can be specified on input cards:                                          */
/*                                                                            */
/*  BFLD is the field strength in kGauss.                                     */
/*  If you do not want to redefine the magnetic field values (strongly *      */
/*  recomended for MAGT=3) BFLD=0.0 should be used.                    *      */
/*  if it is different than 0.0 all field values will be multiplied by *      */
/*  a factor equal:                                                    *      */
/*     BFLD / Bcenter                                                  *      */
/*  where Bcenter is the field strength at (MAGX, 0, MAGZ)             *      */
/*                                                                            */
/*  The position of the map can always be changed by the MOFF parameter       */
/*  the point VECT is always offset and  a point defined as:                  */
/*       VECTOFF = VECT - MOFF  (vector)                                      */
/*  is used as a position in the map.                                         */
/*                                                                            */
/*  MAGT = 0: No magnetic field mode                                          */
/*                                                                            */
/*  MAGT = 1: Diagnostic mode                                                 */
/*   The field is a simple dipole field: Bx = Bz = 0,                         */
/*   By = BFLD if r < RFLD, By = 0 otherwise,                                 */
/*   where rx = x - MAGX and rz = z - MAGZ.                                   */
/*                                                                     *      */
/*  MAGT = 2: ASCII 3D-field map mode                                         */
/*   At the first call, a map of the magnetic field components is read *      */
/*   in.  First the values of the coordinates at grid points are read  *      */
/*   Then there are the lines with the indexes coresponding to the     *      */
/*   position of the point and the field vector (Bx,By,Bz).            *      */
/*   The field values not exactly at the known point are linearly      *      */
/*   interpolated using FINT for each of components Bx, By, Bz.  The   *      */
/*   coordinates are transformed from the GEANT mother system to the   *      */
/*   fieldmap system using information in COMMON.SYS.                  *      */
/*                                                                     *      */
/*  MAGT=3 The magnetic field is a superposition of two fields from    *      */
/*   magnets. The field from each magnet is calculated in the case of  *      */
/*   cylindrical symetry. Thus the magnetic field map is two dimensio- *      */
/*   nal only. The map is read from an ASCII file with a similar format*      */
/*   as for MAGT=2 (the difference is in the definition of points:     *      */
/*   (x,y,z) for MAGT=2 and (r,v) for MAGT=3                           *      */
/*   The field values from two-dimensional map are interpolated if     *      */
/*   the point lies inside the boundaries. In the case of points       *      */
/*   outside the boundaries the value from the closest point exactly   *      */
/*   at the boundary is taken and multiplied by the factor lineary     *      */
/*   decreasing with the distance from the boundary. It protects from  *      */
/*   dangers of extrapolation of the field and avoids the sharp change *      */
/*   of the field before it is set to zero                             *      */
/*                                                                            */
/*  Arguments:                                                                */
/*  *VECT: x = VECT[0]... = point at which B is to be evaluated   (input)     */
/*  B*     B[0]...B[2] = field at point VECT                     (output)     */
/*                                                                            */
/*  Called by:                                                                */
/*  Geant tracking subroutines GHELIX or GRKUTA, called by GUSWIM             */
/*                                                                            */
/*  Calls:                                                                    */
/*  Cern routine FINT to do linear interpolation                              */
/*  STOPER - in case of error in the map                                      */
/*                                                                            */
/*  Implicit inputs, outputs, side effects:                                   */
/*  The field maps are read on unit Iusrmap, assumed to have been             */
/*    opened in UGINIT; later, open will be done in GUFLD.                    */
/*  Initialization on local switch                                            */
/*                                                                            */
/*  History:                                                                  */
/*  Date       Name        Description                                        */
/*  ----       ----        -----------                                        */
/*  10-Oct-96  K.W.        C version                                          */
/*  21-Jul-96  R. V.       Removed unused zc, j, yneg                         */
/*  12-Jul-96  K.Wozniak   magnetic field center (MAGX,MAGZ) separated from   */
/*                         the field offset (MOFF[3])                         */
/*  06-Jul-96  K.Wozniak   Removed ZEBRA map, added output of magnet          */
/*                         information from the map to Ascii OUput file       */
/*                         general reorganization of the code                 */
/*  03-Jul-96  A. Trzupek  Prepared map for the CDR magnet (MAGT=2)           */
/*                         which comprises two maps precise one for           */
/*                         spectrometer and less precise but larger           */
/*                         in x and z for TOF. Changed gufld to allow for     */
/*                         reading them.                                      */
/*  10-Jul-95  K. Wozniak  Redefinition of the MAGT=2 mode                    */
/*  02-Mar-93  K. Wozniak  Added facility for cylindrical field map           */
/*  09-Dec-92  C. Conner   Added facility for pre-sorted Zebra field map      */
/*  17-Jun-92  C. Parsons  Extracted from GPH with unspecified changes        */
/*  22-Mar-92  Don Mcleod  Initial version with Ascii field map, in GPH       */
/************************************************************************     */
/*EndDocumentationBlock                                                       */
 
 
 
/*     Local variables                                                        */
 
  Char_t cbuf[80];
  Char_t *cbufpt;

  Double_t x[3];
  Double_t vectoff[3];
  Int_t naa[3];
 
  Int_t i,k, ix,iy,iz;

  Double_t b1[3], b2[3], br, bv, r1, r2, sina, cosa, magnew, bmax3;
  Int_t ir, iv;
  Int_t nx, ny, nz;
  Int_t nlines, ifile, imap;


/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++        */
/*    code                                                                    */

 if(gufldread != 0)   goto lab200;

    gufldread = 1;

    if(magt == 0 | magt == 1)  
    {
      if(isaou > 0)  
        fprintf(fusraou, "*  magnetic field set in the control file\n");
      if(magt == 0)  
      {
        printf(" gufld - no magnetic field\n");
        if(bfld != 0.0)  
         printf(" gufld - bfld = %g  set in the control file was ignored\n", 
                               bfld);
        printf(" gufld - bfld = 0.0\n");
      }
      else 
      {
         if(magt == 1)  
         {
            printf(" gufld - magnetic field constant in two cylinders\n");
            printf(" gufld - bfld = %g\n", bfld);
            if(fabs(bfld) < 1.0e-10)
             printf(" gufld - use  magt = 0  to obtain zero field! \n");
         }
      }
        goto lab199;
    }   /*       if(magt == 0 | magt == 1)    */

 
/*******************************************************                      */
/*   Read field map file in the first call             *                      */
/*******************************************************                      */
 
 
    if (magt >= 2)    /*  read parameters of the ma                 */
    {
/*                               !  from the magnetic field map file          */
       strcpy(MapFileName, FileName);
       fusrmap = fopen(MapFileName, "r");
       if(fusrmap == NULL)
       {
          printf(" ReadFieldMap: file %s can not ne opened\n", MapFileName);
          fclose(fusrmap);
          b[0] = b[1] = b[2] = 0.0;
          return kFALSE;
       }  
       ifile = 0;
       if(isaou > 0)  
         fprintf(fusraou, 
          "*  magnetic field defined by the magnetic field map file\n");
 
       nx = 0;   /*  initialization for any kind of map                     */
       ny = 0;   /*  zero denotes undefined field                           */
       nz = 0;
       nnr = nnv = 0;
       nlines = nmap = 0;
 
   lab10:  if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
       cbufpt = cbuf;
       nlines = nlines + 1;
        if(keyword(cbuf, "*", "*", 1) == 1)  goto lab10;
        if(keyword(cbuf, "magv", "MAGV", 4) == 4)
        {   /*  magnetic field version number    */
           if(sscanf(cbufpt+5, "%lg", &magv) != 1)  goto lab942;
           else   goto lab10;
        }  
        if(keyword(cbuf, "magt", "MAGT", 4) == 4)
        {   /*  redefinition of the field type   */
           if(sscanf(cbufpt+5, "%d", &magt) != 1)  goto lab942;
           if(magt < 0  |  magt > 3)  goto lab946;  /*  illegal value    */
           printf(" gufld - map type found in the file = %d\n", magt);
           goto lab10;
        }  
        if(keyword(cbuf, "magx", "MAGX", 4) == 4)
        {   /*  redefinition of magnet center    */
           if(sscanf(cbufpt+5, "%lg", &magnew) != 1)  goto lab942;
           if(magx == 0.0)   
           { 
              magx = magnew;   /*  MAGX changed only if it was zero        */
              printf(" gufld - magx = %g\n", magx);
           }
           else
           {
              printf(" gufld - ignored magx from the file: %g\n", magnew);
              printf(" gufld - used magx: %g\n", magx);
           }  
           goto lab10;
        }  
        if(keyword(cbuf, "magz", "MAGZ", 4) == 4)
        {   /*  redefinition of magnet center    */
           if(sscanf(cbufpt+5, "%lg", &magnew) != 1)  goto lab942;
           if(magz == 0.0)   
           { 
              magz = magnew;   /*  MAGZ changed only if it was zero       */
              printf(" gufld - magz = %g\n", magz);
           }
           else
           {
              printf(" gufld - ignored magz from the file: %g \n", magnew);
              printf(" gufld - used magz: %g\n", magz);
           }  
           goto lab10;
        }  
        if(keyword(cbuf, "rfld", "RFLD", 4) == 4)
        {   /*  redefinition of magnet radius    */
           if(sscanf(cbufpt+5, "%lg", &magnew) != 1)  goto lab942;
           if(rfld == 0.0)   
           { 
              rfld = magnew;   /*  RFLD changed only if it was zero      */
              printf(" gufld - rfld = %g\n", rfld);
           }
           else
           {
              printf(" gufld - ignored rfld from the file: %g\n", magnew);
              printf(" gufld - used rfld: %g\n", rfld);
           }  
           goto lab10;
        }  
        if(keyword(cbuf, "m ", "M ", 2) == 2)
        {   /*  information to be copied to AOU file   */
           if(isaou > 0)  
             fprintf(fusraou, "%-80s\n", cbuf);
           goto lab10;
        }  
 
/****************************************************************             */
/*   End of general information - check the type of the field   *             */
/****************************************************************             */
 
        if(magt == 0)  goto lab199;
        if(magt == 1)  goto lab199;
        if(magt == 2)  goto lab20;
        if(magt == 3)  goto lab111;
       goto lab945;

      }
      else
      {
 
         goto lab945;   /*  wrong type of the magnetic fiel                      */
 
      }     /*   MAGT >= 2                                                 */
 
 
/*******************************************************                      */
/*  ASCII map method: field values given in a box      *                      */
/*******************************************************                      */
 
 
/*********************************************************************        */
/*  Read the three-dimensional ASCII map (in a box)                  *        */
/*  Format:                                                          *        */
/*   - comments (starting with *) - description of the file contents *        */
/*   - 'XAXIS:'  nx      - definition of the number of grids in      *        */
/*                         x - direction                             *        */
/*   - nx     values of X from 0.0 to maximum (sorted) - free format *        */
/*   - comments (if any)                                             *        */
/*   - 'YAXIS:'  ny      - definition of the number of grids in      *        */
/*                         y - direction                             *        */
/*   - ny     values of Y from 0.0 to maximum (sorted) - free format *        */
/*   - comments (if any)                                             *        */
/*   - 'ZAXIS:'  nz      - definition of the number of grids in      *        */
/*                         z - direction                             *        */
/*   - nz     values of Z from 0.0 to maximum (sorted) - free format *        */
/*   - comments (if any)                                             *        */
/*   - 'FIELD:'  bmax  - declaration of the begining of field values *        */
/*                       bmax - maximal field for y=0                *        */
/*   - field values in the nodes of the grid, one node per line:     *        */
/*     ix  iy  iz   Bx  By  Bz      where:                           *        */
/*     ix, iy, iz   - definition of the node point: (ix),Y(iy),Z(iz) *        */
/*     Bx, By, Bz   - components of the field                        *        */
/*  IT IS ASSUMED, THAT THE FIELD IS DEFFINED FOR:                   *        */
/*        X >= 0,  Y >= 0,   Z in desired range                      *        */
/*********************************************************************        */
 
   lab20:
       nmap = nmap + 1;
       if(nmap >= NMAPMAX)  goto lab957;
       impos[nmap] = 0;
       igpos[nmap] = 0;
       if(nmap > 1)
       {
          impos[nmap] = impos[nmap-1] + nx * ny * nz;
          igpos[nmap] = igpos[nmap-1] + nx + ny + nz;
       }
       nx = ny = nz = 0;
       iphoind[nmap] = 1;
       goto lab22;

   lab21:  
       if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
       cbufpt = cbuf;
       nlines = nlines + 1;
   lab22:  
       if(keyword(cbuf, "*", "*", 1) == 1)  goto lab21;
       if(keyword(cbuf, "m ", "M ", 2) == 2)
       {   /*  information to be copied to AOU file   */
          if(isaou > 0)  fprintf(fusraou, "%-80s\n", cbuf);
          goto lab21;
       }  
       if(keyword(cbuf, "xaxis:", "XAXIS:", 6) == 6)
       { 
          if(sscanf(cbufpt+6, "%d", &nx) != 1)  goto lab942;
          if(nx < 2)   goto lab956;
          if(nx+ny+nz+igpos[nmap] > MAXGRID) goto lab954;
          for(i=0; i<nx; i++) 
            if(fscanf(fusrmap, "%lg", &a[igpos[nmap]+i]) != 1)   goto lab942;
          nlines = nlines + 1;
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
          goto lab21;
       }  
       if(keyword(cbuf, "yaxis:", "YAXIS:", 6) == 6)
       { 
          if(nx < 1)  goto lab951;
          if(sscanf(cbufpt+6, "%d", &ny) != 1)  goto lab942;
          if(ny < 2)   goto lab956;
          if(nx+ny+nz+igpos[nmap] > MAXGRID) goto lab954;
          for(i=0; i<ny; i++) 
            if(fscanf(fusrmap, "%lg", &a[igpos[nmap]+nx+i]) != 1)   goto lab942;
          nlines = nlines + 1;
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
          goto lab21;
       }  
       if(keyword(cbuf, "zaxis:", "ZAXIS:", 6) == 6)
       { 
          if(nx < 1)  goto lab951;
          if(ny < 1)  goto lab952;
          if(sscanf(cbufpt+6, "%d", &nz) != 1)  goto lab942;
          if(nz < 2)   goto lab956;
          if(nx+ny+nz+igpos[nmap] > MAXGRID) goto lab954;
          for(i=0; i<nz; i++) 
            if(fscanf(fusrmap, "%lg", &a[igpos[nmap]+nx+ny+i]) != 1)   goto lab942;
          nlines = nlines + 1;
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
          goto lab21;
       }  
       if(keyword(cbuf, "phoind:", "PHOMAG:", 7) == 7)
       { 
          if(sscanf(cbufpt+7, "%d", &iphoind[nmap]) != 1)  goto lab942;
          if(iphoind[nmap] != 0 && iphoind[nmap] != 1)  goto lab958;
          goto lab21;
       }  
       if(keyword(cbuf, "field:", "FIELD:", 6) == 6)
       { 
         if(nx < 1  |  ny < 1  |  nz < 1)  goto lab953;
         if(nx*ny*nz+impos[nmap] > MAXMAP)  goto lab955;
         if(sscanf(cbufpt+6, "%lg", &bmax2[nmap]) != 1)  goto lab942;
                                         /*  mominal maximum at y=0        */
         printf(" gufld - maximum abs(by) in y=0 plane: %g\n", bmax2[nmap]);
         if(bfld != 0.0 && iphoind[nmap] != 0)   
         { 
            printf(" gufld - rescaled value = %g\n", bfld);
         }
         else
         {
            printf(" gufld - no rescaling\n");
         }  
         for(i=impos[nmap]; i<MAXMAP; i++) bx[i] = by[i] = bz[i] = 0.0;
          nax[nmap] = nx;
          nay[nmap] = ny;
          naz[nmap] = nz;

  lab25:  if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab35;
          cbufpt = cbuf;
          nlines = nlines + 1;
          if((k=sscanf(cbufpt, "%d %d %d %lg %lg %lg", &ix,&iy,&iz, 
                                   &b1[0],&b1[1],&b1[2])) != 6)  goto lab31;
          if(ix < 1  |  iy < 1  |  iz < 1  |  
             ix > nx  |  iy > ny  |  iz > nz)  goto lab959;
          k = impos[nmap] + ix - 1 + nx * (iy-1 + ny*(iz-1));
          bx[k] = b1[0];
          by[k] = b1[1];
          bz[k] = b1[2];
	  if(ix==nx && iy==ny && iz==nz) goto lab30;
          goto lab25;
       }  
 
       goto lab942;  /*   ERROR IN THE MAP                                    */
 
  lab30:
      if(fgets(cbuf, 81, fusrmap) == NULL)  goto  lab35;
      nlines = nlines + 1;

  lab31:
      goto lab20;

  lab35:
      fclose(fusrmap);
      strcpy(MapFileName, FileName);
      addnumber(MapFileName, ifile+1);
      fusrmap = fopen(MapFileName, "r");
      if(fusrmap != NULL)
      {
         nlines = 0;
         ifile = ifile + 1;
         if(isaou > 0)  fprintf(fusraou, "*M Map file name (file %d): %s\n",
                                     ifile, MapFileName);
         goto lab30;
      }

//  lab99:  
      goto lab199;
 
 
/*********************************************************************        */
/*  Magnetic field is a superposition of two cylinrically symetrical *        */
/*  fields from two magnets                                          *        */
/*  2-dimensional ASCII map (IN cylindrical coordinates).            *        */
/*  Format:                                                          *        */
/*   - comments (starting with *) - description of the file contents *        */
/*   - 'RADIAL:'  nnr   - definition of the number of grids in       *        */
/*                        radial direction                           *        */
/*   - nnr values of R from 0.0 to maximum (sorted) - free format    *        */
/*   - comments (if any)                                             *        */
/*   - 'VERTICAL:' nnv  - definition of the number of grids in       *        */
/*                        vertical direction                         *        */
/*   - nvv values of V from 0.0 to maximum (sorted) - free format    *        */
/*   - comments (if any)                                             *        */
/*   - 'FIELD:'      - declaration of the begining of field values   *        */
/*   - field values in the nodes of the grid, one node per line:     *        */
/*     ir iv  Br  Bv          where:                                 *        */
/*     ir, iv - definition of the node point:  R(ir), V(iv)          *        */
/*     Br  - radial component of the field                           *        */
/*     Bv  - vertical component of the field                         *        */
/*********************************************************************        */
 
  lab110:  if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
       cbufpt = cbuf;
       nlines = nlines + 1;
  lab111:  
       if(keyword(cbuf, "*", "*", 1) == 1)  goto lab110;
       if(keyword(cbuf, "radial:", "RADIAL:", 7) == 7)
       { 
          if(sscanf(cbufpt+7, "%d", &nnr) != 1)  goto lab942;
          if(nnr < 2   |   nnr > NNRMAX)  goto lab961;
          for(i=0; i<nnr; i++) 
            {  if(fscanf(fusrmap, "%lg", &xtab[i]) != 1)  goto lab942;  }
          nlines = nlines + 1;
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
          goto lab110;
       }  
       if(keyword(cbuf, "vertical:", "VERTICAL:", 9) == 9)
       { 
          if(sscanf(cbufpt+9, "%d", &nnv) != 1)  goto lab942;
          if(nnv < 2   |   nnv > NNVMAX)  goto lab962;
          for(i=0; i<nnv; i++) 
            {  if(fscanf(fusrmap, "%lg", &xtab[nnr+i]) != 1)  goto lab942;  }
          nlines = nlines + 1;
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab941;
          goto lab110;
       }  
       if(keyword(cbuf, "field:", "FIELD:", 6) == 6)
       { 
         if(nnr < 2   |   nnv < 2)   goto lab963;
         for(i=0; i<nnr*nnv; i++)  brtab[i] = bvtab[i] = 0.0;
  lab112:    
          if(fgets(cbuf, 81, fusrmap) == NULL)  goto lab120;
          nlines = nlines + 1;
          if(sscanf(cbufpt, "%d %d %lg %lg", &ir,&iv, &br,&bv) != 4)  
                                                              goto lab942;
          if(ir < 1  |  iv < 1  |  ir > nnr  |  iv > nnv)   goto lab969;
          brtab[ir-1+nnr*(iv-1)] = br;
          bvtab[ir-1+nnr*(iv-1)] = bv;
          goto lab112;
       }  
       goto lab942;
 
  lab120:  
/**************************************************************************   */
/*   Check if the map is completly filled (there should be no nods with   *   */
/*   field equal zero)                                                    *   */
/**************************************************************************   */
       for(ir=0; ir<nnr; ir++)
       {
          for(iv=0; iv<nnv; iv++)
          {
             if(brtab[ir+nnr*iv] == 0.0   && bvtab[ir+nnr*iv] == 0.0)    
             { 
               printf(" gufld - zero field at: %g %g", xtab[ir], xtab[nnr+iv]);
             }  
          }
       }
       cyfld(0.0, 0.0, &b1[0], &b1[1]);
       cyfld(2.0*magx, 0.0, &br, &bv);
       b[0] = b1[0] - br;
       b[1] = b1[1] - bv;
       bmax3 = sqrt(b[0]*b[0] + b[1]*b[1]);
       printf("magt = 3  bmax = %g\n", bmax3);
       if(bfld != 0.0)   
       { 
         printf(" gufld - rescaled value = %g\n", bfld);
       }
       else
       {
         printf(" gufld - no rescaling\n");
       }  
 
 
/***********************************************************                  */
/*   Output of the field parameters to Ascii OUtput file   *                  */
/***********************************************************                  */
 
  lab199: 
      if(isaou > 0)   
      { 
         fprintf(fusraou, "m magv %g\n", magv);
         fprintf(fusraou, "m magt %d\n", magt);
         fprintf(fusraou, "m magx %g\n", magx);
         fprintf(fusraou, "m magz %g\n", magz);
         fprintf(fusraou, "m rfld %g\n", rfld);
         fprintf(fusraou, "m moff %g %g %g\n", moff[0], moff[1], moff[2]);
         fprintf(fusraou, "m bfld %g\n", bfld);
      }     /*  ISAOU >                                                  */
 


/***********************************************************                  */
/*   Calculation of the field values                       *                  */
/***********************************************************                  */
 
  lab200: 
      if(magt == 0)  goto lab990;   /*  zero field everywhere               */
 
      vectoff[0] = vect[0] - moff[0];
      vectoff[1] = vect[1] - moff[1];
      vectoff[2] = vect[2] - moff[2];
 
      if(magt == 1)  
      {       /*  perfect dipol                             */
        x[0] = (vectoff[2] - magz);
/* Bx,Bz always zero                                                          */
        b[0]=0;
        b[2]=0;
        if(vectoff[0] > 0)   
        {    /*positive side of fiel                 */
          x[1] = (vectoff[0] - magx);
          if( sqrt(x[0]*x[0]+x[1]*x[1]) < rfld ) 
          { 
            b[1]=bfld;
          }
          else
          {
            b[1]=0;
          }  
        }
        else
        {
          x[1] = (vectoff[0] + magx);
          if( sqrt(x[0]*x[0]+x[1]*x[1]) < rfld ) 
          { 
            b[1]=-bfld;
          }
          else
          {
            b[1]=0.0;
          }  
        }  
        return kTRUE;
      }        /*   MAGT == 1                                              */
 
 
      if (magt == 2)    
      {    /*   field from the 3-D map of any kind of magnet       */
        x[0] = fabs(vectoff[0]);
        x[1] = fabs(vectoff[1]);
        x[2] = vectoff[2];

        for(imap=1; imap <= nmap; imap++)
        {
          nx = nax[imap];
          ny = nay[imap];
          nz = naz[imap];
          if(iphoind[imap] == 1)
	  {
             if(x[0] > a[igpos[imap]+nx-1])  goto lab220;
             if(x[1] > a[igpos[imap]+nx+ny-1])  goto lab220;
             if(x[2] < a[igpos[imap]+nx+ny] |
                x[2] > a[igpos[imap]+nx+ny+nz-1])  goto lab220;
             goto lab221;
          }
          else
	  {
             if(vect[0] < a[igpos[imap]] | 
                vect[0] > a[igpos[imap]+nx-1])  goto lab220;
             if(vect[1] < a[igpos[imap]+nx+1] | 
                vect[1] > a[igpos[imap]+nx+ny-1])  goto lab220;
             if(vect[2] < a[igpos[imap]+nx+ny] | 
                vect[2] > a[igpos[imap]+nx+ny+nz-1])  goto lab220;
             goto lab221;
          }
          lab220: ;
        }     //  for(imap=1;  ...
        goto lab990;

      lab221:  
        naa[0] = nax[imap];
        naa[1] = nay[imap];
        naa[2] = naz[imap];
        if(iphoind[imap] == 0)
	{
          b[0] = fint(3, vect, naa, a, bx, igpos[imap], impos[imap]);
          b[1] = fint(3, vect, naa, a, by, igpos[imap], impos[imap]);
          b[2] = fint(3, vect, naa, a, bz, igpos[imap], impos[imap]);
        }  
        else  //  iphoind[imap] == 1
	{
           if(bfld != 0.0)   
           { 
             b[0] = fint(3, x, naa, a, bx, igpos[imap], impos[imap]) 
                         * bfld/bmax2[imap];
             b[1] = fint(3, x, naa, a, by, igpos[imap], impos[imap])
                         * bfld/bmax2[imap];
             b[2] = fint(3, x, naa, a, bz, igpos[imap], impos[imap])
                         * bfld/bmax2[imap];
           }
           else
           {
             b[0] = fint(3, x, naa, a, bx, igpos[imap], impos[imap]);
             b[1] = fint(3, x, naa, a, by, igpos[imap], impos[imap]);
             b[2] = fint(3, x, naa, a, bz, igpos[imap], impos[imap]);
           }  
/*----- transformations in the case the point is not in y>0, x>0 part         */
           if(vectoff[0] < 0.0)   
           { 
             b[1] = -b[1];
             b[2] = -b[2];
           }  
           if(vectoff[1] < 0.0)   
           { 
             b[0] = -b[0];
             b[2] = -b[2];
           }  
        } 
 
        return kTRUE;
 
//  lab230:      /*  outside box the field is equal                         */
        goto lab990;
 
      }      /*   MAGT == 2                                                */
 
 
      if(magt == 3)    
      {    /*   field from the 2-D map of a cylindricall  */
/*                              !   symetric magnet                           */
 
        r1 = sqrt((vectoff[0]-magx)*(vectoff[0]-magx) + 
                   (vectoff[2]-magz)*(vectoff[2]-magz));
        r2 = sqrt((vectoff[0]+magx)*(vectoff[0]+magx) + 
                   (vectoff[2]-magz)*(vectoff[2]-magz));
 
        cyfld(r1, vectoff[1], &br, &bv);
 
        if(r1 > 0.1e-20)   
        { 
          sina = (vectoff[0]-magx) / r1;
          cosa = (vectoff[2]-magz) / r1;
        }
        else
        {
          sina = 0.0;
          cosa = 0.0;
        }  
        b1[0] = sina * br;
        b1[1] = bv;
        b1[2] = cosa * br;
 
        cyfld(r2, vectoff[1], &br, &bv);
 
        if(r2 > 0.1e-20)   
        { 
          sina = (vectoff[0]+magx) / r2;
          cosa = (vectoff[2]-magz) / r2;
        }
        else
        {
          sina = 0.0;
          cosa = 0.0;
        }  
        b2[0] = - sina * br;
        b2[1] = - bv;
        b2[2] = - cosa * br;
 
        b[0] = b1[0] + b2[0];
        b[1] = b1[1] + b2[1];
        b[2] = b1[2] + b2[2];
 
        if(bfld != 0.0)   
        { 
          b[0] = b[0] * bfld/bmax3;
          b[1] = b[1] * bfld/bmax3;
          b[2] = b[2] * bfld/bmax3;
        }  
 
      }      /*    MAGT == 3                                              */
 
      return kTRUE;
 
 
/***********************************************************                  */
/*   General error messages                                *                  */
/***********************************************************                  */
 
  lab941: 
      printf(" gufld - unexpected eof in the field map file\n");
      printf(" gufld - map number: %d\n", nmap);
      goto lab970;
 
  lab942: 
      printf(" gufld - error reading field map file\n");
      printf(" gufld - map number: %d\n", nmap);
      goto lab970;
 
  lab945: 
      printf(" gufld - undefined type of the magnetic field: %d\n", magt);
      goto lab970;
 
  lab946: 
      printf(" gufld - error in the magnetic field map file\n");
      printf(" gufld - illegal type of the magnetic field: %d\n", magt);
      printf(" gufld - only  magt = 2  or  magt = 3  are allowed\n");
      goto lab970;
 
/***********************************************************                  */
/*   Error messages for MAGT=2                             *                  */
/***********************************************************                  */
 
  lab951: 
      printf(" gufld - error: x grid has to be defined first\n");
      printf(" gufld - map number: %d\n", nmap);
      goto lab970;
 
  lab952: 
      printf(" gufld - error: y grid has to be defined before z grid\n");
      printf(" gufld - map number: %d\n", nmap);
      goto lab970;
 
  lab953: 
   printf(" gufld - error: all grids has to be defined before field values\n");
      printf(" gufld - map number: %d\n", nmap);
      goto lab970;
 
  lab954: 
      printf(" gufld - too big map (magt=2) number: %d\n", nmap);
      printf(" gufld - x, y, z grid: %d %d %d\n", nx, ny, nz);
      printf(" gufld - increase parameter MAXGRID\n");
      goto lab970;
 
  lab955: 
      printf(" gufld - too big map (magt=2) number: %d\n", nmap);
      printf(" gufld - x, y, z grid: %d %d %d\n", nx, ny, nz);
      printf(" gufld - increase parameter MAXMAP\n");
      goto lab970;

  lab956: 
      printf(" gufld - wrong grid in map (magt=2) number: %d\n", nmap);
      printf(" gufld - x, y, z grid: %d %d %d\n", nx, ny, nz);
      goto lab970;

  lab957:
      printf(" gufld - too many maps (magt=2) in the file: %d\n", nmap);
      printf(" gufld - increase parameter NMAXMAP\n");
      goto lab970;

  lab958:
      printf(" gufld - illegal PHOMAG value: %d\n", iphoind[nmap]);
      printf(" gufld - map (magt=2) number: %d\n", nmap);
      goto lab970;

  lab959: 
      printf(" gufld - error in index of the map (magt=2) element: %d %d %d \n",
                         ix, iy, iz);
      printf(" gufld - map number: %d\n", imap);
      goto lab970;
 
/***********************************************************                  */
/*   Error messages for MAGT=3                             *                  */
/***********************************************************                  */
 
  lab961: 
      printf(" gufld - too big size of the field map (magt=3) ");
      printf(" or wrong value of r-grid, r,v= %d %d\n", nnr, nnv);
      goto lab970;
 
  lab962: 
      printf(" gufld - too big size of the field map (magt=3): ");
      printf(" or wrong value of v-grid, r,v= %d %d\n", nnr, nnv);
      goto lab970;
 
  lab963: 
      printf(" gufld - error: r,v grids has to be defined before ");
      printf(" field values definitions\n");
      goto lab970;
 
  lab969: 
      printf(" gufld - error in index of the map (magt=3) element: %d %d\n",
                ir, iv);
      goto lab970;
 
/***********************************************************                  */
/*   Final error message                                   *                  */
/***********************************************************                  */
 
  lab970: 
      printf(" gufld - error in the map file, about line: %d\n", nlines);
      printf(" gufld - error line: \n %s\n", cbuf);
      stoper("gufld", "fatal error", 1);
 
 
/***********************************************************                  */
/*   Magnetic field equal 0                                *                  */
/***********************************************************                  */
 
  lab990: 
      b[0] = 0.0;
      b[1] = 0.0;
      b[2] = 0.0;
 
      return kFALSE;
}

Int_t TPhGufldMagnet::keyword(Char_t *c, Char_t *k, Char_t *kk, Int_t n)
{ 
  Int_t nequ=0;
  Int_t i,j;
  for(i=0; i<10; i++) { if(c[i] != ' ')   break; }
  for(j=0; j<n; j++)  
  {  if(c[i+j] != k[j] && c[i+j] != kk[j]) break;  else nequ++; }
 return nequ;
}


Int_t TPhGufldMagnet::addnumber(Char_t *c, Int_t n)
{ int i, k;
for(k=0; c[k] == ' ';  k++);
for(i=0; c[k] != ' ' && c[k] != '\0';  i++) c[i] = c[k++];
c[i] = (char) ((n - n % 10) / 10 + (int) '0');
c[i+1] = (char) (n % 10 + (int) '0');
c[i+2] = '\0';
return i+2;
}


void TPhGufldMagnet::stoper(Char_t *funct, Char_t *mess, Int_t errnum)
{
  printf(" error in the function:  %s \n", funct);
  printf(" error number: %d \n", errnum);
  printf(" error desription: %s \n", mess);
  exit(EXIT_FAILURE);
}


Double_t TPhGufldMagnet::fint(Int_t dim, Double_t *x, Int_t *na, Double_t *a, Double_t *b, Int_t ig, Int_t im)
  //   version for several maps - with an offset
{
  Int_t ii[3], i, k, k1, k2, off, ioff[3];
  Double_t g1,g2, h1,h2, t[3];

  if(dim < 1 || dim > 3)   return 0.0;

  for(k=off=0; k<dim; k++)
  {  ii[k] = na[k];
     ioff[k] = off;
     for(i=0; i<na[k]; i++)
     {   if(a[ig+i+off] > x[k])   {  ii[k] = i;  break;   }
     }
     off+= na[k];
  }
  for(k=0; k<dim; k++)
  {  if(ii[k]==0 || ii[k]==na[k])   return 0.0;
     t[k] = (x[k]-a[ig+ii[k]-1+ioff[k]]) / (a[ig+ii[k]+ioff[k]]-a[ig+ii[k]-1+ioff[k]]);
  }
  if(dim==1)  return (1.0-t[0])*b[im+ii[0]-1] + t[0]*b[im+ii[0]];

  if(dim==2)
  {  k1 = ii[0]-1 + na[0] * (ii[1]-1);
     k2 = ii[0]   + na[0] * (ii[1]-1);
     g1 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     k1 = ii[0]-1 + na[0] * ii[1];
     k2 = ii[0]   + na[0] * ii[1];
     g2 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     return (1.0-t[1])*g1 + t[1]*g2;
  }

  if(dim==3)
  {  
     k1 = ii[0]-1 + na[0] * (ii[1]-1 + na[1] * (ii[2]-1) );
     k2 = ii[0]   + na[0] * (ii[1]-1 + na[1] * (ii[2]-1) );
     g1 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     k1 = ii[0]-1 + na[0] * (ii[1] + na[1] * (ii[2]-1) );
     k2 = ii[0]   + na[0] * (ii[1] + na[1] * (ii[2]-1) );
     g2 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     h1 = (1.0-t[1])*g1 + t[1]*g2;

     k1 = ii[0]-1 + na[0] * (ii[1]-1 + na[1] * ii[2] );
     k2 = ii[0]   + na[0] * (ii[1]-1 + na[1] * ii[2] );
     g1 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     k1 = ii[0]-1 + na[0] * (ii[1] + na[1] * ii[2] );
     k2 = ii[0]   + na[0] * (ii[1] + na[1] * ii[2] );
     g2 = (1.0-t[0]) * b[im+k1] + t[0] * b[im+k2];

     h2 = (1.0-t[1])*g1 + t[1]*g2;
     return (1.0-t[2])*h1 + t[2]*h2;
  }
  return 0.0;
}

Double_t TPhGufldMagnet::fint(Int_t dim, Double_t *x, Int_t *na, Double_t *a, Double_t *b)
{
  Int_t ii[3], i, k, k1, k2, off, ioff[3];
  Double_t g1,g2, h1,h2, t[3];

  if(dim < 1 || dim > 3)   return 0.0;

  for(k=off=0; k<dim; k++)
  {  ii[k] = na[k];
     ioff[k] = off;
     for(i=0; i<na[k]; i++)
     {   if(a[i+off] > x[k])   {  ii[k] = i;  break;   }
     }
     off+= na[k];
  }
  for(k=0; k<dim; k++)
  {  if(ii[k]==0 || ii[k]==na[k])   return 0.0;
     t[k] = (x[k]-a[ii[k]-1+ioff[k]]) / (a[ii[k]+ioff[k]]-a[ii[k]-1+ioff[k]]);
  }
  if(dim==1)  return (1.0-t[0])*b[ii[0]-1] + t[0]*b[ii[0]];

  if(dim==2)
  {  k1 = ii[0]-1 + na[0] * (ii[1]-1);
     k2 = ii[0]   + na[0] * (ii[1]-1);
     g1 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     k1 = ii[0]-1 + na[0] * ii[1];
     k2 = ii[0]   + na[0] * ii[1];
     g2 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     return (1.0-t[1])*g1 + t[1]*g2;
  }

  if(dim==3)
  {  
     k1 = ii[0]-1 + na[0] * (ii[1]-1 + na[1] * (ii[2]-1) );
     k2 = ii[0]   + na[0] * (ii[1]-1 + na[1] * (ii[2]-1) );
     g1 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     k1 = ii[0]-1 + na[0] * (ii[1] + na[1] * (ii[2]-1) );
     k2 = ii[0]   + na[0] * (ii[1] + na[1] * (ii[2]-1) );
     g2 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     h1 = (1.0-t[1])*g1 + t[1]*g2;

     k1 = ii[0]-1 + na[0] * (ii[1]-1 + na[1] * ii[2] );
     k2 = ii[0]   + na[0] * (ii[1]-1 + na[1] * ii[2] );
     g1 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     k1 = ii[0]-1 + na[0] * (ii[1] + na[1] * ii[2] );
     k2 = ii[0]   + na[0] * (ii[1] + na[1] * ii[2] );
     g2 = (1.0-t[0]) * b[k1] + t[0] * b[k2];

     h2 = (1.0-t[1])*g1 + t[1]*g2;
     return (1.0-t[2])*h1 + t[2]*h2;
  }
  return 0.0;
}


void TPhGufldMagnet::cyfld(Double_t r, Double_t v, Double_t *br, Double_t *bv)
{ 
/*CMZ :  1.00/00 17/12/93  07.30.06  by  Phobos                               */
/*-- Author :                                                                 */
/*Documentation_Block                                                         */
/************************************************************************     */
/*                                                                            */
/* Description:                                                               */
/*   Calculate the magnetic field from the two dimensional                    */
/*   cylindrical field map.                                                   */
/*   The map is stored in common block defined in CYFLDC.CMN                  */
/*   and read by GUFLD prior tothe first call to CYFLD                        */
/*                                                                            */
/*   It is possible to use the magnetic field map calculated for the magnet   */
/*   with smaller radius for other with the radius increased                  */
/*   by REXT (local parameter). In this case the field values for R < REXT are*/
/*   identical with those for R=0 for the same value  V (in the original map) */
/*   This option should however be used only temporarily -                    */
/*   the magnetic field created this way is only aproximate                   */
/*                                                                            */
/* Arguments:                                                                 */
/*  *R   - radius of the requested point (cylindrical coordinate 1)           */
/*  *V   - vertical position of the point (cylindrical coordinate 2)          */
/*  BR*  - radial component of the field                                      */
/*  BV*  - vertical component of the field                                    */
/*                                                                            */
/* Called by:                                                                 */
/*  GUFLD                                                                     */
/*                                                                            */
/* Calls:                                                                     */
/*  FINT                                                                      */
/*                                                                            */
/* Implicit inputs, outputs, side effects:                                    */
/*  NONE                                                                      */
/*                                                                            */
/* Filename: CYFLD.FOR                                                        */
/*                                                                            */
/* History:                                                                   */
/* Date      Name       Description                                           */
/* ----      ----       -----------                                           */
/* 2-Dec-93  K.W.        Creation                                             */
/************************************************************************     */
/*End_Documentation_Block                                                     */
 
  Double_t x[2], p;
  Double_t rext=0.0;
 
  Int_t nn[2];
 
  nn[0] = nnr;
  nn[1] = nnv;

  x[0] = r - rext;
  if(x[0] < 0.0)  x[0] = 0.0;

//ao correction abs-fabs
  x[1] = fabs(v);
 
  if(r < 0.0)  
  {   printf(" gufld: negative  r  value \n");
      goto lab995;
  }
 
  if(x[0] > 2.0*xtab[nnr-1])   goto lab995;
  if(x[1] > 2.0*xtab[nnr+nnv-1])   goto lab995;
 
  if(x[0] > xtab[nnr-1])   x[0] = xtab[nnr-1];
  if(x[1] > xtab[nnr+nnv-1])   x[1] = xtab[nnr+nnv-1];
 
  *br = fint(2, x, nn, xtab, brtab);
  *bv = fint(2, x, nn, xtab, bvtab);
 
  p = 1.0;
  if(r-rext > xtab[nnr-1])  p = (2.0*xtab[nnr-1]-r+rext) / xtab[nnr-1];
  if(fabs(v) > xtab[nnr+nnv-1])   
     p = p * (2.0*xtab[nnr+nnv-1] - fabs(v)) / xtab[nnr+nnv-1];
  *br = *br * p;
  *bv = *bv * p;
 
  if(v < 0.0)  *br = - *br;
 
  goto lab999;
 
lab995: 
  *br = 0.0;
  *bv = 0.0;

lab999: 
      return;
}


void TPhGufldMagnet::ls()
{
  TPhMagnet::ls();

  printf("\n --> Gufld parameters <--\n");
  printf("  magv(ersion) = %g\n", magv);
  printf("  magt(ype)    = %d\n", magt);
  printf("  bfld(max)    = %g\n", bfld);
  printf("  magx         = %g\n", magx);
  printf("  magz         = %g\n", magz);
  printf("  r(adius)fld  = %g\n", rfld);
  printf("  moff(set)[]  = %g %g %g\n", moff[0],moff[1],moff[2]);
  printf(" --> Gufld parameters <--\n\n");
}
