#include "TPhRingSensor.h"

#include "TTUBS.h"

ClassImp(TPhRingSensor)

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// This class implements the Ring Sensor. 
//
//   There are two shapes in this class: 
//  1. A trapazoid to implement the Ring Sensor Geometry as it was done up to UGCData V3.8
//     USE TPhRingSensorV3.8 for that!
//
//  2. A tube to implement the Ring Sensor Geometry from UGCData V4.0 upwards.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


TPhRingSensor::TPhRingSensor() 
{
  // Default Ring Multiplicity Sensor constructor
  SetTitle("Ring Multiplicity Sensor");
  fRmin = 0;
  fRmax = 0;
  fDz = 0;
  fPhi1 = 0;
  fPhi2 = 0;
}

TPhRingSensor::~TPhRingSensor() 
{
  // Normal destructor
  SafeDelete(fMatrix);
  SafeDelete(fShape);
}
  
void TPhRingSensor::SetParameters(Float_t Rmin, Float_t Rmax, Float_t Dz, 
				  Float_t Phi1, Float_t Phi2)
{
  // Set the parameters of the TUBE - this is for UGCData V4.0 and above!
  Char_t name[40];
  fRmin = Rmin;
  fRmax = Rmax;
  fDz = Dz;
  fPhi1 = Phi1;
  fPhi2 = Phi2;
  sprintf(name, "Tubs_%s", GetName());
  fShape = new TTUBS(name, name, "", fRmin, fRmax, fDz, fPhi1, fPhi2);
}

void TPhRingSensor::SetParameters(Float_t *Parameter)
{
  // Set the parameters of the TUBE - this is for UGCData V4.0 and above!

  SetParameters(Parameter[0], Parameter[1], Parameter[2], Parameter[3], Parameter[4]);
}

void TPhRingSensor::SetPadParameters(Float_t Dphi, Float_t R0, Float_t R1, Float_t R2, Float_t R3, 
                                     Float_t R4, Float_t R5, Float_t R6, Float_t R7, Float_t R8)
{
  // Set the parameters of pixels in the TUBE - this is for UGCData V4.0 and above!
  fDphi = Dphi;
  fRs[0] = R0; fRs[1] = R1; fRs[2] = R2; fRs[3] = R3; fRs[4] = R4; 
  fRs[5] = R5; fRs[6] = R6; fRs[7] = R7; fRs[8] = R8;
}

void TPhRingSensor::SetPadParameters(Float_t *PParameter)
{
  // Set the parameters of pixels in the TUBE case - this is for UGCData V4.0 and above!

  SetPadParameters(PParameter[0], PParameter[1], PParameter[2], PParameter[3], PParameter[4], 
                   PParameter[5], PParameter[6], PParameter[7], PParameter[8], PParameter[9]);
}

void TPhRingSensor::Draw(Option_t *option)
{
  // Draw the ring sensors in green

  TPhSensor::Draw(option);
  SetLineColor(kGreen);
}
  
