#ifndef _TPhDept_INCLUDED_
#define _TPhDept_INCLUDED_

////////////////////////////////////////////////////////////////////////
//
// Header file for the Phobos multiplicity energy deposition class
// The TPhDept class gets the average energy deposition per track as a 
// function of pseudorapidity eta for each detector plane. 
// Energies are obtained by interpolating values of PEmip(1,nn,idet) 
// at eta = PEmip(2,nn,idet), nn = 1...Neseg.
// The Fit methods use values from MC studies to obtain new values.
// The values specified in TPhDept.v were taken from fits to Phobos MC
// results.
//
// File: TPhDept.h
//
// Date     Vers  Description
// 16-Dec-98 1.0  Comment-out unused args for inimplemented Fit method (RV)
// 
// $Id: TPhDept.h,v 1.5 1998/12/21 21:39:00 verdier Exp $
//
////////////////////////////////////////////////////////////////////////

// 22-Sep-98 0.3  Moved kNSeg out of class scope and kNSeg->kNSegE (mpd)
// 22-May-98 0.3  Implement GetPar and SetPar methods, add TPhSensDef
//                argument to constructor (RV)
// 02-Mar-98 0.3  Add gFitDept for access to user fit functions (RV)
// 02-Feb-98 0.1  Add methods for fitting parameters (RV)
// 04-Nov-97 0.1  Creation based on M_GetDept (Robin Verdier)

#include <stdio.h>

// PHOBOS Classes

// forward declarations

 class TObject;        

// Storage limit kNSensMax is defined in TPhMul.h
#include "TPhMul.h"

extern "C" Double_t CWrapFit(Double_t *x, Double_t *par);

const Int_t       kNSegE = 7; // Number of eta-ranges for each detector

class TPhDept : public TObject

{

  public:

  TPhDept(TPhSensDef &sd);

//  TPhDept(FILE *values); // UNIMPLEMENTED


  Int_t    Fit (/*Char_t *sensorName, TPhDept *edep, TPhPadMap **disps, 
             Int_t varIndex, Int_t varRMSIndex, Char_t *outFileName*/);
           // UNIMPLEMENTED

  Float_t  GetDept (Int_t IDet, Float_t Eta);

  Int_t    GetFitSensorId (); // Id of sensor being fitted

  Int_t    GetFitType (Char_t *sensorName); // UNUSED

  Double_t GetFitValue (Double_t *x, Double_t *pars); // for ROOT fit class

  Int_t    GetNPars (Int_t sensorId);

  Float_t  GetPar (Int_t sensorId, Int_t parIndex);
           // Returns dE[0], Eta[0], dE[1], Eta[1], ...

// The GetPar and SetPar methods by name ASSUME the name ordering is the
// standard one, which MUST therefore agree with the external .v file order.
  Float_t  GetPar (Char_t *sensorName, Int_t parIndex);

  void     SetFitSensorId (Int_t sensorID);

  void     SetPar (Int_t sensorId, Int_t parIndex, Float_t value);

  void     SetPar (Char_t *sensorName, Int_t parIndex, Float_t value);

  void     SetupFit();

  // Set methods allowing different fit parameterizations will be added.

  private:

  Int_t       fFitSensId; // Sensor being fitted, NON-SHARABLE
  TPhSensDef *fSensDefs; // Access to sensor names and properties

// The following array of values has to be declared static in order that 
// the defaults can be loaded from TPhDept.v, which should be included 
// by the .cxx file at some point following the include for this file.

  static Float_t fF[kNSensMax][kNSegE][2];

public:

ClassDef(TPhDept,1)      // Phobos multiplicity energy deposition class

};

#endif
