#ifndef _TPhFsec_INCLUDED_
#define _TPhFsec_INCLUDED_
////////////////////////////////////////////////////////////////////////
// TPhFsec is a secondary fraction calculator class for Phobos analysis
// that gives the total / primary track ratio as a function of 
// pseudorapidity eta for each detector plane. 
// Values are obtained by interpolating values of Fsec[idet][nn][0] 
// at eta = Fsec[idet][nn][1], nn = 0...NSeg-1.
// The values specified in TPhFsec.v were taken from fits to Phobos MC
// results.
//
// File: TPhFsec.h
//
// Date     Vers  Description
// 16-Dec-98 1.0  Comment-out unused args for inimplemented Fit method (RV)
//
// $Id: TPhFsec.h,v 1.3 1998/12/21 21:43:43 verdier Exp $
// 
////////////////////////////////////////////////////////////////////////

// 22-Sep-98 0.3  Moved kNSeg out of class scope.
// 22-May-98 0.3  Implement GetPar and SetPar methods, add TPhSensDef
//                argument to constructor (RV)
// 04-Nov-97 0.1  Creation based on M_GetFsec (R.Verdier)

#include <stdio.h>

// PHOBOS Classes

// forward declarations

 class TObject;        

// Storage limit kNSensMax is defined in TPhMul.h
#include "TPhMul.h"

const Int_t kNSeg = 7;       // Number of eta-ranges for each detector

class TPhFsec : public TObject

{

public:

  TPhFsec(TPhSensDef &sd);

//  TPhFsec(FILE *values); // UNIMPLEMENTED

  Int_t   Fit (/*Char_t *sensorName, TPhFsec *fsec, TPhPadMap **disps, 
                    Int_t varIndex, Int_t varRMSIndex, Char_t *outFileName*/);
  // UNIMPLEMENTED

  Int_t   GetFitSensorId ();

  Int_t   GetFitType (Char_t *sensorName); // UNIMPLEMENTED

// Double_t GetFitValue (Double_t *x, Double_t *pars) {} UNIMPLE-
// MENTED, see TPhDept

  Float_t GetFsec (Int_t IDet, Float_t Eta);

  Int_t   GetNPars (Int_t sensorId);

  Float_t GetPar (Int_t sensorId, Int_t parIndex);

  Float_t GetPar (Char_t *sensorName, Int_t parIndex);

  void    SetFitSensorId (Int_t sensorId);

  void    SetPar (Int_t sensorId, Int_t parIndex, Float_t value);

  void    SetPar (Char_t *sensorName, Int_t parIndex, Float_t value) ;

// void   SetupFit() {} UNIMPLEMENTED, see TPhDept

private:

  Int_t       fFitSensId; // Sensor being fitted, NON-SHARABLE
  TPhSensDef *fSensDefs; // Access to sensor names and properties

// The following array of values has to be declared static in order that 
// the defaults can be loaded from TPhFsec.v, which should be included 
// by the .cxx file at some point following the include for this file.

  static Float_t fF[kNSensMax][kNSeg][2];

#ifndef _STANDALONE_
public:
ClassDef(TPhFsec,1)      // Phobos secondary fraction calculator class
#endif

};

#endif
