#ifndef _TPhMapSet_INCLUDED_
#define _TPhMapSet_INCLUDED_
////////////////////////////////////////////////////////////////////////
//
// TPhMapSet manages an aggregate of TPhPadMaps.
//
// File: TPhMapSet.h
//
// Date     Vers  Description
// 09-Feb-99 1.0  Make print and report levels public; add SetFree (RV)
//
// $id$
//
////////////////////////////////////////////////////////////////////////

// 24-Jan-99 1.0  Creation (Robin Verdier)

// ROOT Classes

class TObject;        

// Phobos Classes

class TPhDispDef;

class TPhPadMap;

class TPhSensDef;

// forward declarations

class TPhMapSet : public TObject

{

public:

// static causes a link failure
  Int_t fIPrint;
  Int_t fNPrint;
  Int_t fReportLevel;

  TPhMapSet
  ( Int_t mapType = 0  // 0 = physics displays, else sensor maps
  );

  virtual ~TPhMapSet();

  Int_t Add (TPhPadMap *map = (TPhPadMap *)NULL);

  void Clear();

  Int_t Delete (Int_t iFirst, Int_t nMaps = 1);

  Int_t Delete (Char_t *name, Int_t nMaps = 1);

//  void *GetDMD() { return fDMD; } // Get default map definitions

  TPhDispDef *GetDDD() { return fDDD; } // Get default display map definitions

  TPhSensDef *GetDSD() { return fDSD; } // Get default sensor map definitions

  TGraphErrors *GetGraph
  ( Char_t *rName,         // Name of TGraphErrors for gROOT->GetObject access
    Char_t *selector,      // Selector string such as "MPA3" or "MPA*"
    Int_t   iVar = 0,      // Index of variable, defined in TPhMul.h
    Int_t   iVarErr = -1,  // Index of error variable, -1 if none
    Float_t norm = 1.0,    // Normalizer, e.g. 360 to integrate on phi
    Int_t   nXBins = 0,    // Number of X-bins in histo
    Int_t   nYBins = 0,    // Number of Y-bins in histo
    Int_t   nXMinSkip = 0, // Number of low X-bins to omit
    Int_t   nYMinSkip = 0, // Number of low Y-bins to omit
    Int_t   nXMaxSkip = 0, // Number of high X-bins to omit
    Int_t   nYMaxSkip = 0  // Number of high Y-bins to omit
  );

  TGraphErrors *GetGraph
  ( Char_t *rName,         // Name of TGraphErrors for gROOT->GetObject access
    Int_t   iMap = -1,     // Index of map from ListDisplays() or ListSensors()
    Int_t   iVar = 0,      // Index of histo'd variable, defined in TPhMul.h
    Int_t   iVarErr = -1,  // Index of error variable, -1 if none
    Float_t norm = 1.0,    // Normalizer, e.g. 360 to integrate on phi
    Int_t   nXBins = 0,    // Number of X-bins in histo
    Int_t   nYBins = 0,    // Number of Y-bins in histo
    Int_t   nXMinSkip = 0, // Number of low X-bins to omit
    Int_t   nYMinSkip = 0, // Number of low Y-bins to omit
    Int_t   nXMaxSkip = 0, // Number of high X-bins to omit
    Int_t   nYMaxSkip = 0  // Number of high Y-bins to omit
  );

  TH2F *GetHist
  ( Char_t *rName,         // Name of TH2F for access via gROOT->GetObject
    Char_t *selector,      // Selector string such as "MPA3" or "MPA*"
    Int_t   iVar = 0,      // Index of histo'd variable, defined in TPhMul.h
    Int_t   iVarErr = -1,  // Index of error variable, presently unused
    Float_t norm = 1.0,    // Normalizer, e.g. 360 to integrate on phi
    Int_t   nXBins = 0,    // Number of X-bins in histo
    Int_t   nYBins = 0,    // Number of Y-bins in histo
    Int_t   nXMinSkip = 0, // Number of low X-bins to omit
    Int_t   nYMinSkip = 0, // Number of low Y-bins to omit
    Int_t   nXMaxSkip = 0, // Number of high X-bins to omit
    Int_t   nYMaxSkip = 0  // Number of high Y-bins to omit
  );

  TH2F *GetHist
  ( Char_t *rName,         // Name of TH2F for access via gROOT->GetObject
    Int_t   iMap = -1,     // Index of map from ListDisplays() or ListSensors()
    Int_t   iVar = 0,      // Index of histo'd variable, defined in TPhMul.h
    Int_t   iVarErr = -1,  // Index of error variable, presently unused
    Float_t norm = 1.0,    // Normalizer, e.g. 360 to integrate on phi
    Int_t   nXBins = 0,    // Number of X-bins in histo
    Int_t   nYBins = 0,    // Number of Y-bins in histo
    Int_t   nXMinSkip = 0, // Number of low X-bins to omit
    Int_t   nYMinSkip = 0, // Number of low Y-bins to omit
    Int_t   nXMaxSkip = 0, // Number of high X-bins to omit
    Int_t   nYMaxSkip = 0  // Number of high Y-bins to omit
  );

  TPhPadMap  *GetMap (Int_t iMap); // Get TPhPadMap by index

  TPhPadMap  *GetMap (Char_t *name); // Get TPhPadMap by name

  Int_t  GetMapIndex (Char_t *name); // Get index of named TPhPadMap

// Note that the # events may become different for different maps, but not yet
  Int_t  GetNEvents () { return fNEvents; } // Get # of events accumulated

  Int_t  GetNMaps () { return fNMaps; } // Get maximum # of TPhPadMaps

  Int_t  GetNMapsAct (); // Get # of 'active' TPhPadMaps (see implementation)

  Int_t  GetNStats () { return fNStats; } // Get size of statistics buffers

  Int_t  GetType () { return fMapType; } // 0 for display, 1 for sensor

  void   List (Int_t iMapMin = 0, Int_t iMapMax = 0);

  Int_t Replace (TPhPadMap *map);

  Int_t Replace (Char_t *name, TPhPadMap *map = (TPhPadMap *)NULL);

  Int_t Replace (Int_t iMap = -1, TPhPadMap *map = (TPhPadMap *)NULL);

  Int_t SetFree(Int_t iFirstMap = -1, Int_t iLastMap = -1);

private:

  Int_t       fMapType;  // 0 = physics displays, else sensor maps
//  void       *fDMD;      // Default map definitions, TPh{Disp|Sens}Def
  TPhDispDef *fDDD;       // Default display map definitions
  TPhSensDef *fDSD;       // Default sensor map definitions
  Int_t       fNMaps;     // Number of maps
  TPhPadMap **fMaps;      // List of map pointers

// Event-specific data that may affect the analysis.  Some is stored in
// other places

  Int_t   fNEvents;          // # events entered in the cumulative maps

// The following will become structs:
  Float_t fVX;               // Position of vertex in current event, in cm.
  Float_t fVY;
  Float_t fVZ;
  enum  { ENStats = 20 };
  Int_t   fNStats;           // Size of statistics buffers, words
  Float_t fStatsEv[ENStats]; // Statistics for the current event
  Float_t fStats[ENStats];   // Cumulative statistics for the run
 
public:

ClassDef(TPhMapSet,1)      // Phobos TPhPadMap aggregate class

};

#endif
