#ifndef _TPhulGeo_INCLUDED_
#define _TPhMulGeo_INCLUDED_
/////////////////////////////////////////////////////////////////////////////
// TPhMulGeo is a class specifying the geometry of a single Phobos sensor
// or display map.
// The coordinate system of a pad array numbers the coordinates as 1, 2,
// and 3, where 1 and 2 lie in the plane of the array and 3 is the
// vector product 1 cross 2.  We will choose coordinate 1 along the
// negative gradient of theta, that is, along the +Z beam direction for
// the octagon and inward along a radius for the rings, and coordinate
// 2 along the negative gradient of phi at the center of the sensor.
// Coordinate 3 then points away from the interaction point.
// Orig and Axes vectors are given in the world coordinate system.
// Distance units are centimeters.
// This structure will be expanded to deal with segmented ring counters.
//
// File: TPhMulGeo.h
//
// Date     Vers  Description
// 14-Nov-98 1.0  Created from existing struct TPhSensGeo.h (RV)
//
// $Id: TPhMulGeo.h,v 1.3 1998/12/03 16:50:25 verdier Exp $
//
/////////////////////////////////////////////////////////////////////////////

// 23-Feb-98 0.2  Name length extended to allow *-V*-MN*-MP* (RV)
// 20-Jan-98 0.2  Name length extended (RV)
// 03-Nov-97 0.1  Creation (Robin Verdier)
/////////////////////////////////////////////////////////////////////////////

// forward declarations

class TObject;

class TPhMulGeo : public TObject

{

enum  { EnameSize = 20 }; // Storage limit on name length, characters

public:

TPhMulGeo();
TPhMulGeo(Char_t Name[], Int_t Id, Int_t Type, Int_t NXcells, Float_t Xmin,
  Float_t Xsize, Int_t NYcells, Float_t Ymin, Float_t Ysize, Float_t Dist,
  Float_t Orig[3], Float_t Axes[3][3], Float_t Version);

  Char_t  Name[EnameSize+1]; // Name of sensor, null-terminated
  Int_t   Id;         // ID # of associated sensor
  Int_t   Type;       // Sensor type, 1 for barrel/vertex, 2 for ring
  Int_t   NXcells;    // Number of pads in direction 1
  Float_t Xmin;       // Low edge of active area in local coordinate 1
  Float_t Xsize;      // Size of cell in direction 1
  Int_t   NYcells;    // Number of pads in direction 2
  Float_t Ymin;       // Low edge of active area in local coordinate 2
  Float_t Ysize;      // Size of cell in direction 2
  Float_t Dist;       // Distance of plane from world coordinate origin
  Float_t Orig[3];    // Origin of array in world coordinates
  Float_t Axes[3][3]; // Coordinate axis set
  Float_t Version;    // Version number for this format

ClassDef(TPhMulGeo,1) // Phobos multiplicity sensor geometry struct

};

#endif

