// Default sensor definitions for Phobos class TPhSensDef
// File: TPhSensDefV.h
// Date     Vers  Description
// 16-Dec-98 3.0  Use syntax {{...},{...},...} to avoid compiler warnings (RV)
// $Id: TPhSensDefV.h,v 1.4 1998/12/21 21:46:49 verdier Exp $
//////////////////////////////////////////////////////////////////////////////

// 30-Oct-98 2.0  Reverse phi sense again for MOA-D & VBA (RV)
// 06-Aug-98 1.0  Adjust MOB, D, F, H and VTB to ugcdata v 4.2; add \n  (RV)
// 31-Jul-98 1.0  Change id's -> active as noted by M. Reuter (RV)
// 10-Jul-98 1.0  Update MOxx and Vxx, reverse phi sense for MOA -> MOE,
//                VBA, VBB, and VTB4-7; update MP and MN from UGCDATA 4.2
//                since there are apparently no TPhRingSensor Gets (RV)
//                Note VTB4-7 now OVERLAY VTB0-3!  Phat needs Phixing.
// 20-Jun-98 1.0  Correct vertex sensors (RV)
// 19-Jun-98 1.0  Change fSensors initializer for OSF1 (cf. TPhDispDef) (RV)
// 16-Jun-98: Values from showgeom.C corrected for reversed sense in Phat (RV)
// 04-May-98: Values for UGCDATA V4.1 incl. dr/r .1040->.1035 (RV)
//            Note the rings may be oriented about 90 degrees off #s here;
//            to be fixed when the smoke clears 
// 12-Mar-98: c225 etc. declared here for Abacus (RV)
// 20-Feb-98: Renamed from TPhSensGeo.v
// 20-Nov-97: Values for current UGCDATA V3.8: most important,
//            octagon slats doubled phi resolution (RV)
// 13-Oct-97: Creation using values from mulcontrol.nml (Robin Verdier)

static Float_t c225 = 0.92388; //! NO STREAMER
static Float_t s225 = 0.38268; //! NO STREAMER
static Float_t c45  = 0.70711; //! NO STREAMER

static TPhSensGeo gMulSensors[] =
{
 { "MNC0",  1,  2,  8,  5.00,  0.1035,  8,   0.0,  5.62, -504.8, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC1",  1,  2,  8,  5.00,  0.1035,  8,  45.0,  5.62, -504.8, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  -504.8, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  -504.8, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  -504.8, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  -504.8, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  -504.8, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNC7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  -504.8, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

 { "MNB0",  1, 2,   8,  5.00, 0.1035, 8,   0.0,  5.62,  -234.9, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB1",  1, 2,   8,  5.00, 0.1035, 8,  45.0,  5.62,  -234.9, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  -234.9, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  -234.9, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0},  {0, 0,1}}, 0.1 },
 { "MNB4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  -234.9, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  -234.9, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  -234.9, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNB7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  -234.9, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

 { "MNA0",  1, 2,   8,  5.00, 0.1035, 8,   0.0,  5.62,  -112.8, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA1",  1, 2,   8,  5.00, 0.1035, 8,  45.0,  5.62,  -112.8, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  -112.8, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  -112.8, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0},  {0, 0,1}}, 0.1 },
 { "MNA4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  -112.8, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  -112.8, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  -112.8, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MNA7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  -112.8, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

 { "MOA0",  1,  1,   30, -55.12,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOA1",  1,  1,   30, -46.62,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOA2",  1,  1,   30, -38.10,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOA3",  1,  1,   30, -29.59,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOA4",  1,  1,   30, -21.08,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOA5",  1,  1,   30, -12.57,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },

 { "MOA9",  1,  1,   30,  21.47,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOAA",  1,  1,   30,  29.98,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOAB",  1,  1,   30,  38.50,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, {-1, 0, 0}},  0.1 },
 { "MOAC",  1,  1,   30,  47.00,  0.271,  4,  -21.0, 10.5,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },

 { "MOB0",  1,  1,   30, -55.12,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB1",  1,  1,   30, -46.62,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB2",  1,  1,   30, -38.10,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB3",  1,  1,   30, -29.59,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB4",  1,  1,   30, -21.08,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB5",  1,  1,   30, -12.57,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB6",  1,  1,   30,  -4.06,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB7",  1,  1,   30,   4.45,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB8",  1,  1,   30,  12.96,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOB9",  1,  1,   30,  21.47,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOBA",  1,  1,   30,  29.98,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOBB",  1,  1,   30,  38.50,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },
 { "MOBC",  1,  1,   30,  47.00,  0.271,  4,  23.94, 10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, -c45, 0}, { c45, c45, 0}},  0.1 },

 { "MOC0",  1,  1,   30, -55.12,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOC1",  1,  1,   30, -46.62,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOC2",  1,  1,   30, -38.10,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOC3",  1,  1,   30, -29.59,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOC4",  1,  1,   30, -21.08,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },

 { "MOC8",  1,  1,   30,  12.96,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOC9",  1,  1,   30,  21.47,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOCA",  1,  1,   30,  29.98,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOCB",  1,  1,   30,  38.50,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "MOCC",  1,  1,   30,  47.00,  0.271,  4,  69.0,  10.50,  4.53, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },

 { "MOD0",  1,  1,   30, -55.12,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD1",  1,  1,   30, -46.62,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD2",  1,  1,   30, -38.10,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD3",  1,  1,   30, -29.59,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD4",  1,  1,   30, -21.08,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD5",  1,  1,   30, -12.57,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD6",  1,  1,   30,  -4.06,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD7",  1,  1,   30,   4.45,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD8",  1,  1,   30,  12.96,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MOD9",  1,  1,   30,  21.47,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MODA",  1,  1,   30,  29.98,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MODB",  1,  1,   30,  38.50,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },
 { "MODC",  1,  1,   30,  47.00,  0.271,  4, 113.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { c45, c45, 0}, { -c45, c45, 0}},  0.1 },

 { "MOE0",  1,  1,   30, -55.12,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOE1",  1,  1,   30, -46.62,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOE2",  1,  1,   30, -38.10,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOE3",  1,  1,   30, -29.59,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOE4",  1,  1,   30, -21.08,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOE5",  1,  1,   30, -12.57,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },

 { "MOE9",  1,  1,   30,  21.47,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOEA",  1,  1,   30,  29.98,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOEB",  1,  1,   30,  38.50,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },
 { "MOEC",  1,  1,   30,  47.00,  0.271,  4, 158.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { 0, 1, 0}, { -1, 0, 0}},  0.1 },

 { "MOF0",  1,  1,   30, -55.12,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF1",  1,  1,   30, -46.62,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF2",  1,  1,   30, -38.10,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF3",  1,  1,   30, -29.59,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF4",  1,  1,   30, -21.08,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF5",  1,  1,   30, -12.57,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF6",  1,  1,   30,  -4.06,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF7",  1,  1,   30,   4.45,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF8",  1,  1,   30,  12.96,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOF9",  1,  1,   30,  21.47,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOFA",  1,  1,   30,  29.98,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOFB",  1,  1,   30,  38.50,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },
 { "MOFC",  1,  1,   30,  47.00,  0.271,  4, 203.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, c45, 0}, { -c45, -c45, 0}},  0.1 },

 { "MOG0",  1,  1,   30, -55.12,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOG1",  1,  1,   30, -46.62,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOG2",  1,  1,   30, -38.10,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOG3",  1,  1,   30, -29.59,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOG4",  1,  1,   30, -21.08,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },

 { "MOG8",  1,  1,   30,  12.96,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOG9",  1,  1,   30,  21.47,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOGA",  1,  1,   30,  29.98,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOGB",  1,  1,   30,  38.50,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "MOGC",  1,  1,   30,  47.00,  0.271,  4, 248.8,  10.6,  4.50, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },

 { "MOH0",  1,  1,   30, -55.12,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH1",  1,  1,   30, -46.62,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH2",  1,  1,   30, -38.10,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH3",  1,  1,   30, -29.59,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH4",  1,  1,   30, -21.08,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH5",  1,  1,   30, -12.57,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH6",  1,  1,   30,  -4.06,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH7",  1,  1,   30,   4.45,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH8",  1,  1,   30,  12.96,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOH9",  1,  1,   30,  21.47,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOHA",  1,  1,   30,  29.98,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOHB",  1,  1,   30,  38.50,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },
 { "MOHC",  1,  1,   30,  47.00,  0.271,  4, 293.94,  10.53,  4.52, 
    {0, 0, 0}, {{ 0, 0, 1}, { -c45, -c45, 0}, { c45, -c45, 0}},  0.1 },

 { "MPA0",  1, 2,   8,  5.00, 0.1035, 8,   0.0,  5.62,  +112.8, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA1",  1, 2,   8,  5.00, 0.1035, 8,  45.0,  5.62,  +112.8, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  +112.8, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  +112.8, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0},  {0, 0,1}}, 0.1 },
 { "MPA4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  +112.8, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  +112.8, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  +112.8, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPA7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  +112.8, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

 { "MPB0",  1, 2,   8,  5.00, 0.1035, 8,   0.0,  5.62,  +234.9, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB1",  1, 2,   8,  5.00, 0.1035, 8,  45.0,  5.62,  +234.9, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  +234.9, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  +234.9, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0},  {0, 0,1}}, 0.1 },
 { "MPB4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  +234.9, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  +234.9, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  +234.9, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPB7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  +234.9, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

 { "MPC0",  1, 2,   8,  5.00, 0.1035, 8,   0.0,  5.62,  +504.8, 
    {0, 0, 0}, {{ c225,  s225, 0}, {-s225,  c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC1",  1, 2,   8,  5.00, 0.1035, 8,  45.0,  5.62,  +504.8, 
    {0, 0, 0}, {{ s225,  c225, 0}, {-c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC2",  1, 2,   8,  5.00, 0.1035, 8,  90.0,  5.62,  +504.8, 
    {0, 0, 0}, {{-s225,  c225, 0}, { c225,  s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC3",  1, 2,   8,  5.00, 0.1035, 8, 135.0,  5.62,  +504.8, 
    {0, 0, 0}, {{-c225,  s225, 0}, { s225,  c225, 0},  {0, 0,1}}, 0.1 },
 { "MPC4",  1, 2,   8,  5.00, 0.1035, 8, 180.0,  5.62,  +504.8, 
    {0, 0, 0}, {{-c225, -s225, 0}, { s225, -c225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC5",  1, 2,   8,  5.00, 0.1035, 8, 225.0,  5.62,  +504.8, 
    {0, 0, 0}, {{-s225, -c225, 0}, { c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC6",  1, 2,   8,  5.00, 0.1035, 8, 270.0,  5.62,  +504.8, 
    {0, 0, 0}, {{ s225, -c225, 0}, {-c225, -s225, 0}, {0, 0, 1}}, 0.1 },
 { "MPC7",  1, 2,   8,  5.00, 0.1035, 8, 315.0,  5.62,  +504.8, 
    {0, 0, 0}, {{ c225, -s225, 0}, {-s225, -c225, 0}, {0, 0, 1}}, 0.1 },

// From the TPhSensor::Get(xx) methods after reading geometry V. 4.2
// (source: UGCDATA_V42.TXT), inner pads "VxAn" are 1.202 x 0.04725
// at y = -5.640 and +5.640, and outer pads "VxBn" are 2.4045 x 0.04725
// at y = -11.785 and +11.785, in cm.  The angles here are taken from 
// these extents, slightly modified to try to get the same phi
// acceptance as the sensors including their gaps.

 { "VBA0",  1,  1,  128, -12.49,  0.047,  4, 246.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBA1",  1,  1,  128,  -6.18,  0.047,  4, 246.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBA2",  1,  1,  128,  +0.13,  0.047,  4, 246.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBA3",  1,  1,  128,   6.44,  0.047,  4, 246.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },

 { "VBB0",  1,  1,  128, -12.49,  0.047,  2, 292.88, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB1",  1,  1,  128,  -6.18,  0.047,  2, 292.88, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB2",  1,  1,  128,  +0.13,  0.047,  2, 292.88, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB3",  1,  1,  128,   6.44,  0.047,  2, 292.88, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB4",  1,  1,  128, -12.49,  0.047,  2, 269.20, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB5",  1,  1,  128,  -6.18,  0.047,  2, 269.20, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB6",  1,  1,  128,   0.13,  0.047,  2, 269.20, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },
 { "VBB7",  1,  1,  128,   6.44,  0.047,  2, 269.20, -11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { -1, 0, 0}, { 0, -1, 0}},  0.1 },

 { "VTA0",  1,  1,  128, -12.49,  0.047,  4,  66.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTA1",  1,  1,  128,  -6.18,  0.047,  4,  66.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTA2",  1,  1,  128,   0.13,  0.047,  4,  66.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTA3",  1,  1,  128,   6.44,  0.047,  4,  66.90,  11.55,  5.64, 
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },

 { "VTB0",  1,  1,  128, -12.49,  0.047,  2,  67.12,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB1",  1,  1,  128,  -6.18,  0.047,  2,  67.12,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB2",  1,  1,  128,  +0.13,  0.047,  2,  67.12,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB3",  1,  1,  128,   6.44,  0.047,  2,  67.12,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB4",  1,  1,  128, -12.49,  0.047,  2,  90.80,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB5",  1,  1,  128,  -6.18,  0.047,  2,  90.80,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB6",  1,  1,  128,  +0.13,  0.047,  2,  90.80,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 },
 { "VTB7",  1,  1,  128,   6.44,  0.047,  2,  90.80,  11.04, 11.78,
    {0, 0, 0}, {{ 0, 0, 1}, { 1, 0, 0}, { 0, 1, 0}},  0.1 }
};
