// apxtoroot.C: 21-Nov-98 ConvApxToRoot w. filename dialogue modified (RV)
// Made by A.O. on 4.11.1997
// Converts .AOU or .APX ascii output files from Phobos MC Geant based program
// to root format, creating MCTracks, MCHits, MCHitArrays and 
// MCRawDataBuffers data containers for each data event.
//main()
{
  gROOT.Reset();

  TStopwatch timer;      // create a timer
  TPhEvent *event;

// Original file setup, replaced because it put the output file into 
// the same directory as the input files, usually read-only for the 
// user
/*
  Char_t PathName[150];
  Char_t FileName[200];
  Char_t EventName[20];

  const Char_t *phathome = gSystem->Getenv("PHATHOME");
  sprintf(PathName, "%s/data/",phathome); 
  // Prompt for file to be opened. Depending on navigation in
  // dialog the current working directory can be changed.
  // The returned file name is always with respect to the
  // current directory.
  const char *gOpenAsTypes[] = {
    "APX files",  "*.apx",
    "AOU files",   "*.aou",
    0, 0
  };

  static TGFileInfo fi;
  fi.fFileTypes = gOpenAsTypes;
  fi.fIniDir = PathName;

  if(!gROOT->IsBatch()) {
     new TGFileDialog(gClient->GetRoot(), gClient->GetRoot(), kFDOpen, &fi);
     strcpy(FileName, fi.fFilename);
  } else {
     sprintf(FileName, "%s/data/cv0_00000e1.apx",phathome);
  }
*/

// Options dialogue:

  const Int_t maxReply = 132;
  Int_t ReportLevel = 1+2; // Severe errors + summary
  enum platforms { bnl, ppcln, abacus };
  platforms platform = ppcln;
  Char_t rep[maxReply]; // Reply buffer
  Char_t fileName[maxReply+1]; // Used .apx filename
  Char_t fileNames[5][maxReply+1]; // List of .apx filenames
  Char_t outFileName[maxReply];
  if (platform == ppcln)
  { // 50 events:
    strncpy (fileNames[0], "/mnt/data02/sim/sim42/cv0_00000.apx", maxReply);
    strncpy (fileNames[1], "/mnt/data02/sim/sim42/cv0_1event.apx", maxReply);
    strncpy (fileNames[2], "/mnt/data02/sim/sim42/cv0_10events.apx", maxReply);
    strncpy (fileNames[3], "/mnt/phobos/sim/pro/scr/hij1_10apx.txt", maxReply);
    strncpy (fileNames[4], "/mnt/phobos/sim/pro/scr/hij2x12kbaou.apx",
       maxReply);
  }
  else // platform
  {
    strncpy (fileNames[0], "/data02/sim/sim42/cv0_00000.apx", maxReply);
    strncpy (fileNames[1], "/data02/sim/sim42/cv0_1event.apx", maxReply);
    strncpy (fileNames[2], "/data02/sim/sim42/cv0_10events.apx", maxReply);
    strncpy (fileNames[3], "/phobos/sim/pro/scr/hij1_10apx.txt", maxReply);
    strncpy (fileNames[4], "/phobos/sim/pro/scr/hij2x12kbaou.apx", maxReply);
  }
  strncpy (outFileName, "~/dat/events.root", maxReply);

// Get the name of the input .apx file
  
  printf ("Enter name of PMC .apx-format data file, or\n");
  printf (" 1 or CR = %s,\n", fileNames[0]);
  printf (" 2 = %s,\n", fileNames[1]);
  printf (" 3 = %s,\n", fileNames[2]);
  printf (" 4 = %s,\n", fileNames[3]);
  printf (" 5 = %s: ",  fileNames[4]);
  if (fgets (rep, maxReply, stdin))
  { Int_t iName = (rep[0] == '\n')? 0 : rep[0] - '1';
    if ((iName >= 0 && iName <= 4 && rep[1] == '\n') || rep[0] == '\n')
      strcpy (fileName, fileNames[iName]);
    else
    { strcpy (fileName, rep);
      fileName[strlen(rep)-1] = 0; // cut newline
    } 
  }
        
// Get the name for the ROOT-format output file:

  printf ("Enter name for root-format output file, or cr for\n %s: ",
     outFileName);
  if (fgets(rep, maxReply, stdin))
    if (rep[0] != '\n')
    { strcpy(outFileName, rep);
      outFileName[strlen(rep)-1] = 0;
    }

// Get number of events to skip initially and to process after that:

  Int_t nSkip = 0;  
  printf ("# of events to skip initially (0, 1, ...) [%d]: ", nSkip);
  if (fgets(rep, maxReply, stdin) && rep[0] != '\n')
  { rep[strlen(rep)-1] = 0;
    sscanf(rep, "%d", &nSkip);
  }

  Int_t nProc = -1;  
  printf ("# of events to process (cr = all): ");
  if (fgets(rep, maxReply, stdin) && rep[0] != '\n')
  { rep[strlen(rep)-1] = 0;
    sscanf(rep, "%d", &nProc);
  }

  if (ReportLevel >= 2) 
    printf ("Input file:  %s\nOutput file: %s\n", fileName, outFileName);

// End of options dialogue

  // Attention!, now pointer to TPhDetector object is known only after
  // first event has been converted (AOU header has been read).
  // So only then the pointer value is known.
  TPhDetector *detector;

  // TPhASCIIData has been tested to work with one file, 
  // then new objects should be created to handle next (if any) files
  TPhASCIIData *data = new TPhASCIIData(fileName);
  if( !data )
  { fprintf(stderr,
	    "AouToRoot: problems creating TPhASCIIData object - exiting.");
    exit(0);
  }

  TFile *out = new TFile(outFileName,"RECREATE");
  if( !out )
  { fprintf(stderr,
	    "AouToRoot: could not create %s output file - exiting.",
	    outFileName);
    exit(0);
  }

  int cnt = 0;
  TPhDataObject *cont;
  timer.Start();
  while( event = data->GetNextEvent() && (cnt < nSkip + nProc || nProc == -1))
  { if (cnt >= nSkip)
    { out.cd();
      // New mapping of electronics is used here to produce raw data container
      detector = TPhDetector::GetInstance();
      if( !detector ) {
        fprintf(stderr,
          "AouToRoot: could not obtain TPhDetector object - exiting.");
        exit(0);
      }
      detector->SetEvent(event);
      // Works for APX type files
      if(((TPhHashContainer *) 
	event->GetObject("MCHitArrays"))->GetEntries()) {
        //detector->MakeRawHitFECn0("MCHitArrays", "MCRawHitFECn0s");
      } 
      else if(((TPhHashContainer *) 
  	     event->GetObject("MCHits"))->GetEntries()) {
        // In case of AOU file the following two lines replace the above 
        detector->MakeHitArraysOutOfHits("MCHits","HitArraysFromHits");
        //detector->MakeRawHitFECn0("HitArraysFromHits", "MCRawHitFECn0s");
      } 
      // the MCHits and MCTracks containers can be deleted for faster 
      // access and smaller files
//       cont = (TPhDataObject *) event->GetObject("MCHits");          //  
//       event->RemoveObject(cont);                                    //   
//       delete cont;                                          // cleaning   
//       cont = (TPhDataObject *) event->GetObject("MCTracks");        //   
//       event->RemoveObject(cont);                                    //  
//       delete cont;                                                  // 
      ////////////////////////////////////////////////////////////////

      event->Write();
      if (ReportLevel >= 3)
      { event->Print("1");
        timer.Stop();
        timer.Print();
        timer.Continue();
      }
    }
    delete event;
    cnt++;
  } // while (

  timer.Stop();
  if (ReportLevel >= 2) timer.Print();
  
  detector->Write();
  data.Close();
  delete data;
  out.Close();
}

