{ // fitdept.c: script to fit single-track energy deposition 
// values using data from a root "dst" file.
// Use: run phat, then 
// % .x fitdept.C
// and follow prompts.
// Status: Fits fail on low statistics; need to use with grouped
// displays and take function errors into account.
// 31-Aug-98: Exit correctly on end of data file
// 23-Aug-98: Get vertex from event (RV)
// 21-Aug-98: Robin Verdier
//

  gROOT->Reset();

  Int_t   Nprint = 400;
  Int_t   ReportLevel = 5; // Errors and diagnostics
  Int_t   MaxReply = 100;

  Char_t  DSTFile[MaxReply+1]; // +1 for terminal null
  Char_t *DSTFiles[5] =
  { "~verdier/mul/dat/events1_10.root",
    "/mnt/phobos/output/sim/events1_10.root",
    "/mnt/data04/phobos/olszewsk/exports/iv0_200_all_containers.root",
    "/mnt/data04/phobos/olszewsk/exports/cvr_200_all_containers.root",
    "/mnt/data04/phobos/olszewsk/exports/cv0_50_all_containers.root"
  };
  Int_t    DataSource = 2; // 1 for HitArrays, 2 for full MC hits
  Char_t  *EOLKeys[] = { "\n" };
  Char_t   inFile[MaxReply];
  Char_t   modFile[MaxReply];
  Bool_t   oldData; // set after DSTFile is specified
  Char_t   outFile[MaxReply];
  Char_t   prevText[20];
  Char_t   reply[MaxReply];
  Char_t   text[20];
  Char_t  *TextKeys[] = { "// MO", "// V" };
  Char_t  *vertexName = "MainVertex";
  Bool_t   NewNames = kFALSE;
  Char_t  *HitArrayName = "HitArraysFromHits";

// The following should be a dimensionless fraction:
  Float_t  xExtMax[2] = {-0.5, +2.0}; // Maximum extrapolation distances
  Float_t  norm = 1.0;

// const // This makes CInt fail on re-entry
  Int_t  eNumb = 1; // CInt can't handle this enum in TPHOpts.h
  Int_t  idFound = -1;
  Int_t  idLen = 4; // "// MOB3"
  Int_t  iErr = 16;
  Int_t  iprint = 0;
  Int_t  iVal = 0;
  Int_t  iVar = 15; // E1 per track, MC
  Int_t  MaxPars = 10;
  Int_t  nCallLim = 100000;
  Int_t  nCalls = 0;
// Note if the following is called, more accurately, nFitPars, CInt
// fails on "reinitialization of a constant".
// const
  Int_t  nPars = 3;
  Int_t  nPoints; // # of points in gr
  Int_t  nVars = 0; // # read for this text key
  Int_t  status = 0;
  Int_t  sum = 0; // flag for summing in phi
  Int_t  targets = 1 + 2; // Text and numeric fields

  TPhDetector  *det;
  TPhPadMap    *disp;
  TPhDST       *dst;
  TPhEvent     *ev;
  TGraphErrors *gr;
  TMatrix       mX (nPars, nPars); // Fit point matrix
  TVector       vP (nPars); // Coefficients for fit
  TVector       vX (nPars); // Fit points
  TVector       vY (nPars); // Fit data

  prevText[0] = char(0); // null

// Create the detector

  if ((det = TPhDetector::GetInstance()) == NULL)
    det = TPhDetector::ReadLatestGeometry();

// Allow use of a previously instantiated and modified multiplicity processor

  printf ("Enter cr to create a new TPhMul processor, anything to use an\n"
    " existing one created with 'TPhMul mp'? ");
  if (fgets(reply, MaxReply, stdin))
    if (reply[0] == '\n') TPhMul mp;

// Get name of data file, or space for diagnostic mode

  if (!dst)
// Get the name of the DST file.
  { printf ("Enter name of root-format DST file, space for none, or\n");
    printf (" 1 or CR = %s,\n", DSTFiles[0]);
    printf (" 2 = %s,\n", DSTFiles[1]);
    printf (" 3 = %s,\n", DSTFiles[2]);
    printf (" 4 = %s, or \n", DSTFiles[3]);
    printf (" 5 = %s:\n", DSTFiles[4]);
    if (fgets (reply, MaxReply, stdin))
    { reply[strlen(reply)-1] = 0; // remove '\n'
      if (!strcmp(reply, "1") || !strcmp(reply,""))
        strncpy (DSTFile, DSTFiles[0], MaxReply);
      else if (!strcmp(reply, "2")) strncpy (DSTFile, DSTFiles[1], MaxReply);
      else if (!strcmp(reply, "3")) strncpy (DSTFile, DSTFiles[2], MaxReply);
      else if (!strcmp(reply, "4")) strncpy (DSTFile, DSTFiles[3], MaxReply);
      else if (!strcmp(reply, "5")) strncpy (DSTFile, DSTFiles[4], MaxReply);
      else if (!strcmp(DSTFile, " ")) strcpy (DSTFile, "");
      else strncpy (DSTFile, reply, MaxReply);
    }
// KLUDGES:
// (1) Use old name for old data hitarrays
    oldData = strcmp(reply, "2") <= 0;
    if (oldData) HitArrayName = "MCHitArrays";
// (2) Skip vertexing for the new data.
//    if (!oldData) vertexName = "";

    if (strcmp(DSTFile, ""))
    { if (ReportLevel >= 8) printf ("Reading %s:\n", DSTFile);
      dst = new TPhDST(DSTFile);
    }
    else
// Create an empty event to satisfy calls in diagnostic mode with
// fDataSource = EDiagnostic (3)
      ev = new TPhEvent();

    Int_t nEvents = 999999; // This should be generalized to use -1
    printf ("Number of events to process (cr = all): ");
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] != '\n') sscanf (reply, "%d", &nEvents);

    Int_t nEvSkip = 0;
    printf ("Number of events to skip initially (cr = %d): ", nEvSkip);
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] != '\n') sscanf (reply, "%d", &nEvSkip);

    strncpy (modFile, "setmul.C", MaxReply);
    printf ("Startup script filename (cr for none, 1 = %s)\n: ", modFile);
    if (fgets(reply, MaxReply, stdin))
    { reply[strlen(reply)-1] = 0; // Kill \n
      if (strcmp(reply, "1"))
        strncpy(modFile, reply, MaxReply);
    }

// The following would need a system call to translate $PHATHOME
//    strncpy (inFile, "$PHATHOME/phatmul/inc/TPhDeptV.h", MaxReply);
    strncpy (inFile, "../inc/TPhDeptV.h", MaxReply);
    printf ("Input template file name (cr for none, 1 = %s)\n: ", inFile);
    if (fgets(reply, MaxReply, stdin))
      if (reply[0] == '\n') strcpy (inFile, "");
      else if (strcmp(reply, "1\n"))
      { strncpy(inFile, reply, MaxReply);
        inFile[strlen(reply)-1] = 0;
      }

    Bool_t fit = strcmp(inFile, "") != 0;
    if (fit)
    { strncpy (outFile, "newTempV.h", MaxReply);
      printf ("Output template file name (space for none, cr = %s)\n: ",
        outFile);
      if (fgets(reply, MaxReply, stdin))
        if (reply[0] != '\n')
        { strncpy(outFile, reply, MaxReply);
          outFile[strlen(reply)-1] = 0;
          if (strncmp(outFile, " ", MaxReply) == 0) strcpy (outFile, "");
        }
      printf ("Variable index to fit, 15 = dE, 19 = fSec [cr = %d]? ", iVar);
      if (fgets (reply, MaxReply, stdin))
        if (reply[0] != '\n')
        { reply[strlen(reply)-1] = char(0);
          sscanf (reply, "%d", &iVar);
        }
    }

    if (fit)
// Create options class for access to data templates
    { TPhOpts op (inFile, outFile);
      op.SetEOLKeys (0, 1, EOLKeys); // Default
      op.SetTextKeys (2, TextKeys);
      op.ReportLevel = 1;
    }

// Loop over events in the file.  Use Phat's event numbering 0, 1, ...
// to avoid confusion.
    for (Int_t iev = 0; iev < nEvents + nEvSkip; iev++)
    { if (dst)
      { if (ReportLevel >= 2) printf ("Reading event %d...\n", iev);
// Delete previous event if it exists
        if (ev) delete ev;
        ev = dst->GetNextEvent();
        if (ev)
        { det->SetEvent(ev);
          det->FillWithEventHitArrays(HitArrayName);
          if (strcmp(vertexName, ""))
          { ev->FindVertex();
            TPhVertex *v1 = (TPhVertex *) ev->GetObject(vertexName);
            if (v1)
            { mp.SetVertex(v1);
              if (ReportLevel >= 3) 
                printf ("Found vertex Z = %6.2f cm.\n", v1->GetZ());
            }
          }
        } // if (ev
      } // if (dst
      if (ev && iev >= nEvSkip)
      { if (ReportLevel > 2) printf ("Reconstructing multiplicity...\n");
// Reconstruct multiplicity if not already done)
        if (iev == nEvSkip && (oldData || strcmp(modFile, "")))
// Optionally modify the multiplicity processor after initialization.
// The modifier file should refer to the TPhMul object through the pointer mp.
        { mp.Process(ev, -1);
          mp.fDataSource = DataSource;
          if (oldData) mp.fHitArrayId = HitArrayName;
          gROOT->Macro(modFile);
          mp.Process(ev, 1);
        }
        else
          mp.Process(ev);
      }
      if (!ev && ReportLevel >= 2)
      { printf ("EOF found after %d events.\n", iev);
        break;
      }
    }
  }

  if (fit)
  { printf ("Fitting energy deposition parameters...\n");
    while (nCalls++ < nCallLim && status >= 0)
    { status = op.Find(targets);
      if (op.fLine == 123456) ReportLevel = 11; // SPECIAL TESTS
      if (status == targets)
      { strncpy(text, op.fFields[0]+3, idLen);
        text[idLen] = char(0); // Make the name a string
// We need a way of dealing with multi-event displays named, say, MOA*-N.
// The next version will match nChMatch characters at offset nChSkip,
// in this case 3 and 3 but for full match 4 and 3.  Using Select and
// indexing backwards over displays should handle most cases.
// This script may need an interactive interface.

// If this is a new identifier, delete old PadMap if any and get new one

        if (strcmp(text, prevText) != 0)
        { if (disp) disp->Delete();
          disp = mp.GetDisplay(text);
// Get fit from sensor SSS** if no display SSSn is available.
          if (!disp)
          { strcpy (prevText, text); 
            prevText[idLen-1] = '*';
            disp = mp.GetDisplay(prevText);
          }
          if (ReportLevel >= 4 && iprint < Nprint) 
          { if (disp)
              printf ("New key %s in line %d matches display %s.\n", text, 
                op.fLine, disp->GetName());
            else
            { printf ("New key %s in line %d matches no display; active "
                "displays are\n", text, op.fLine);
              mp.ListDisplays();
            }
            iprint++;
          }
          strcpy (prevText, text);
          nVars = 0;

// Float_t GetYFit (TPhPadMap &disp, Int_t iVar...
// Get the information from a TGraphErrors created by the PadMap.
// This allows us to use its standard methods for displaying, fitting,
// ... the selected data, optionally integrated on phi.  A similar 
// interface, GetHist, provides access to azimuthal dependences.
          gr = NULL;
          if (disp)
          { gr = disp->GetGraph(mp.GetDisplays(), "d1", text, iVar, iErr, 
              norm, -1, -1, 0, 0, 0, 0);
            if (gr && ReportLevel >= 10 && iprint < Nprint)
            { gr->Print();
              iprint++;
            }
          }
        } // != prevText
        if (!(nVars & 1))
        { Float_t dept = op.fNumValue;
// Save the location of dept for possible replacement by a fitted velue.
          Int_t DecOffSave = op.fDecOffs;
          Int_t ExpOffSave = op.fExpOffs;
          Int_t LengthSave = op.fLengths[eNumb];
          Int_t NumOffSave = op.fOffsets[eNumb];
        }
        else
        { Float_t eta = op.fNumValue;
// Get a fit value from nPars non-zero data values closest to eta.
// This will become a method of TPhPadMap or TPhOpts with arguments 
// gr, nPars, and xFit.
          Float_t xFit = eta;
          Float_t yFit;
          Float_t x, y;
          Int_t nParUse;
          nPoints = gr->GetN();
// Find the closest set of nPars non-zero data points to use in fit
// The zero data points are present only in octagon layers with cutouts
// such as MOA*, MOC*, ...  A better way to handle this might be to
// make displays MOA0-MOA5, say; but this needs a non-trivial extension
// of TPhPadMap::Select.
          if (ReportLevel >= 7 && iprint < Nprint) 
          { printf ("FitDept: data ");
            for (int ir = 0; ir < nPoints; ir++)
            { gr->GetPoint (ir, x, y);
              printf (" (%10.3e at %5.2f)", y, x);
            }
            printf ("\n");
            iprint++;
          }
          Float_t dmin = 0.;
          Int_t iCent = -1;
          Int_t id;
// The following fails later to allow iDataUse[1] under CInt. **?
//          Int_t iDataUse[nPars];
//          Int_t iDataUse[MaxPars];
          Int_t iDataUse[10];
          Float_t yRejMin = 0;
          Float_t yRejMax = 1.0e10;
          for (id = 0; id < nPoints; id++)
          { gr->GetPoint(id, x, y);
// TMath function Abs requires TMath:: here but not in the 
// standard ROOT environment.  To be understood.
          Float_t dx = TMath::Abs(xFit - x);
          if ( y > yRejMin && y < yRejMax && (iCent < 0 || dx < dmin))
          { iCent = id;
            dmin = dx;
          }
        }
// Find nPars non-zero data points surrounding iCent. This will become
// Find the nPars non-zero data points whose center-of-gravity is closest
// to xFit.
// => This must be completed.
        id = iCent - nPars / 2;
        if (id > nPoints - nPars) id = nPoints - nPars;
        if (id < 0) id = 0;
        nParUse = 0;
        for (Int_t ip = 0; ip < nPars; ip++)
        { Int_t nParStart = nParUse;
          while (id < nPoints && nParUse == nParStart)
          { gr->GetPoint(id, x, y);
            if (y > yRejMin && y < yRejMax)
            { iDataUse[ip] = id;
              nParUse++;
            }
            id++; 
          }
        }
        if (ReportLevel >= 6 && iprint < Nprint) 
        { printf ("FitDept: for fit at %5.2f, use data", xFit);
          for (int ir = 0; ir < nParUse; ir++)
          { gr->GetPoint (iDataUse[ir], x, y);
            printf (" (%10.3e at %5.2f)", y, x);
          }
          printf ("\n");
          iprint++;
        }
// Create the (nPars x nPars) matrix mX[r,c] == x[r]**c
// and the corresponding nPars-dimensional vector vY[r] = y[r], 
// where r actually indexes over non-zero data points in iDataUse.
// The parameter vector vP is the solution of mX * vP = vY and
// the fitted function at x = vP[0] + vP[1] * x + vP[2] * x * x
        for (int ir = 0; ir < nParUse; ir++)
        { gr->GetPoint (iDataUse[ir], x, y); 
          vX(ir) = x;
          vY(ir) = y;
          for (int ic = 0; ic < nParUse; ic++)
            mX(ir, ic) = (ic)? x * mX(ir, ic-1) : 1.0;
        }
        if (ReportLevel >= 7 && iprint < Nprint) 
        { printf ("vY =");
          for (int nn = 0; nn < nParUse; nn++) printf ("%10.3e", vY(nn));
          printf ("\n");
          printf ("mX =");
          for (int nn = 0; nn < nParUse; nn++)
            for (int mm = 0; mm < nParUse; mm++) 
              printf ("%8.3f", mX(nn,mm));
          printf ("\n");
          iprint++;
        }
        vP = vY;
        mX.Invert();
        vP *= mX;
        if (ReportLevel >= 7 && iprint < Nprint) 
        { printf ("vP =");
          for (int nn=0; nn<3; nn++) printf ("%10.3e", vP(nn)); printf ("\n");
          printf ("1/mX =");
          for (int nn=0; nn<3; nn++) for (int mm=0; mm<3; mm++) 
          printf ("%8.3f", mX(nn,mm)); printf ("\n");
          iprint++;
        }
// Test the fit function by generating it at each used data point
        if (ReportLevel >= 8 && iprint < Nprint) 
        { printf ("(Fit : Data) =");
          for (int ic = 0; ic < nParUse; ic++)
          { gr->GetPoint (iDataUse[ic], x, y); 
            yFit = vP(nParUse-1);
            for (int ip = nParUse - 2; ip >= 0; ip--)
              yFit = yFit * x + vP(ip);
            printf ("(%10.3e : %10.3e) ", yFit, y);
          }
          printf ("\n");
          iprint++;
	}
// Evaluate yFit at xFit
        yFit = vP(nParUse-1);
        for (int ip = nParUse - 2; ip >= 0; ip--)
          yFit = yFit * xFit + vP(ip);
// Reject the fit if it's being extrapolated too far or if the result is
// out of bounds.  This selection will be generalized.
          Int_t fitQual = 0; 
          if (xFit < vX(0) - xExtMax[0]) fitQual |= 1;
          if (xFit > vX(nParUse-1) + xExtMax[1]) fitQual |=2;
          if (yFit < yRejMin) fitQual |=4;
          if (yFit > yRejMax) fitQual |=8;
          if (ReportLevel >= 5 && iprint < Nprint)
          { printf ("Fitdept line %d, var %d, eta = %5.2f: dE = %10.3e,"
            " fit = %10.3e",
              op.fLine, op.fLineFields[1] / 2, xFit, dept, yFit);
            if (fitQual) printf (" f=%d\n", fitQual);
            else printf ("\n");
            iprint++;
          }
// At this point, if there's an output file and the fit was successful,
// replace the original y by yFit.
// This pointer save and recall should be a TPhOpts method.
// This code can fail when the line is overwritten by a wrong-length fit,
// probably an unanticipated prefix - sign.  ==> FIX.
          if (strcmp(outFile, "") && !fitQual)
          { Int_t DecOffSave2 = op.fDecOffs;
            Int_t ExpOffSave2 = op.fExpOffs;
            Int_t LengthSave2 = op.fLengths[eNumb];
            Int_t NumOffSave2 = op.fOffsets[eNumb];
            op.fDecOffs = DecOffSave;
            op.fExpOffs = ExpOffSave;
            op.fLengths[eNumb] = LengthSave;
            op.fOffsets[eNumb] = NumOffSave;
            op.SetField(yFit);
            op.fDecOffs = DecOffSave2;
            op.fExpOffs = ExpOffSave2;
            op.fLengths[eNumb] = LengthSave2;
            op.fOffsets[eNumb] = NumOffSave2;
          }
        } // if (!(nVars & 1))
      nVars++;
      } // status == targets
      else
        printf ("Returned status %d.\n", status);
    } // while (nCalls
  } // if (strncmp
  if (fit && disp) disp->Delete();
  if (fit && op) op->Delete();
  exit;
}
