{
// setmul.C: root macro for setting multiplicity reconstruction
// parameters
// Use: respond to fitdept.C question with name of this macro
// Notes: if you misspell a variable name, CInt will give a segmentation
// violation and may never recover.
// CInt ignores the execution logic and evaluates all the code in its 
// first pass, thus failing on syntax errors even if they are never
// executed.
// 3-Nov-98: Tune resets
// 12-Aug-98: Robin Verdier

  Bool_t combineRaw  = kFALSE; // This will produce biased displays!
  Bool_t createMul   = kFALSE;
  Bool_t diagData    = kFALSE; // true to use dummy diagnostic data
  Bool_t flipPhi     = kFALSE;
  Bool_t listBadIndices = kFALSE;
  Bool_t makeDiags   = kFALSE; // true for diagnostic dE/dx histos etc
  Bool_t makeTrackSum = kFALSE; // true for MC Track and hit summaries
  Bool_t resetVertex = kFALSE;
  Bool_t resetEdep   = kFALSE;
  Bool_t resetFsec   = kFALSE;
  Bool_t traceTracks = kFALSE;
  Bool_t useMCTracks = kTRUE;

  Char_t *macroName = "setmul.C";

  if (combineRaw)
  { TPhPadMap *pm;
    for (Int_t is = 0; is < 164; is++) // mp.kNSensMax; is++)
    { if ((pm = mp.GetSensMap(is))) pm->SetMaxEvents(999);
    }
    printf ("%s set all sensor maps to accumulate all events.\n", macroName);
    printf ("MaxEvents for MOB6 = %d\n", 
      mp.GetSensMap("MOB6")->GetMaxEvents());
  }

  if (createMul)
  { TPhMul mp;
    printf ("%s created TPhMul mp.\n", macroName);
  }

  if (diagData)
  { mp.fDataSource = 3;
    printf ("USING DUMMY DIAGNOSTIC INPUT DATA.\N");
  }

  if (flipPhi)
  { for (Int_t is = 0; is < 164; is++)
    { TPhPadMap *mm = mp.GetSensMap(is);
      if (!strncmp(mm.GetName(), "MOA", 3) ||
          !strncmp(mm.GetName(), "MOB", 3) ||
          !strncmp(mm.GetName(), "MOC", 3) ||
          !strncmp(mm.GetName(), "MOD", 3) ||
          !strncmp(mm.GetName(), "VBA", 3))
      { Int_t iY = 1;
        mm->SetMin(iY, mm->GetMin(iY) + mm->GetNCells(iY) * mm->GetSize(iY));
        mm->SetSize(iY, - mm->GetSize(iY));
      }
    }
  }

  if (listBadIndices)
  { mp.fMaxBadIndexMsgs = 5; // 200;
    mp.RLNPrint = 300;
  }

  if (makeDiags)
  { mp.fGDNDiagBins[0] = 50;
    mp.fGDNDiagBins[1] = 50;
    mp.fGDNDiagBins[2] = 50;
    mp.fGDNDiagBins[3] = 100;
    mp.fGDNDiagBins[4] = 100;
    printf ("%s turned on diagnostics 0 - 4.\n", macroName);
  }

  if (makeTrackSum)  mp.RLGetData = 3; // add track and hit count summaries

  if (traceTracks)
  { mp.fGDTrackTrace = 39; // 500;
    mp.fGDNTracksTrace = 20; // 200;
    printf ("%s Tracing %d tracks starting with track %d.\n", macroName,
      mp.fGDNTracksTrace, mp.fGDTrackTrace); 
    mp.fNTracksUse = 59; // 700;
    mp.RLNPrint = 300;
    mp.RLGetData = 3;
  }

  if (resetEdep)
  { Float_t newEdep[5][7][2] =     // for MN, MO, MP, VB, VT
    {{{110.0e-6,-6.0},  {108.0e-6,-4.0},  {105.0e-6,-2.0},  {110.0e-6,0.0},
      {112.0e-6, 2.0},  {109.0e-6, 4.0},  {110.0e-6, 6.0}},
     {{ 76.0e-6,-3.6},  {112.0e-6,-2.56}, {132.0e-6,-0.4},  {126.0e-6,0.9},
      {127.0e-6, 1.16}, {108.0e-6, 2.72}, { 55.0e-6, 3.6}},
     {{110.0e-6,-6.0},  {108.0e-6,-4.0},  {105.0e-6,-2.0},  {110.0e-6,0.0},
      {112.0e-6, 2.0},  {109.0e-6, 4.0},  {112.0e-6, 6.0}},
     {{ 65.0e-6,-2.4},  {130.0e-6,-1.36}, {148.0e-6,-0.6},  {158.0e-6,0.2},
      {154.0e-6, 0.82}, {124.0e-6, 1.24}, {145.0e-6, 2.4}},
     {{118.0e-6,-2.4},  {125.0e-6,-1.40}, {166.0e-6, 0.0},  {148.0e-6,0.4},
      {153.0e-6, 0.65}, {127.0e-6, 1.20}, {134.0e-6, 2.4}}
    };

// The following three lines should be redundant; but if they are not
// present, CINT will actually execute some of the following code even
// if resetEdep is false.
  }
  if (resetEdep)
  { 

//    struct deptVals { Char_t *firstSens; Char_t *lastSens; Float_t e0;
//		     Float_t dEdEtaAbs; };
//    deptVals dv[2] = {{ "VBA0", "VBA3", 145.0e-6, -32.0e-6 },
//                      { "VTA0", "VTA3", 145.0e-6, -57.0e-6 }};
//    Char_t *Names[2] = {"MOA0", "MOHC"};

    printf ("%s resetting dE/hit values.\n", macroName);
    Int_t iSetMax = 1;
    for (Int_t iSet = 0; iSet < iSetMax; iSet++)
    { //Int_t iSensMin = mp.GetSensMapIndex(dv[iSet].firstSens);
      //Int_t iSensMax = mp.GetSensMapIndex(dv[iSet].lastSens);
      Int_t iSensMin = 0; // mp.GetSensMapIndex(Names[0]);
      Int_t iSensMax = 163; // mp.GetSensMapIndex(Names[1]);
      for (Int_t iSens = iSensMin; iSens <= iSensMax; iSens++)
      { Char_t *name = mp.GetSensMap(iSens)->GetName();
        TPhDept *dept = mp.GetDeptP(0);
        Int_t nPars = dept->GetNPars(0);
        for (Int_t nn = 0; nn < nPars; nn++)
	{
//         Float_t absEta = TMath::Abs(dept->GetPar(iSens, 2*nn+1));
//         Float_t Eta = dept->GetPar(iSens, 2*nn+1);
//         Float_t dENew; // = dv[iSet].e0 + dv[iSet].dEdEtaAbs * absEta;
//          Float_t absEta = (Eta >= 0)? Eta : -Eta;
//          // dENew = dv[iSet].e0 + dv[iSet].dEdEtaAbs * absEta;
//          if (!strncmp(name, "MO", 2)) dENew = (Eta >= 0)?
//            135.0e-6 - 11.0e-6 * Eta : 135.0e-6 + 9.0e-6 * Eta;
//          if (name[0] == 'V') dENew = (Eta >= 0)?
//            164.0e-6 - 22.0e-6 * Eta : 175.0e-6 + 36.0e-6 * Eta;
//          if (name[0] == 'M' && name[1] != 'O') dENew = (Eta >= 0)?
//            119.0e-6 - 2.2e-6 * Eta : 119.0e-6 + 2.2e-6 * Eta;

          Int_t iNew = 1;
          if (!strncmp(name, "MN", 2)) iNew = 0;
          if (!strncmp(name, "MP", 2)) iNew = 2;
          if (!strncmp(name, "VB", 2)) iNew = 3;
          if (!strncmp(name, "VT", 2)) iNew = 4;
          dept->SetPar(iSens, 2*nn, newEdep[iNew][nn][0]);
          dept->SetPar(iSens, 2*nn+1, newEdep[iNew][nn][1]);
          if (iSens == 24 || iSens == 160)
            printf ("%s: set dE/track for %s at Eta = %f to %f\n", 
              macroName, mp.GetSensMap(iSens)->GetName(), 
              dept->GetPar(iSens, 2*nn+1), dept->GetPar(iSens, 2*nn));
        } // for (Int_t nn
      } // for (Int_t iSens
    } // for (Int_t iSet
  } // if (resetEdep

  if (resetFsec)
  { printf ("%s resetting Fsec values.\n", macroName);
    Int_t iSetMax = 1;
    Float_t NewFsec[5][7][2] =     // for MN, MO, MP, VB, VT
    {{{ 2.15, -6.00 }, { 1.55, -4.90 }, { 1.60, -4.50 }, { 1.44, -4.30},
      { 1.38, -3.90 }, { 1.24, -3.40 }, { 2.10, -2.50 }},
     {{ 2.60, -4.00 }, { 1.53, -2.84 }, { 1.15, -2.00 }, { 1.12, 1.00 },
      { 1.30,  2.24 }, { 1.71,  2.84 }, { 3.80,  3.60 }},
     {{ 1.87,  2.50 }, { 1.34,  3.30 }, { 1.32,  3.70 }, { 1.53, 4.55 },
      { 1.48,  4.72 }, { 1.83,  5.50 }, { 1.83,  6.00 }},
     {{ 1.36, -2.40 }, { 1.15, -1.16 }, { 1.24, -0.90 }, { 1.16, 0.00 },
      { 1.22,  0.88 }, { 1.13,  1.26 }, { 1.55,  2.40 }},
     {{ 1.25, -2.40 }, { 1.20, -0.90 }, { 1.20, -0.10 }, { 1.17, 0.12 },
      { 1.22,  0.90 }, { 1.16,  1.20 }, { 1.18,  2.40 }}
    };
  }
  if (resetFsec)
  { for (Int_t iSet = 0; iSet < iSetMax; iSet++)
    { Int_t iSensMin = 0;
      Int_t iSensMax = 163;
      for (Int_t iSens = iSensMin; iSens <= iSensMax; iSens++)
      { Char_t *name = mp.GetSensMap(iSens)->GetName();
//*** TEST:: deactivate most sensors
//        TPhPadMap *sens = mp.GetSensMap(iSens);
//        if (strcmp(name,"MOBB") && strcmp(name,"MOBC")) sens->SetId(0);
        TPhFsec *fsec = mp.GetFsecP(0);
        Int_t nPars = fsec->GetNPars(0);
        for (Int_t nn = 0; nn < nPars; nn++)
	{ Int_t iNew = 1;
          if (!strncmp(name, "MN", 2)) iNew = 0;
          if (!strncmp(name, "MP", 2)) iNew = 2;
          if (!strncmp(name, "VB", 2)) iNew = 3;
          if (!strncmp(name, "VT", 2)) iNew = 4;
          for (Int_t kk = 0; kk < 2; kk++)
            fsec->SetPar(iSens, 2*nn+kk, NewFsec[iNew][nn][kk]);
          if (iSens == 24 || iSens == 160)
          printf ("%s: set Fsec for %s at %f to %f\n", macroName,
              name, fsec->GetPar(iSens, 2*nn+1), fsec->GetPar(iSens, 2*nn));
        } // for (Int_t nn
      } // for (Int_t iSens
    } // for (Int_t iSet
  } // if (resetFsec

  if (resetVertex)
  { TPhVertex *v0 = new TPhVertex("v0");
    v0->SetZ(15);
    mp.SetVertex(v0);
    printf ("%s: set interaction vertex to z = %f cm.\n", macroName,
      mp.GetVertex()->GetZ());
  }

  if (useMCTracks)
  { mp.fDataSource = 2; // 1 = EMCData, 2 = EMCHits
//    mp.RLGetData = 5;
    printf ("%s: reconstruction will use data from MC Tracks.\n", macroName);
  }
}
