#include "TAxis.h"
#include "TCanvas.h"
#include "TGraphErrors.h"
#include "TH1.h"
#include "TH2.h"
#include "TMarker.h"
#include "TObject.h"
#include "TPaveLabel.h"
#include "TROOT.h"
#include "TPhMapSet.h"
#include "TPhMul.h"
#include "TPhMulDisp.h"
#include "TPhPadMap.h"
ClassImp(TPhMulDisp)
////////////////////////////////////////////////////////////////////////////
//
// TPhMulDisp displays raw and reconstructed physics data produced by TPhMul.
//
// File: TPhMulDisp.cxx
//
// Date     Vers  Description
// 29-Jan-99 2.0  Add new constructors and Draw methods (RV)
//
// $id$
//
////////////////////////////////////////////////////////////////////////////

// 18-Jan-99 1.0  Fix erroneous warning; x-Delete -> delete x (RV)
// 15-Jan-99 1.0  Update GetGraph and GetHist calls (RV)
// 16-Dec-98 1.0  Remove unused variables (RV)
// 13-Nov-98 1.0  Create labels on heap (RV)
// 09-Oct-98 1.0  Complete revision (RV)
// 03-Dec-97 0.1  Creation (Robin Verdier)

TPhMulDisp::TPhMulDisp() {}

TPhMulDisp::TPhMulDisp  // Constructor for data from a TPhMapSet
( TPhMapSet *ms,        // TPhMapSet containing TPhPadMaps to be plotted
  Char_t *sel,          // Selector string for display, e.g. "*+in"
  Int_t   iMap,         // If >= 0, override selector and use display or 
                        // sensor TPhPadMap with index iMap as shown by 
                        // mp.ListDisplays() or mp.ListSensors()
  Int_t   var,          // TPhPadMap variable index to plot, 0, 1, ...
                        // Indices and numbers of variables in the sensor and
                        // display padmaps are enumerated in TPhMul.h.
  Int_t   varRMS,       // Error variable index if >= 0, else none
  Float_t scale,        // Total coverage <= 360 if integrating on phi
  Int_t   grHist,       // 0 for TGraphErrors, 1 for TH2F
  Int_t   diSens,       // 0 for display map, 1 for sensor map
  Char_t *xLabel,       // X axis text
  Char_t *yLabel,       // Y axis label, e.g. "dNch/dEta"
  Char_t *key,          // Key text, e.g. "Reconstructed"
  Int_t   mStyle,       // Marker style, usually 20, 21, ...
  Int_t   mColor,       // Marker color: 0-12 for none, black, red, green,
                        //   blue, yellow, magenta, cyan, green, blue,
                        //   white, gray, black.  Others may be possible.
  Float_t mSize         // Marker size, usually 0.5 - 1.0
)
{ fSource = 0;
  fPadMap = 0;
  fMapSet = ms;
//  fMul = 0; // (TPhMul &)0; fails
  fSel = sel;
  fIMap = iMap;
  SetDefaults (var, varRMS, scale, grHist, diSens, xLabel, yLabel, key, 
    mStyle, mColor, mSize);
}

TPhMulDisp::TPhMulDisp  // Constructor for data from a TPhPadMap
( TPhPadMap *pm,        // TPhPadMap to be plotted
                        // Other variables are as in the first constructor:
  Int_t   var,          // TPhPadMap variable index to plot, 0, 1, ...
                        // Indices and numbers of variables in the sensor and
                        // display padmaps are enumerated in TPhMul.h.
  Int_t   varRMS,       // Error variable index if >= 0, else none
  Float_t scale,        // Total coverage <= 360 if integrating on phi
  Int_t   grHist,       // 0 for TGraphErrors, 1 for TH2F
  Int_t   diSens,       // 0 for display map, 1 for sensor map
  Char_t *xLabel,       // X axis text
  Char_t *yLabel,       // Y axis label, e.g. "dNch/dEta"
  Char_t *key,          // Key text, e.g. "Reconstructed"
  Int_t   mStyle,       // Marker style, usually 20, 21, ...
  Int_t   mColor,       // Marker color: 0-12 for none, black, red, green,
                        //   blue, yellow, magenta, cyan, green, blue,
                        //   white, gray, black.  Others may be possible.
  Float_t mSize         // Marker size, usually 0.5 - 1.0
  )
{ fSource = 1;
  fPadMap = pm;
  fMapSet = 0;
//  fMul = fMul * 0; // (TPhMul &)0; fails
  fSel = 0;
  fIMap = -1;
  SetDefaults (var, varRMS, scale, grHist, diSens, xLabel, yLabel, key, 
    mStyle, mColor, mSize);
}

TPhMulDisp::TPhMulDisp  // Constructor for data from a TPhMul
( TPhMul &mp,           // TPhMul instance containing TPhPadMaps to be plotted
  Char_t *sel,          // Selector string for display, e.g. "*+in"
  Int_t   iMap,         // If >= 0, override selector and use display or 
                        // sensor TPhPadMap with index iMap as shown by 
                        // mp.ListDisplays() or mp.ListSensors()
  Int_t   var,          // TPhPadMap variable index to plot, 0, 1, ...
                        // Indices and numbers of variables in the sensor and
                        // display padmaps are enumerated in TPhMul.h.
  Int_t   varRMS,       // Error variable index if >= 0, else none
  Float_t scale,        // Total coverage <= 360 if integrating on phi
  Int_t   grHist,       // 0 for TGraphErrors, 1 for TH2F
  Int_t   diSens,       // 0 for display map, 1 for sensor map
  Char_t *xLabel,       // X axis text
  Char_t *yLabel,       // Y axis label, e.g. "dNch/dEta"
  Char_t *key,          // Key text, e.g. "Reconstructed"
  Int_t   mStyle,       // Marker style, usually 20, 21, ...
  Int_t   mColor,       // Marker color: 0-12 for none, black, red, green,
                        //   blue, yellow, magenta, cyan, green, blue,
                        //   white, gray, black.  Others may be possible.
  Float_t mSize         // Marker size, usually 0.5 - 1.0
)
{ fSource = 2;
  fPadMap = 0;
  fMapSet = 0;
  fMul = mp;
  fSel = sel;
  fIMap = iMap;
  SetDefaults (var, varRMS, scale, grHist, diSens, xLabel, yLabel, key, 
    mStyle, mColor, mSize);
}

TPhMulDisp::~TPhMulDisp() {}

void TPhMulDisp::Delete()
{ if (fTG) delete fTG;
  for (Int_t nn = 0; nn < EMaxPlots; nn++)
  { if (fTM[nn]) delete fTM[nn];
    if (fTP[nn]) delete fTP[nn];
  }
  if (fTPDa) delete fTPDa;
  if (fTGName) delete[] fTGName;
  fTGName = 0;
  if (fLegend) delete[] fLegend;
  fLegend = 0;
}

void TPhMulDisp::Draw(Int_t var, Int_t varRMS, Float_t scale, Char_t *key)
{ if (var >= 0) fVar = var;
  if (varRMS >= -1) fVarRMS = varRMS;
  if (scale >= 0) fScale = scale;
  if (key) fKey = key;

// Limit fSeq to avoid overwrites

  if (fSeq >= EMaxPlots)
  { if (fReportLevel) printf ("TPhMulDisp/Draw: fSeq reduced from %d to"
      " %d to avoid overwriting.\nRecompile with increased EMaxPlots if"
      " necessary.\n", fSeq, EMaxPlots);
    fSeq = EMaxPlots - 1;
  }

// Create character string for ROOT id on the heap unless updating  

  Float_t keyLim[2][2];
  if (!fNoNew)
  { Int_t keyLen = strlen(fSel);
    Int_t idLen = keyLen + strlen("de___") + 2 + 2;
    fTGName = new Char_t[idLen+1];
    sprintf (fTGName, "de_%s_%d_%d", fSel, fVar, fSeq);
    if (fDiSens == 1) fTGName[0] = 's';

// Get or create (if selector picks multiple maps) graph or 2d histogram

    if (fGrHist == 0) // Graph
    { if (fSource == 0)
      { if (fIMap < 0)
          fTG = fMapSet->GetGraph(fTGName, fSel, fVar, fVarRMS, fScale);
        else
          fTG = fMapSet->GetGraph(fTGName, fIMap, fVar, fVarRMS, fScale);
      }
      if (fSource == 1)
        fTG = fPadMap->GetGraph(fTGName, fVar, fVarRMS, fScale);
      if (fSource == 2)
      { if (fIMap < 0)
          fTG = fMul.GetGraph(fTGName, fSel, fVar, fVarRMS, fScale);
        else
          fTG = fMul.GetGraph(fTGName, fIMap, fVar, fVarRMS, fScale);
      }
    }

    else // histogram

    { if (fSource == 0)
      { if (fIMap < 0)
          fH2 = fMapSet->GetHist(fTGName, fSel, fVar, fVarRMS, fScale);
        else
          fH2 = fMapSet->GetHist(fTGName, fIMap, fVar, fVarRMS, fScale);
      }
      if (fSource == 1)
        fH2 = fPadMap->GetHist(fTGName, fVar, fVarRMS, fScale);
      if (fSource == 2)
      { if (fIMap < 0)
          fH2 = fMul.GetHist(fTGName, fSel, fVar, fVarRMS, fScale);
        else
          fH2 = fMul.GetHist(fTGName, fIMap, fVar, fVarRMS, fScale);
      }
    } // if (fGrHist
  } // if (!fNoNew

// Set style options and draw the graph or histo

  if (!fCanvas) printf ("TPhMulDisp: No canvas found; expect problems.\n");
  if (fGrHist == 0)
  { fTG->SetMarkerColor(fMColor);
    fTG->SetMarkerSize(fMSize);
    fTG->SetMarkerStyle(fMStyle);
    if (fSeq && fDrawOpts[0] == 'a') fDrawOpts = "pe";
    fTG->Draw(fDrawOpts);
  }
  else
    fH2->Draw(fDrawOpts);

  if (fCanvas)
    fCanvas->Update(); // Apparently required in a CInt macro or compiled class

// For the first plot (fSeq = 0), set the x- and y- titles.
//  This can't be done for a TGraphErrors until it's
// actually been drawn; the ROOT team will fix this after the second coming.
// The TH2F works correctly, so this just overrides an existing title.

  if (!fSeq)
  { TH1F *hh = 0;
    if (!fGrHist && (hh = fTG->GetHistogram()))
    { hh->SetXTitle(fXLabel);
      hh->SetYTitle(fYLabel);
    }
    else if (fH2)
    { fH2->SetXTitle(fXLabel);
      fH2->SetYTitle(fYLabel);
    }

// Set the size of the plotting frame, for key and title placement.  We can't
// use fTG->GetYaxis()->GetXmax() because it always returns 0-1.
// Until ROOT is fixed, get the actual ranges of the variables in the plot.

    Float_t x, xMin, xMax, y, yMin, yMax, zMin, zMax;
    if (fGrHist == 0)
    { for (Int_t ip = 0; ip < fTG->GetN(); ip++)
      { fTG->GetPoint (ip, x, y);
        if (!ip || x < xMin) xMin = x;
        if (!ip || x > xMax) xMax = x;
        if (!ip || y < yMin) yMin = y;
        if (!ip || y > yMax) yMax = y;
      }
    }
    else
// It's not clear where to put the keys on a 2-d Histo, maybe nowhere,
// but find the limits for possible later use
    { for (Int_t ix = 0; ix < fH2->GetXaxis()->GetNbins(); ix++)
        for (Int_t iy = 0; iy < fH2->GetYaxis()->GetNbins(); iy++)
        { Stat_t vv = fH2->GetCellContent (ix+1, iy+1);
          if (!(ix * iy) || x < xMin) xMin = x;
          if (!(ix * iy) || x > xMax) xMax = x;
          if (!(ix * iy) || y < yMin) yMin = y;
          if (!(ix * iy) || y > yMax) yMax = y;
          if (!(ix * iy) || vv < zMin) zMin = vv;
          if (!(ix * iy) || vv > yMax) zMax =vv;
        }
    }

    fPlotLim[0][0] = xMin * fExpand[0];
    fPlotLim[0][1] = xMax * fExpand[0];
    fPlotLim[1][0] = yMin * fExpand[1];
    fPlotLim[1][1] = yMax * fExpand[1];
  }
  if (fReportLevel >= 3) printf ("plot limits = %6.1f, %6.1f, %6.1f, %6.1f\n",
    fPlotLim[0][0], fPlotLim[1][0], fPlotLim[0][1], fPlotLim[1][1]);

// Create and draw key with legend.  This must be saved when other plots
// are drawn, or it will disappear.
  
  if (fTP[fSeq]) delete fTP[fSeq];
  Float_t fk = fKeyH;
  keyLim[0][0] = (1. - fk) * fPlotLim[0][0] + fk * fPlotLim[0][1];
  fk += fKeyHSize * (strlen(fKey) + 4);
  keyLim[0][1] = (1. - fk) * fPlotLim[0][0] + fk * fPlotLim[0][1];
  fk = fKeyV - fSeq * (fKeyVSize + fKeyVSep);
  keyLim[1][0] = (1. - fk) * fPlotLim[1][0] + fk * fPlotLim[1][1];
  fk += fKeyVSize;
  keyLim[1][1] = (1. - fk) * fPlotLim[1][0] + fk * fPlotLim[1][1];
  if (fReportLevel >= 3) printf ("key limits = %6.1f, %6.1f, %6.1f, %6.1f\n",
    keyLim[0][0], keyLim[1][0], keyLim[0][1], keyLim[1][1]);
// Add space for marker
  fLegend = new Char_t[strlen(fKey) + 2];
  sprintf(fLegend, "  %s", fKey);
  fTP[fSeq] = new TPaveLabel(keyLim[0][0], keyLim[1][0],
    keyLim[0][1], keyLim[1][1], fLegend);
  fTP[fSeq]->SetBorderSize(0);
  fTP[fSeq]->Draw();

// Create and draw marker in key
// The marker SHOULD be able to be put in the key pad object
  Float_t xm = keyLim[0][0] + 0.05 * (keyLim[0][1] - keyLim[0][0]);
  Float_t ym = 0.50 * (keyLim[1][0] + keyLim[1][1]);
  if (fTM[fSeq]) delete fTM[fSeq];
  fTM[fSeq] = new TMarker(xm, ym, fMStyle);
  fTM[fSeq]->SetMarkerSize(fMSize);
  fTM[fSeq]->SetMarkerColor(fMColor);
  fTM[fSeq]->Draw();

// The following text is never drawn by a compiled program, although it
// works in CInt:
//  fCanvas->DrawText(0.95 * fKeyH, 0.98 * fKeyV, fKey);
//  fCanvas->DrawText(0.2, 0.3, "0.2*0.3");
//  fCanvas->DrawText(-5.0, 1000., "-5*1000.");
//  fCanvas->Update();

// Add formatted current date and time

  if (!fSeq)

  { Char_t orig[] = "Thu Dec 24 00:50:29 1998";
//                   012345678901234567890123
    Char_t mod[]  = "24-Dec-98 00:50";
    Int_t locs[] = { 2,8,0, 3,4,3, 2,22,7, 5,11,10, 0 }; // #, orig, mod
    TDatime da;
    strncpy (orig, da.AsString(), 24);
    Int_t ll = 0;
    while (locs[ll])
    { for (Int_t jj = 0; jj < locs[ll]; jj++)
        mod[locs[ll+2]+jj] = orig[locs[ll+1]+jj];
      ll += 3;
    }
    if (fTPDa) delete fTPDa;
    Float_t xMin = 0.9 * fPlotLim[0][0] + 0.1 * fPlotLim[0][1];
    Float_t xMax = 0.7 * fPlotLim[0][0] + 0.3 * fPlotLim[0][1];
    Float_t yMin = 1.1 * fPlotLim[1][0] - 0.1 * fPlotLim[1][1];
    Float_t yMax = 1.05 * fPlotLim[1][0] - 0.05 * fPlotLim[1][1];
    fTPDa = new TPaveLabel(xMin, yMin + 0.0 * (yMax - yMin),
      xMin + 0.1 * (xMax - xMin), yMin + 0.04 * (yMax - yMin), mod);
    fTPDa->SetBorderSize(0);
    fTPDa->Draw();
  }

// Increase fSeq, color and style as defaults for the next display

  fSeq++;
  fMColor++;
  fMStyle++;

}

void TPhMulDisp::SetDefaults (Int_t var, Int_t varRMS, Float_t scale,
 Int_t grHist, Int_t diSens, Char_t *xLabel, Char_t *yLabel,
 Char_t *key, Int_t mStyle, Int_t mColor, Float_t mSize)
// Set standard defaults for Mul displays.
// fSeq, fMColor, and fMStyle are incremented after each Draw call;
// they can be manually reset but should index into an array of
// values.
{ fVar = var;
  fVarRMS = varRMS;
  fScale = scale;
  fGrHist = grHist;
  fDiSens = diSens;
  fXLabel = xLabel;
  fYLabel = yLabel;
  fKey = key;
  fMStyle = mStyle;
  fMColor = mColor;
  fMSize = mSize;

// Fields not specified as arguments of the creation call can be reset
// like struct fields, e.g. md.fKeyFont = 62.

  fCanvas = 0;        // Pointer to multiplicity canvas
  fCanvasName = "c1"; // Canvas name
//  fCanvas = (TCanvas *)gROOT->FindObject(canvasName);
  Int_t fCanXMin = 0; // Canvas extent in absolute pixels
  Int_t fCanYMin = 0;
  Int_t fCanXMax = 600;
  Int_t fCanYMax = 600;
  fDrawOpts = (fGrHist)? "lego2" : "ape"; // Drawing options
                      // The following values are appropriate when the 
                      // assume the plotLims are based on the plotted 
                      // variables rather than the axes:
  fExpand[0] = 1.0;   // Horizontal plot size / X data extent
  fExpand[1] = 1.1;   // Vertical plot size / X data extent
  fKeyH = 0.0;        // Horizontal position of key in frame, 0 - 1.0
  fKeyHSize = 0.02;   // Horizontal size of key in frame / char, 0 - 1.0
  fKeyVSep = 0.015;   // Vertical separation of keys in frame, 0 - 1.0
  fKeyVSize = 0.08;   // Vertical size of key in frame, 0 - 1.0
  fKeyV = 1.05;       // Vertical position of key, 0 - 1.0
  fKeyFont = 131;     // Font number for key text; 131 MAY be Times
                      // Key marker, size, and color are taken = vars
  fNoNew = 0;         // Used on updates
  fPlotLim[0][0] = -10; // Limits of the plot, in plot variable coordinates
  fPlotLim[0][1] = +10; // will be calculated from the plot itself
  fPlotLim[1][0] = 0;
  fPlotLim[1][1] = 1000;
  fReportLevel = 1;
  fSeq = 0;           // Initialize the sequence number
  fTG = (TGraphErrors *)0;
  fTitle = "";        // title ...
  fTitleH = 0.;
  fTitleV = 0.;
  fTitleFont = 62;
  fTitleColor = 1;
  fTitleSize = 0.8;
  for (Int_t nn = 0; nn < EMaxPlots; nn++)
  { fTM[nn] = (TMarker *)0;
    fTP[nn] = (TPaveLabel *)0;
  }
  fTPDa = (TPaveLabel *)0;
  fXLabelH = 0.;      // X axis text horizontal offset
  fXLabelV = 0.;      // FX axis text vertical offset
  fXLabelFont = fTitleFont;
  fXLabelColor = fTitleColor;
  fXLabelSize = fTitleSize;
  fYLabelH = fXLabelH;
  fYLabelV = fXLabelV;
  fYLabelFont = fXLabelFont;
  fYLabelColor = fXLabelColor;
  fYLabelSize = fXLabelSize;
// Create a default canvas if none is available
  fCanvas = (TCanvas *)gPad;
  if (!fCanvas)
  { fCanvas = new TCanvas(fCanvasName, "Multiplicity",
      fCanXMin, fCanYMin, fCanXMax, fCanYMax);
    if (fReportLevel >= 3)
      printf ("Created new canvas \"%s\"\n", fCanvasName);
  }
}

void TPhMulDisp::Draw (Int_t seq, Text_t canvas)
{ printf ("TPhMulDisp/Draw: replacing obsolete function signature.\n");
  if (canvas) seq ++; // kill 'unused' warnings 
  Draw (fVar, fVarRMS, fScale, fKey);
}


void TPhMulDisp::Update()
{ fNoNew = 1;
  Draw (fVar, fVarRMS, fScale, fKey);
  fNoNew = 0;
}
