       Subroutine PH_RESTART
*Documentation_Block
************************************************************************
*
*  Description:
*  Routine necessary for restat of simulations in PHAT
*  sets the initial values of control flags (FIRSTC mainly)
*  WARNING: PMC is ready for restart, but GEANT is NOT!
*
*  Arguments:
*  none
*
*  Called by:
*  GEPHOB
*
*
*  Implicit inputs, outputs, side effects:
*  Changes flags in common blocks
*
*  Filename: PMC_SRC:PH_RESTART.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  12-Feb-98  K.W.       Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,FIRSTC.
C  PHOBOS "first call" flags,   File FIRSTC.CMN
C  The flags denoting the very first call are put in one common
C  to enable easy restart of simulations in PHAT
C  11-Feb-98  K. W.      Creation
C
      LOGICAL FCGUFLD, FCGUSTEP, FCPHOUT
      COMMON /CFIRSTC/ FCGUFLD, FCGUSTEP, FCPHOUT
*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,SNS.
C SNS.CMN: PHOBOS detector types information
C          recognized, stored and used in UGCSENS routine
C
c    9-Dec-97  K. W.  Creation
c Storage limit for number of detector types and number of parameters
c in the definition of active volume
      integer SNSMAX,SNSPARMAX
      parameter (SNSMAX=200,SNSPARMAX=6)

      CHARACTER*4 SNSSHAPE(SNSMAX)
      REAL SNSPAR(SNSPARMAX,SNSMAX), SNSPAD(10,SNSMAX)
      INTEGER SNSFILL(SNSMAX), SNSNMED(SNSMAX), SNSIPAD(SNSMAX)
      INTEGER SNSNPAR(SNSMAX)
      INTEGER SNSDIVX(SNSMAX), SNSDIVY(SNSMAX), SNSDIVZ(SNSMAX)
      INTEGER SNSGDTH, SNSVIDT, SNS NEHI, SNSNWDI

      COMMON /CSNS/ SNSGDTH, SNSVIDT, SNS NEHI, SNSNWDI,
     +     SNSSHAPE, SNSNMED, SNSPAR, SNSNPAR, SNSIPAD,
     +     SNSPAD, SNSDIVX, SNSDIVY, SNSDIVZ
*KEND.

      INTEGER I

      FCGUFLD = .TRUE.
      FCGUSTEP = .TRUE.
      FCPHOUT = .TRUE.

      EPSTATE = 0
      IDEVT = 0
      IEVENT = 0

      NDETS = 0
      DO  I=1, SNSMAX
        SNSFILL(I) = 0
      ENDDO

      RETURN
      END
