       Subroutine PIXEL(IDET, DETHIT, ILAST)
*Documentation_Block
************************************************************************
*
*  Description:
*  Pixelization of the hits, the hit information from Geant bank (DETHIT)
*  is transered to the tables in HITPIX common.
*
*  Depending on the pixelization flags performs:
*   IPIXL=0   -  copies HIT information to output tables
*                pixel numbers are always 0
*   IPIXL=1   - for BOX  volumes (IPADFL=1) calculates  pixel numbers
*               only one hit is created in HITPIX tables
*             - for TUBS volumes (IPADFL=6)
*                - performs pixelization in radial direction
*                  (only if NDIVX = 0)
*                  thus several hits may be put in HITPIX tables
*                - calculates  pixel numbers (also for NDIVX > 0)
*                  for one hit or several hits obtained by splitting
*                  of the input hit
*   IPIXL=2   - as for IPIXL=1, but the hit center is moved to the
*               center of the pixel
*     for other volumes (IPADFL=2,3,4,5, 7,8,...) only copies the data
*     (like for IPIXL=0)
*     the list of supported volumes types may be extended, if necessary
*
*  Arguments:
*  IDET    - detector number in RHSIL tables
*  DETHIT  - data from hits bank
*  ILAST   - the sequential number of previous hit in this detector
*
*  Called by:
*  PH_OUT
*
*  Calls:
*  DETGEOM, RDIVISION
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  17-Dec-98  K.W.       Corrected rare rounding error in ring pixelization
*   7-Jul-98  K.W.       Added recalculation of the length of the track
*                        up to the hit (when the hit is divided)
*                        Corrected error in the rings pixelization
*                        IPIXL=2 mode added
*  28-JAN-98  K.W.       merging of multiple hits from the same particle
*                        in the same pixel (correction of GEANT error)
*   4-Nov-97  K.W.       TUBS volumes pixelized, either
*                         - by Geant only (for equal r divisions)
*                         - or by Geant (phi) and RDIVISION
*                           (in case of not equal r disvisions)
*  17-Sep-97  K.W.       corected pixel numbers for 2 divisions
*  21-Aug-97  K.W.       Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE


*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,HITPIX.
C HITPIX.CMN: PHOBOS hits information for AOU output
c   20-Nov-98  K. W.  Added INDX table for hits sorting
c    7-Jul-98  K. W.  Increased NHITPAR to 12, but allow variable NHITSPAR
c                     depending on UGCDATA (in PH_OUT code)
c   10-Feb-98  K. W.  Increased NHHTMAX  parameter,
C                     NHITSPAR parameter defined
c   21-Aug-97  K. W.  Creation
c Storage limit for number of hits on a single track (including
c multiple hits in a plane due to traversing several pixels):
      integer NHHTMAX, NHITSPAR, NHITSBM
      parameter (NHITSPAR=12, NHHTMAX=500)
      parameter (NHITSBM=NHITSPAR*NHHTMAX)

      INTEGER NHHT, IPLHT(NHHTMAX), INDX(NHHTMAX)
      CHARACTER*4 PLHT(NHHTMAX)
      EQUIVALENCE (IPLHT(NHHTMAX),PLHT(NHHTMAX))
      REAL XHT(NHHTMAX), YHT(NHHTMAX), ZHT(NHHTMAX)
      REAL DEHT(NHHTMAX), DTHT(NHHTMAX), TRACKLNG(NHHTMAX)
      REAL DXHT(NHHTMAX), DYHT(NHHTMAX), DZHT(NHHTMAX)
      INTEGER IXPIX(NHHTMAX), IYPIX(NHHTMAX)

      REAL HITSBUF(NHITSBM)
      INTEGER NUMBV(NHHTMAX), ITRATAB(NHHTMAX)

      COMMON /CHITPIX/ NHHT, IPLHT, XHT, YHT, ZHT, DEHT, DTHT,
     +     DXHT, DYHT, DZHT, IXPIX, IYPIX, TRACKLNG,
     +     HITSBUF, NUMBV, ITRATAB, INDX
*KEND.

      INTEGER IDET, ILAST
      REAL DETHIT(NHITSPAR)


C     Local definitions:
C     ==================

      INTEGER NDETPAR, NPARMAX, IX, IY, IDETSHP
      PARAMETER (NPARMAX=30)
      REAL DETPAR(NPARMAX), DETPOS(3), DETROT(9)
      REAL*8 V0(3), V1(3), VR(3), VDET0(3), VDET1(3), VDETR(3)
      REAL*8 RM, PHIM, VM(3), VDETM(3), T1, T2
      REAL*8 HIT(NHITSPAR), R, R0, R1, PHI0, PHI1, DEGRAD
      REAL*8 TLNGR, TLNG, RDV(9), XDIF,YDIF,ZDIF
      INTEGER I, IX0, IX1, IXR, IY0, IY1, IHIT


      DEGRAD = 0.0174532925199432958
      IX = 0
      IY = 0
C
      DO  I=1,NHITSPAR
         HIT(I) = DETHIT(I)
      ENDDO
c                                    !**********************
      IF(IPIXL .EQ. 0)  GO TO 600    !  Pixelization OFF  >>>>
c                                    !**********************

c                              !*********************
      IF(IPIXL .GT. 0)  THEN   !  Pixelization ON   *
c                              !*********************

        IF(IPADFL(IDET) .EQ. 1)   THEN  !  BOX volume
c                                          division done by GEANT
c                                          only pixel numbers counted
           NDETPAR = NPARMAX
           CALL DETGEOM(IDET, IDETSHP, NDETPAR, DETPAR, DETPOS, DETROT)
           V0(1) = HIT(1) - DETPOS(1)
           V0(2) = HIT(2) - DETPOS(2)
           V0(3) = HIT(3) - DETPOS(3)
           VDET0(1) = DETROT(1)*V0(1) + DETROT(2)*V0(2) +
     +                     DETROT(3)*V0(3)
           VDET0(2) = DETROT(4)*V0(1) + DETROT(5)*V0(2) +
     +                     DETROT(6)*V0(3)
           VDET0(3) = DETROT(7)*V0(1) + DETROT(8)*V0(2) +
     +                     DETROT(9)*V0(3)
C
C   assumption for BOX volumes: no divisions in Z direction (internal)
C
           VDET0(1) = VDET0(1) + DETPAR(1)  !  move to the corner
           VDET0(2) = VDET0(2) + DETPAR(2)
           VDET0(3) = VDET0(3) + DETPAR(3)

           IF(2.0*DETPAR(1) .GT. PAD1(IDET) .AND.
     +          PAD1(IDET) .GT. 0.00001)
     +                     IX = VDET0(1) / PAD1(IDET)
           IF(2.0*DETPAR(2) .GT. PAD2(IDET) .AND.
     +          PAD2(IDET) .GT. 0.00001)
     +                     IY = VDET0(2) / PAD2(IDET)

           IF(IPIXL .EQ. 2)  THEN   !  move hit to the center of the pixel
              VDETM(1) = (FLOAT(IX)+0.5) * PAD1(IDET) - DETPAR(1)
              VDETM(2) = (FLOAT(IY)+0.5) * PAD2(IDET) - DETPAR(2)
              VDETM(3) = 0.0
c                     !  VDET0(3) = 0  can be ignored
              VM(1) = DETROT(1)*VDETM(1) + DETROT(4)*VDETM(2)
              VM(2) = DETROT(2)*VDETM(1) + DETROT(5)*VDETM(2)
              VM(3) = DETROT(3)*VDETM(1) + DETROT(6)*VDETM(2)
              HIT(1) = VM(1) + DETPOS(1)
              HIT(2) = VM(2) + DETPOS(2)
              HIT(3) = VM(3) + DETPOS(3)
           ENDIF   !  (IPIXL .EQ. 2)

           GO TO 600

        ELSE  IF(IPADFL(IDET) .EQ. 6)   THEN  !  TUBS volume

           NDETPAR = NPARMAX
           CALL DETGEOM(IDET, IDETSHP, NDETPAR, DETPAR, DETPOS, DETROT)
           IF(NDIVX(IDET) .EQ. 0)  THEN   ! bins in R defined by PADn
              RDV(1) = PAD2(IDET)
              RDV(2) = PAD3(IDET)
              RDV(3) = PAD4(IDET)
              RDV(4) = PAD5(IDET)
              RDV(5) = PAD6(IDET)
              RDV(6) = PAD7(IDET)
              RDV(7) = PAD8(IDET)
              RDV(8) = PAD9(IDET)
              RDV(9) = PAD10(IDET)
           ENDIF   !  (NDIVX(IDET) .EQ. 0)

           V0(1) = HIT(1) - DETPOS(1) - 0.5 * HIT(5)  !  entrance point (*0)
           V0(2) = HIT(2) - DETPOS(2) - 0.5 * HIT(6)
           V0(3) = HIT(3) - DETPOS(3) - 0.5 * HIT(7)

           V1(1) = HIT(1) - DETPOS(1) + 0.5 * HIT(5)  !  exit point  (*1)
           V1(2) = HIT(2) - DETPOS(2) + 0.5 * HIT(6)
           V1(3) = HIT(3) - DETPOS(3) + 0.5 * HIT(7)

           VDET0(1) = DETROT(1)*V0(1) + DETROT(2)*V0(2) +
     +                     DETROT(3)*V0(3)
           VDET0(2) = DETROT(4)*V0(1) + DETROT(5)*V0(2) +
     +                     DETROT(6)*V0(3)
           VDET0(3) = DETROT(7)*V0(1) + DETROT(8)*V0(2) +
     +                     DETROT(9)*V0(3)

           VDET1(1) = DETROT(1)*V1(1) + DETROT(2)*V1(2) +
     +                     DETROT(3)*V1(3)
           VDET1(2) = DETROT(4)*V1(1) + DETROT(5)*V1(2) +
     +                     DETROT(6)*V1(3)
           VDET1(3) = DETROT(7)*V1(1) + DETROT(8)*V1(2) +
     +                     DETROT(9)*V1(3)
           R0 = DSQRT(VDET0(1)**2 + VDET0(2)**2)
           R1 = DSQRT(VDET1(1)**2 + VDET1(2)**2)
           PHI0 = DATAN2(VDET0(2), VDET0(1)) / DEGRAD
           PHI1 = DATAN2(VDET1(2), VDET1(1)) / DEGRAD
           IY0 = INT((PHI0 - DETPAR(4)) / PAD1(IDET))
           IY1 = INT((PHI1 - DETPAR(4)) / PAD1(IDET))

           IF(NDIVX(IDET) .GT. 0)  THEN   ! equal bins in R
              IX0 = INT((R0-DETPAR(1)) / PAD2(IDET))
              IX1 = INT((R1-DETPAR(1)) / PAD2(IDET))
              IF(IX0 .LT. 0)  IX0 = 0
              IF(IX1 .LT. 0)  IX1 = 0
              IX = IX0
              IY = IY0
              IX1 = IX0
              GO TO 100
           ENDIF   ! (NDIVX(IDET) .GT. 0)

C      division in R specified by a set of values - up to 8 bins

           IX0 = 0
           IX1 = 0
           DO  I=1,8
              IF(R0 .GE. RDV(I))  IX0 = I - 1
              IF(R1 .GE. RDV(I))  IX1 = I - 1
           ENDDO
           IX = IX0
           IY = IY0

  100     CONTINUE
           IF(IX0 .NE. IX1)   GO TO 110  !  entrance and exit in different pads

           IF(IPIXL .EQ. 2)  THEN   !  move hit to the center of the pixel
             PHIM = (FLOAT(IY0)+0.5) * PAD1(IDET) + DETPAR(4)

             IF(NDIVX(IDET) .GT. 0)   THEN  ! equal bins in R
                RM = (FLOAT(IX0)*PAD2(IDET)+DETPAR(1))**2 +
     +                ((FLOAT(IX0)+1.0)*PAD2(IDET)+DETPAR(1))**2
             ELSE   !  non equal R bins
                RM = RDV(IX0+1)**2 + RDV(IX0+2)**2
             ENDIF   !  (NDIVX(IDET) .GT. 0)
             RM = DSQRT(RM / 2.0)  ! RM = sqrt((r1**2+r2**2)/2)
c                                  ! and not  RM = (r1+r2)/2

             VDETM(1) = RM * DCOS(PHIM*DEGRAD)
             VDETM(2) = RM * DSIN(PHIM*DEGRAD)
             VDETM(3) = 0.0
             VM(1) = DETROT(1)*VDETM(1) + DETROT(4)*VDETM(2)
             VM(2) = DETROT(2)*VDETM(1) + DETROT(5)*VDETM(2)
             VM(3) = DETROT(3)*VDETM(1) + DETROT(6)*VDETM(2)
             HIT(1) = VM(1) + DETPOS(1)
             HIT(2) = VM(2) + DETPOS(2)
             HIT(3) = VM(3) + DETPOS(3)
           ENDIF   !  (IPIXL .EQ. 2)

c                                        !     >>>>>>>>>
          GO TO 600
c
c  division in R direction necessary
c
  110     CONTINUE
           TLNG = DSQRT((VDET1(1)-VDET0(1))**2 +
     +                           (VDET1(2)-VDET0(2))**2)
           IF(TLNG .LT. 0.1E-4)  GO TO 600  !  too close points to be divided

           IF(IX0 .LT. IX1)   THEN    !  R crossed depends on
              R = RDV(IX0+2)          !  direction of the track
              IXR = IX0 + 1
           ELSE
              R = RDV(IX0+1)
              IXR = IX0 - 1
           ENDIF   ! (IX0 .LT. IX1)
           CALL RDIVISION(VDET0(1), VDET0(2), VDET1(1), VDET1(2), R,
     +                       VDETR(1), VDETR(2))
c                                            VDETR - point on the boundary
c                                                    between the pixels

           TLNGR = DSQRT((VDETR(1)-VDET1(1))**2 +
     +                          (VDETR(2)-VDET1(2))**2)
           IF(TLNGR .LT. 0.1E-4)  THEN  !  track with exit just after
              IX = IX0                  !  pad boundary - assumed to end
              GO TO 600                 !  in the same pad
           ENDIF   !  (TLNGR .LT. 0.1E-4)

           TLNGR = DSQRT((VDETR(1)-VDET0(1))**2 +
     +                          (VDETR(2)-VDET0(2))**2)

           VDETR(3) = VDET0(3) + (VDET1(3)-VDET0(3)) * TLNGR / TLNG

           IF(TLNGR .LT. 0.1E-4)  THEN  !  track with beginning just before
              IF(IXR .EQ. IX1)  THEN    !  pad boundary - assumed to start
c                                       !  in the next pad
                 IF(IX0 .LT. IX1)  IX0 = IX0 + 1
                 IF(IX0 .GT. IX1)  IX0 = IX0 - 1
                 IX = IX0
                 GO TO 100
              ENDIF
              GO TO 150
           ENDIF   !  (TLNGR .LT. 0.1E-4)
c
c    additiona hit created here - effect of splitting of Geant hit
c    first part stored here, the rest processed once more
           NHHT = NHHT + 1
           IF(NHHT .GE. NHHTMAX)   go to 900  !  overflow

           IF(IPIXL .EQ. 2)  THEN  ! move hit to the center of the pixel
             PHIM = (FLOAT(IY0)+0.5) * PAD1(IDET) + DETPAR(4)

             IF(NDIVX(IDET) .GT. 0)   THEN  ! equal bins in R
                RM = (FLOAT(IX0)*PAD2(IDET)+DETPAR(1))**2 +
     +                ((FLOAT(IX0)+1.0)*PAD2(IDET)+DETPAR(1))**2
             ELSE   !  non equal R bins
                RM = RDV(IX0+1)**2 + RDV(IX0+2)**2
             ENDIF   !  (NDIVX(IDET) .GT. 0)
             RM = DSQRT(RM / 2.0)  ! RM = sqrt((r1**2+r2**2)/2)
c                                  ! and not  RM = (r1+r2)/2

             VDETM(1) = RM * DCOS(PHIM*DEGRAD)
             VDETM(2) = RM * DSIN(PHIM*DEGRAD)
             VDETM(3) = 0.0

           ELSE   !  (IPIXL .EQ. 1)  - normal pixelization
             VDETM(1) = (VDET0(1) + VDETR(1)) / 2.0
             VDETM(2) = (VDET0(2) + VDETR(2)) / 2.0
             VDETM(3) = (VDET0(3) + VDETR(3)) / 2.0
           ENDIF   !  (IPIXL .EQ. 2)

           VM(1) = DETROT(1)*VDETM(1) + DETROT(4)*VDETM(2) +
     +               DETROT(7)*VDETM(3)
           VM(2) = DETROT(2)*VDETM(1) + DETROT(5)*VDETM(2) +
     +               DETROT(8)*VDETM(3)
           VM(3) = DETROT(3)*VDETM(1) + DETROT(6)*VDETM(2) +
     +               DETROT(9)*VDETM(3)

           VR(1) = DETROT(1)*VDETR(1) + DETROT(4)*VDETR(2) +
     +               DETROT(7)*VDETR(3)
           VR(2) = DETROT(2)*VDETR(1) + DETROT(5)*VDETR(2) +
     +               DETROT(8)*VDETR(3)
           VR(3) = DETROT(3)*VDETR(1) + DETROT(6)*VDETR(2) +
     +               DETROT(9)*VDETR(3)

           IPLHT(NHHT) = IDET
           XHT(NHHT) = VM(1) + DETPOS(1)
           YHT(NHHT) = VM(2) + DETPOS(2)
           ZHT(NHHT) = VM(3) + DETPOS(3)
           DEHT(NHHT) = HIT(4) * TLNGR / TLNG
           DTHT(NHHT) = HIT(9)
           DXHT(NHHT) = VR(1) - V0(1)
           DYHT(NHHT) = VR(2) - V0(2)
           DZHT(NHHT) = VR(3) - V0(3)
           IXPIX(NHHT) = IX0
           IYPIX(NHHT) = IY0
           TRACKLNG(NHHT) = HIT(12) -  TLNG / 2.0 + TLNGR / 2.0

  150     CONTINUE    !  the trajectory in the next pixel(s) hit
c                     !  now pixeled as the whole hit
c                     !  only hit information has to be updated
          HIT(1) = (V1(1) + VR(1))/2.0  + DETPOS(1)
          HIT(2) = (V1(2) + VR(2))/2.0  + DETPOS(2)
          HIT(3) = (V1(3) + VR(3))/2.0  + DETPOS(3)
          HIT(4) = HIT(4) * (1.0 - TLNGR/TLNG)
          HIT(5) = V1(1) - VR(1)
          HIT(6) = V1(2) - VR(2)
          HIT(7) = V1(3) - VR(3)
          HIT(12) = HIT(12) - TLNG / 2.0 +
     +                TLNGR + (TLNG - TLNGR) / 2.0

          V0(1) = VR(1)
          V0(2) = VR(2)
          V0(3) = VR(3)
          VDET0(1) = VDETR(1)
          VDET0(2) = VDETR(2)
          VDET0(3) = VDETR(3)
          IX0 = IXR
          IX = IXR

          GO TO 100    !  continue division of the track

        ELSE
           GO TO 600   !  (other types of volumes)

        ENDIF    !   IPADFL(IDET) .EQ. 1

      ENDIF  !  IPIXL .GT. 0


  600 CONTINUE   !  store hit and pixel information in the tables
c                   for IPIXL=0 mode and single hits
      IF(NHHT .GE. NHHTMAX)   THEN
           GO TO 900
      ENDIF


c  Test if the previous and actual hits should be merged. It is
c  necessary when a particle flies exactly along a pixel boundary
c  (possible only in simulations without multiple scattering)
c  because then Geant several times signal that the particle
c  leaves the volume, but then finds that it is still in the same
c  volume. Merging should be done only when:
c   A. two (or more) hits are in the same pixel
c   B. they have continous hit numbers (sequential along the track) -
c      the particle did not enter any other pixel/sensor between them
c   C. the time difference in the hits is negligible:
c      (c * dt < 0.6 cm  limit is used)
c   D. the difference between the exit for the first hit and entrance
c      for the second can not be larger than 1 micron
c
      IF(NHHT .GT. 0  .AND. IPIXL .GT. 0)  THEN
       IF(IX .EQ. IXPIX(NHHT) .AND. IY .EQ. IYPIX(NHHT)  .AND.
     +   IDET .EQ. IPLHT(NHHT) )  THEN  !  (merging condition A.)

         IHIT = INT(HIT(11))
         IF((IHIT - ILAST .EQ. 1) .AND.
     +         ABS(DTHT(NHHT)-HIT(9)) .LT. 0.2E-10) GO TO 700
C                        !  (merging conditions  B. and C.)
       ENDIF
      ENDIF

  610 CONTINUE
       NHHT = NHHT + 1   !  add new hit
       IPLHT(NHHT) = IDET
       XHT(NHHT) = HIT(1)
       YHT(NHHT) = HIT(2)
       ZHT(NHHT) = HIT(3)
       DEHT(NHHT) = HIT(4)
       DTHT(NHHT) = HIT(9)
       DXHT(NHHT) = HIT(5)
       DYHT(NHHT) = HIT(6)
       DZHT(NHHT) = HIT(7)
       IXPIX(NHHT) = IX
       IYPIX(NHHT) = IY
       TRACKLNG(NHHT) = HIT(12)

      RETURN

  700 CONTINUE   !  two hits in the same pixel - merge them
c                !  (solution necessary to correct GEANT error
c                !   when a particle flies along the pixel boundary)

      IF(IPIXL .EQ. 2)  GO TO 750  !  hit position in the middle of pixel
c                                  !  has to be treated differently

      V0(1) = XHT(NHHT) - DXHT(NHHT) / 2.0
      V0(2) = YHT(NHHT) - DYHT(NHHT) / 2.0
      V0(3) = ZHT(NHHT) - DZHT(NHHT) / 2.0
      V1(1) = XHT(NHHT) + DXHT(NHHT) / 2.0
      V1(2) = YHT(NHHT) + DYHT(NHHT) / 2.0
      V1(3) = ZHT(NHHT) + DZHT(NHHT) / 2.0
      VR(1) = HIT(1) - HIT(5) / 2.0
      VR(2) = HIT(2) - HIT(6) / 2.0
      VR(3) = HIT(3) - HIT(7) / 2.0
      XDIF = ABS(V1(1)-VR(1))
      YDIF = ABS(V1(2)-VR(2))
      ZDIF = ABS(V1(3)-VR(3))

C   check if the hits form a continous track  (merging condition D.)
      IF(SQRT(XDIF**2 + YDIF**2 + ZDIF**2) .GT. 0.0001)
     +                                              GO TO 610

      V1(1) = HIT(1) + HIT(5) / 2.0
      V1(2) = HIT(2) + HIT(6) / 2.0
      V1(3) = HIT(3) + HIT(7) / 2.0

       XHT(NHHT) = (V1(1) + V0(1)) / 2.0   !  new hit position
       YHT(NHHT) = (V1(2) + V0(2)) / 2.0
       ZHT(NHHT) = (V1(3) + V0(3)) / 2.0
       TRACKLNG(NHHT) = TRACKLNG(NHHT) -
     +  SQRT(DXHT(NHHT)**2 + DYHT(NHHT)**2 + DZHT(NHHT)**2) / 2.0

       DXHT(NHHT) =  V1(1) - V0(1)          !  new track length
       DYHT(NHHT) =  V1(2) - V0(2)
       DZHT(NHHT) =  V1(3) - V0(3)

       DEHT(NHHT) = DEHT(NHHT) + HIT(4)   !  add energy loss
       TRACKLNG(NHHT) = TRACKLNG(NHHT) +
     +   SQRT(DXHT(NHHT)**2 + DYHT(NHHT)**2 + DZHT(NHHT)**2) / 2.0

      RETURN

  750 CONTINUE   !  (IPIXL .EQ. 2)
        T1 = SQRT(DXHT(NHHT)**2 + DYHT(NHHT)**2 + DZHT(NHHT)**2)
        T2 = SQRT(HIT(5)**2 + HIT(6)**2 + HIT(7)**2)
        R = DXHT(NHHT)*HIT(5) + DYHT(NHHT)*HIT(6) +
     +       DZHT(NHHT)*HIT(7)
        IF(T1 .GT. 0.0001  .AND.  T1 .GT. 0.0001)  THEN
           R = R / (T1 * T2)
        ELSE
           R = 1.0
        ENDIF
c
c    Different condition D: track directions for merged hits has
c    to be approximately parallel
c
        IF(R .LT. 0.99)  GO TO 610  !  difference greater than 8 degrees

        DXHT(NHHT) = DXHT(NHHT) + HIT(5)   !  add track fragment
        DYHT(NHHT) = DYHT(NHHT) + HIT(6)
        DZHT(NHHT) = DZHT(NHHT) + HIT(7)

        DEHT(NHHT) = DEHT(NHHT) + HIT(4)   !  add energy
        TRACKLNG(NHHT) = TRACKLNG(NHHT) + T2 / 2.0

        RETURN


  900 CONTINUE
      WRITE(*,*) ' *** WARNING IN PIXEL: MAXIMAL NUMBER OF HITS FOR',
     +            ' A SINGLE TRACK EXCEEDED:', NHHT
      WRITE(*,*) '     REMAINING HITS IGNORED'
      RETURN
      END
