      SUBROUTINE UGCONS (Batch, Batchf, NoLog)
*Documentation_Block
************************************************************************
*
* Description:
* UGCONS is a user hook for the interactive Geant main program that
* allows the user to choose the mode of operation, as follows:
* If the program is running in batch mode, as determined by the CERN
* INTRAC function, then if a readable file is assigned to logical
* name BATKUMAC, it is interpreted as a KUMAC file of KUIP commands;
* if it is not readable or not present, the program runs in ordinary
* batch mode.
* This overcomes the problem that the standard Geant way of specifying
* batch mode for some reason ignores the /nodebug qualifier and makes
* the program run in debug mode, requiring an additional batch debug
* control file.
*
*  Arguments:
*  Batch*:    Logical variable specifying batch mode if true
*  Batchf*:   Character string specifying the name of the Batch file
*  Nolog*:    Logical variable: read GTLOGIN.KUMAC if true
*
*  Called by:
*  GXINT, after calling KUARGS to get command line arguments.
*
*  Calls:
*  none
*
*  Implicit inputs, outputs, side effects:
*  via COMMONs included or defined below, in particular RHSIL
*
* Filename: UGCONS.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  13-Aug-96  R. V.      open -> call UFILE; SERVER can't be used
*                          here as it aborts on file not found
*  02-Jun-93  R. Verdier Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

c Argument declarations:

      character*(*) Batchf
      logical Batch, Nolog
      logical INTRAC

      integer dummy, Lun
c End of declarations

      if (INTRAC(dummy)) then
        Batch = .false.
      else
        Batch = .true.
        call UFILE (0, 'BATKUMAC', Lun, Batchf)
        if (Lun .le. 0) Batchf = ' '
      endif
c*      print *, ' UGCONS: Batch =', Batch, ', filename="', batchf, '".'
      Nolog = .true.
      return
      end
