      SUBROUTINE UGCSET (Key, Iielem, Nvalues, Vtypes, Vlengths,
     *  Rvalues, Ivalues, Svalues, Ires)
*Documentation_Block
************************************************************************
*
*  Description:
*  UGCSET copies user geometry and control variables read by the UGC text
*  file input system to local, common or bank storage, for any necessary
*  communication with Geant or user programs.  This is the user-specific
*  section of the UGC system, and must be modified to allow access to
c  new user variables.
*
*  Arguments:
*  *Key:      Character substring identifying the variable to be set
*  *Iielem:   Index of initial variable element
*  *Nvalues:  Number of values to be set
*  *Vtypes:   Variable types, 1=integer*4, 2=real*4, 3=string
*  *Vlengths: Storage lengths of variable elements
*  *Rvalues:  Vector of real values for the variable
*  *Ivalues:  Vector of integer values for the variable
*  *Svalues:  Vector of string values for the variable, not yet
*               implemented
*  Ires*:     Result flag = # of variables copied, 0 if key was not
*             recognized, < 0 if error (wrong variable type or bad value)
*
*  Called by:
*  UGCIN, after parsing an input record and evaluating tokens
*
*  Calls:
*  STOPER
*
*  Implicit inputs, outputs, side effects:
*  via COMMONs included or defined below, in particular RHSIL
*
* Filename: UGCSET.FOR
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  12-Nov-97  K.W.       PADSNEW and PADSOLD moved to a common
*   4-Nov-97  K.W.       increased number of  real  GS SET PADS parameters to 10
*   1-Jul-97  K.W.       modified code for GS SET PADS - to enable GS DVN
*   3-Mar-97  K.W.       modified GS SET SLP  and  GS SET IUDET
*                        added  GS SET PADS
*  23-Sep-96  K.W.       Removed definitions of MAGX, MAGZ, RFLD, BFLD, MOFF
*                        (they must not appear in the UGCDATA file now)
*  21-Jul-96  R. V.      Removed unused par, r1, x1, z1, z2
*   9-Jul-96  K. W.      magnetic field center (MAGX,MAGZ) separated from
*                        the field offset (MOFF(3))
*   9-Jul-96  K. W.      Added geometry version GEOV
*                        MAGT, RFLG etc. are defined now in the magnetic field
*                        map (defaults set in control file) and left here
*                        for backward compatibility only
*  24-Apr-93  R. V.      Added Iielem, Vtypes, and Vlengths args to
*    implement limited vector cabability; moved SIxxx defs to RHSIL.CMN
*  21-Feb-93  R. V.      Added magt, tmag, rfld, nslp = nplanes
*  03-Nov-92  R. Verdier Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

c Argument declarations:

      character*(*) Key
      integer       Iielem, Ires, Nvalues, Ivalues(*), Vlengths(*),
     *              Vtypes(*)
      real          Rvalues(*)
      character*(*) Svalues

c Common declarations:

*KEEP,CARDS.
C  Common blocks for GPH PHOBOS simulation:  FILE CARDS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements.
C
      REAL YGAP,YCRH,THCR
C     1   OUT1,OUT2,BMV1,BMV2,BMT1,BMT2,SPE1,SPE2,CRB1,CRB2,
C     1   CRS1,CRS2,FIL1,FIL2
C
      COMMON/CARDS/YGAP,YCRH,THCR
C         OUT1(6),OUT2(6),BMV1(6),BMV2(6),
C     1   BMT1(6),BMT2(6),SPE1(7),SPE2(7),CRB1(6),CRB2(6),
C     1   CRS1(6),CRS2(6),FIL1(6),FIL2(6)



*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,RHSIL.
C RHSIL.CMN: PHOBOS detector plane dimensions, positions, and
c granularity; UGC parameters
c The actual and maximum numbers of detector planes and detector hit
c parameters.  Their default values are defined in RHSILD.INC.
c 12-Dec-97 K. W.       PADSOLD and PADSNEW from UGCSET moved to a common
c  4-Nov-97 K. W.       Increased number of PADS parameters to 10
c                       Increased NSLPM to 800
c  1-Jul-97 K. W.       Added data for GEANT pixelization (GSDVN)
c  1-May-97 R. V.       Increased NSLPM to 500
c  3-MAR-97 K. W.       Removed SPL, added PADS data
C  3-Jan-97 K. W.       Increased NSLPM to 180
c  3-Jul-96 K. W.       Geometry version number added
c  5-Apr-96 D. McL.     Moved data value of UgcpMax to RHSILD
c  ...
c    Mar-92 D.McLeod    Creation
c Storage limit for number of silicon planes:
      INTEGER NSLPM
      PARAMETER (NSLPM=800)
c Storage limit for number of parameters for each hit:
      integer NSipMaxp
      parameter (NSipMaxp = 12)
c Safety limit for number of parameter values that may be entered in
c a UGC text input record:
      integer UgcpMaxp
      parameter (UgcpMaxp = 10)
c Number of silicon planes in use:
      integer NSLP
c Additional information on detectors - pad sizes:
C  DETNAME  - name of the detector volume
C  DIVXNAME - name for division in X direction
C  DIVYNAME - name for division in Y direction
C  DIVZNAME - name for division in Z direction
C  NDIVX    - number of divisions in X direction
C  NDIVY    - number of divisions in Y direction
C  NDIVZ    - number of divisions in Z direction
C  IPADFL   - pad type flag
C  PADn   - four pad parameters
C             (pad size in X, Y, Z direction for IPADFL=1 for example)
      CHARACTER*4 DETNAME(NSLPM), DIVXNAME(NSLPM),
     +             DIVYNAME(NSLPM), DIVZNAME(NSLPM)
      INTEGER NDIVX(NSLPM), NDIVY(NSLPM), NDIVZ(NSLPM)
      REAL PAD1(NSLPM),PAD2(NSLPM),PAD3(NSLPM),PAD4(NSLPM),PAD5(NSLPM)
      REAL PAD6(NSLPM),PAD7(NSLPM),PAD8(NSLPM),PAD9(NSLPM),PAD10(NSLPM)
      INTEGER NDETS, IPADFL(NSLPM)
      LOGICAL PADSOLD, PADSNEW
      COMMON /RHPADS/ NDETS, PADSOLD, PADSNEW,
     +       DETNAME, DIVXNAME, DIVYNAME, DIVZNAME,
     +       NDIVX, NDIVY, NDIVZ,
     +       IPADFL, PAD1, PAD2, PAD3, PAD4, PAD5,
     +               PAD6, PAD7, PAD8, PAD9, PAD10
c Number of silicon planes - compared with final value of NDETS
      COMMON /RHSIL/ NSLP
c UGC parameter information:
      integer UgcpMax
      real UGCpars(UgcpMaxp)
c Declare silicon plane parameters; their default values are specified
c in RHSILD.INC
      integer NSIP
c Names of parameters:
      CHARACTER*4 SINAMH(NSipMaxp)
c Mumber of bits stored for each parameter:
      INTEGER SIBITH(NSipMaxp)
c Multiplicative scale factors and origins, defined so that
c   Stored value = (Parameter value + SIORIG) * SIFACT:
      REAL SIFACT(NSipMaxp), SIORIG(NSipMaxp)
c Common for access to UGC parameters and hit parameter definitions:
      common /UGCPAR/ UgcpMax, UGCpars,
     1                NSIP, SINAMH, SIBITH, SIORIG, SIFACT
      real GEOV  !  geometry version number
      common /ugGEOV/ GEOV
*KEEP,RHSYS.
C  Common blocks for GPH PHOBOS simulation:  FILE RHSYS.CMN
C  General geometric layout of PHOBOS detector exclusive of detectors
C  These values are on input cards or data statements, except *MAX are
C  calculated to encompass all elements of the system.
C  20-Sep-96  K.W       Removed obsolete BMTBRAD, BERTHIK
C                       Removed FANGL - now local variable in BANG
C  12-Jul-96  K.W       Position of the magnet pole center MAGX, MAGZ
C                       (previously XOFF,ZOFF) separated from the magnet
C                       offset (now MOFF - 3-D vector)
C  14 Dec-92  R.V.      TMAG redefined and changed to integer
C  22-Nov-92 D. McLeod  Renamed from COMMON.SYS; TMAG, ZOFF, XOFF
C                       are added from the original GPH version
      REAL   BFLD         ! Magnetic field strength
     &      ,MAGX,MAGZ    ! Coordinates of field center
     &      ,MOFF(3)      ! offset of the field
     &      ,RFLD         ! Radius of field (perfect dipole only)
     1   ,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      INTEGER MAGT, TMAG  ! Field type: 0 Zebra bank, 1 = dipole,
                          ! 2 = ASCII map
      COMMON/RHSYS/ BFLD,MAGT,MAGX,MAGZ,MOFF
     1  ,RFLD,XMAX,YMAX,ZMAX,ZMIN,XMMAX,ZMMAX
      EQUIVALENCE (TMAG, MAGT)
*KEND.

C  Local declarations:
c UGCDebug = 0 for no output, 1 for fatal errors, ...
      integer IelMin, ipl, nn, Nvstore, UGCDebug /1/

c End of declarations

      Ires = 1
      Nvstore = Nvalues
      IelMin = MAX(Iielem, 1)

c GEOV - geometry version number (real) in format 3.02
c provided to identify the geometry used i nthe simulations
      if (Key .eq. 'GEOV') then
        GEOV = Rvalues(1)

c Bfld is used to scale the magnetic field:
      elseif (Key .eq. 'BFLD') then
        BFLD = Rvalues(1)

c The next vectors are used for calls to the Geant hit subroutine,
c GSDETH

      elseif (Key .eq. 'NBITSH') then
c Get the numbers of bits to save for each hit parameter
        nvstore = MIN(Nvalues, NSIP - IelMin + 1)
        do nn = 1, nvstore
          SIBITH(nn+IelMin-1) = Ivalues(nn)
        enddo

      elseif (Key .eq. 'FACT') then
c Get the hit parameter scale factors
        nvstore = MIN(Nvalues, NSIP - IelMin + 1)
        do nn = 1, nvstore
          SIFACT(nn+IelMin-1) = Rvalues(nn)
        enddo

      elseif (Key .eq. 'IUDET') then
C OBSOLETE - KEPT FOR BACKWARD COMPATIBILITY ONLY:
c Get the user-assigned names of the detector planes, passed in the
c character*4 array DETNAME in common RHSIL to PH_OUT.
c The UGC statement
c   GS SET NPLANES nn
c should precede any SET that depends on NPLANES.
c Names are read in as a single string; long strings can be read
c by specifying segments on more than one record, e.g. by
c   GS SET IUDET(1) 'aaaabbbbcccc...'
c   GS SET IUDET(21) 'wwwwxxxxyyyy...'
        IF(PADSNEW)  THEN
           WRITE(*,*) ' *** ERROR IN UGCSET - GS SET PADS MIXED WITH ',
     +     ' GS SET IUDET  OR  GS SET SLP'
           WRITE(*,*) '     GS SET IUDET  - IGNORED'
        ELSE
          PADSOLD = .TRUE.
          nvstore = MIN((Vlengths(1)+3)/4, NSLP - IelMin + 1)
          do nn = 1, nvstore
            DETNAME(nn+IelMin-1) = Svalues(4*nn-3:4*nn)
          enddo
        ENDIF

      elseif (Key .eq. 'PADS') then
C SUBSTITUTES:   GS SET IUDET  and  GS SET SLP
C Reads the name of the volume, pad type flag and up to 4 pad parameters
        IF(PADSOLD)  THEN
           WRITE(*,*) ' *** ERROR IN UGCSET - GS SET PADS MIXED WITH ',
     +     ' GS SET IUDET  OR  GS SET SLP'
        ELSE
          PADSNEW = .TRUE.
          NDETS = NDETS + 1
          IF(NDETS .GT. NSLPM)
     +      CALL STOPER('UGCSET',
     +        'TOO MANY DETECTORS - INCREASE TABLES SIZE NSLPM',1)
          DETNAME(NDETS) = SVALUES(1:4)
          IPADFL(NDETS) = IVALUES(1)
          PAD1(NDETS) = RVALUES(1)
          PAD2(NDETS) = RVALUES(2)
          PAD3(NDETS) = RVALUES(3)
          PAD4(NDETS) = RVALUES(4)
          PAD5(NDETS) = RVALUES(5)
          PAD6(NDETS) = RVALUES(6)
          PAD7(NDETS) = RVALUES(7)
          PAD8(NDETS) = RVALUES(8)
          PAD9(NDETS) = RVALUES(9)
          PAD10(NDETS) = RVALUES(10)
          DIVXNAME(NDETS) = '????'
          DIVYNAME(NDETS) = '????'
          DIVZNAME(NDETS) = '????'
          NDIVX(NDETS) = 0
          NDIVY(NDETS) = 0
          NDIVZ(NDETS) = 0
        ENDIF

      elseif (Key .eq. 'NAMESH') then
c Assign names of hit parameters to be saved by Geant
        nvstore = MIN((Vlengths(1)+3)/4, NSIP - IelMin + 1)
        do nn = 1, nvstore
          SINAMH(nn+IelMin-1) = Svalues(4*nn-3:4*nn)
        enddo

      elseif (Key .eq. 'NPLANES') then
c Get number of silicon planes
        NSLP = Ivalues(1)
        IF(NSLP .GT. NSLPM)
     +      CALL STOPER('UGCSET',
     +        'TOO MANY DETECTORS - INCREASE TABLES SIZE NSLPM',1)

      elseif (Key .eq. 'NHITPAR') then
c Get number of hit parameters
        NSIP = Ivalues(1)

      elseif (Key .eq. 'ORIG') then
c Get the zeroes of the hit parameters
        nvstore = MIN(Nvalues, NSIP - IelMin + 1)
        do nn = 1, nvstore
          SIORIG(nn+IelMin-1) = Rvalues(nn)
        enddo

      elseif (Key .eq. 'SLP') then
C OBSOLETE - KEPT FOR BACKWARD COMPATIBILITY ONLY:
c Get the hit parameters
        IF(PADSNEW)  THEN
           WRITE(*,*) ' *** ERROR IN UGCSET - GS SET PADS MIXED WITH ',
     +     ' GS SET IUDET  OR  GS SET SLP'
           WRITE(*,*) '     GS SET SLP  - IGNORED'
        ELSE
           PADSOLD = .TRUE.
           Ipl = Ivalues(1)
c Types should be checked here, but aren't at present
           if (Ipl .gt. 0 .and. Ipl .le. NSLPM) then
             IPADFL(IPL) = 1      !   assumed PAD type 1
             PAD1(IPL) = RVALUES(2)
             PAD2(IPL) = RVALUES(4)
             PAD3(IPL) = 0.0
             PAD4(IPL) = 0.0
           else
             Ires = -1
           endif
        ENDIF

      elseif (Key .eq. 'SPARS') then
c note there's no protection for number of values
        nvstore = MIN(Nvalues, UgcpMax - IelMin + 1)
        do nn = 1, nvstore
          UGCpars(nn) = Rvalues(nn)
        enddo

      else
        Ires = 0
      endif

      if (UGCDebug .gt. 0) then
        if (Ires .gt. 0) then
          if (UGCDebug .ge. 2)
     *      print '(3a,i2,a,/5(e12.4,a))', ' UGCSET: variable ', Key,
     *      '(1..', min(Nvstore, 10), ')  set to ',
     *      (Rvalues(nn), ', ', nn=1, min(Nvstore, 10))
        elseif (Ires .eq. 0) then
          if (UGCDebug .ge. 2)
     *      print '(3a)', ' UGCSET: "', Key, '" not recognized.'
        else
          print '(4a)', ' *** ERROR IN UGCSET - ',
     +      'invalid values specified for',
     *      ' variable "', Key, '" ignored.'
        endif
      endif

      return
      end
