      SUBROUTINE USSTEP
*Documentation_Block
************************************************************************
*
*  Description:
*  User extension to handle produced secondary tracks.
*
*  This version creates banks in the JKINE and JVERT data structures for
*  no, selected, or all tracks, depending on the control variable ISECT
*  set by a SECT control statement:
*  SECT 0 drops all secondaries.
*  SECT 1 tracks all secondaries but creates VERT and KINE banks for all
*    primary tracks and for secondary tracks that produce at least one
*    HIT.
*  SECT 2 tracks and creates VERT and KINE banks for all tracks.
*  Additional user words, defined in UJKBUF.CMN, are also stored
*  in the user KINE bank extension.
*
*  Arguments:
*  none
*
*  Called by:
*  GUSTEP once for each step before returning
*
*  Calls:
*  GSKINE to create track banks, GSVERT to create vertex banks, GSKING
*  to put newly created secondaries on the tracking stack JSTAK,
*  MZGARB to release unused space in the storage division containing KINE
*
*  Implicit inputs, outputs, side effects:
*  inputs: GEANT common blocks contain info on current particle
*  outputs: entries in VERT, KINE and STAK data structures
*
*  History:
*  Date       Name       Description
*  ----       ----       -----------
*  13-Nov-98  K.W.       Redefine UBuf(11) as negative for particles which
*                        stop (decay etc.) not inside a sensor
*  20-Aug-97  RV         Save nlts and other variables for Linux
*  20-Nov-96  K. W.      In SECT=1 mode the origin volume name is set to 0
*                        for secondaries in place of the name of detector hit
*                        (which is current volume when the particle is stored)
*                        The name of the origin volume has to be retrieved later
*  21-Jul-96  R. V.      Removed unused id, ip, jst, mm, stdout
*   1-JUL-96  K. W.      Correct position of the last point on the
*                        trajectory stored in the KINE bank
*  08-Jul-96  R. V.      IMPLICIT NONE restored [?], format string con-
*                        tinued across line fixed, ios SAVEd
*  16-May-95  K. W.      IFLG(nn) = ITRA => 0 before GSKING call for
*                        SECT=2, reqired by Geant 3.21, reason unknown
*  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
*  18-Jul-94  K. Wozniak IUBuf(1) -> KCASE, UBuf(11) -> ISTOP
*  18-Jul-94  RV         Implement STRA = 1 to make KINE banks only for
*                        tracks that produce hits; add call to MZGARB;
*                        save # hits on track in common /UJKB/;
*                        replace PSKING call by inline code for clarity
*  1-Nov-93   R. V.      Move NtTot update from GUOUT in case STRA = 1
*  11-Jun-93  MDB        Implement STRA=0; make FLAGS available
*  14-May-93  R. V.      Multipass mods: NtTot, MUPEP.CMN
*   3-Jul-92  RV         Move into USSTEP; call PSKING; change GCBANK
*                        expansion
*  30-Jun-92  RV         Added call to PH_HIT
*  18-Jun-92  RV         Dump only at end of event
*   6-Jun-92  RV         Store secondaries in the JKINE structure, with
*                        final kinematics and parent track index
*  27-May-92  R.Verdier  Original GUSTESI using separate stack
*  10-Jun-92  Chuck Parsons  Creation
************************************************************************
*End_Documentation_Block

      IMPLICIT NONE

c Should the following be declared here, or in the main program?
C
C Declare minimum size GEANT common. True size of the GEANT common is
C determined by the GCBANKSIZE shareable image. The linker will give an
C error message if it is less than this value.
C
      integer nwgean
      real geant
      PARAMETER (NWGEAN=300000)
c* Following changed 4-Jul-92 to avoid overriding the GCBANK layout (RV):
c*      COMMON/GCBANK/GEANT(NWGEAN)
      equivalence (GEANT, NZEBRA)
c* End of mod
C
C
C
      integer npaw,nwpaw
      PARAMETER(NPAW=100000)
C   This declares the MINIMUM size of the PAWC common via the array NWPAW.
C   true size is declared via PAWCSIZE shareable image.
C
      COMMON/PAWC/NWPAW(NPAW)
C
C  The GEANT fixed size commons.
C
*KEEP,GCBANK.
      INTEGER IQ,LQ,NZEBRA,IXSTOR,IXDIV,IXCONS,LMAIN,LR1,JCG
      INTEGER KWBANK,KWWORK,IWS
      REAL GVERSN,ZVERSN,FENDQ,WS,Q
C
      PARAMETER (KWBANK=69000,KWWORK=5200)
      COMMON/GCBANK/NZEBRA,GVERSN,ZVERSN,IXSTOR,IXDIV,IXCONS,FENDQ(16)
     +             ,LMAIN,LR1,WS(KWBANK)
      DIMENSION IQ(2),Q(2),LQ(8000),IWS(2)
      EQUIVALENCE (Q(1),IQ(1),LQ(9)),(LQ(1),LMAIN),(IWS(1),WS(1))
      EQUIVALENCE (JCG,JGSTAT)
      INTEGER       JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
      COMMON/GCLINK/JDIGI ,JDRAW ,JHEAD ,JHITS ,JKINE ,JMATE ,JPART
     +      ,JROTM ,JRUNG ,JSET  ,JSTAK ,JGSTAT,JTMED ,JTRACK,JVERTX
     +      ,JVOLUM,JXYZ  ,JGPAR ,JGPAR2,JSKLT
C
*KEEP,GCFLAG.
      COMMON/GCFLAG/IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT(10),IFINIT(20),NEVENT,NRNDM(2)
      COMMON/GCFLAX/BATCH, NOLOG
      LOGICAL BATCH, NOLOG
C
      INTEGER       IDEBUG,IDEMIN,IDEMAX,ITEST,IDRUN,IDEVT,IEORUN
     +        ,IEOTRI,IEVENT,ISWIT,IFINIT,NEVENT,NRNDM
C
*KEEP,GCKINE.
      COMMON/GCKINE/IKINE,PKINE(10),ITRA,ISTAK,IVERT,IPART,ITRTYP
     +      ,NAPART(5),AMASS,CHARGE,TLIFE,VERT(3),PVERT(4),IPAOLD
C
      INTEGER       IKINE,ITRA,ISTAK,IVERT,IPART,ITRTYP,NAPART,IPAOLD
      REAL          PKINE,AMASS,CHARGE,TLIFE,VERT,PVERT
C
*KEEP,GCKING.
      INTEGER MXGKIN
      PARAMETER (MXGKIN=100)
      COMMON/GCKING/KCASE,NGKINE,GKIN(5,MXGKIN),
     +                           TOFD(MXGKIN),IFLGK(MXGKIN)
      INTEGER       KCASE,NGKINE ,IFLGK,MXPHOT,NGPHOT
      REAL          GKIN,TOFD,XPHOT
C
      PARAMETER (MXPHOT=800)
      COMMON/GCKIN2/NGPHOT,XPHOT(11,MXPHOT)
C
      COMMON/GCKIN3/GPOS(3,MXGKIN)
      REAL          GPOS
C
*KEEP,GCNUM.
      COMMON/GCNUM/NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT
      COMMON /GCNUMX/ NALIVE,NTMSTO
C
      INTEGER      NMATE ,NVOLUM,NROTM,NTMED,NTMULT,NTRACK,NPART
     +            ,NSTMAX,NVERTX,NHEAD,NBIT ,NALIVE,NTMSTO
C
*KEEP,GCSETS.
      COMMON/GCSETS/IHSET,IHDET,ISET,IDET,IDTYPE,NVNAME,NUMBV(20)
C
      INTEGER       IHSET,IHDET,ISET,IDET,IDTYPE,NVNAME,NUMBV
C
*KEEP,GCSTAK.
      INTEGER  NJTMAX,NJTMIN,NTSTKP,NTSTKS,NDBOOK,NDPUSH,NJFREE,NJGARB,
     +         NJINVO,LINSAV,LMXSAV,NWSTAK,NWINT,NWREAL,NWTRAC
      INTEGER ISTORD
      PARAMETER (NWSTAK=12,NWINT=11,NWREAL=12,NWTRAC=NWINT+NWREAL+5)
      COMMON /GCSTAK/ NJTMAX, NJTMIN, NTSTKP, NTSTKS, NDBOOK, NDPUSH,
     +                NJFREE, NJGARB, NJINVO, LINSAV(15), LMXSAV(15)
      EQUIVALENCE (ISTORD,NJTMIN)
C
*KEEP,GCTRAK.
      INTEGER NMEC,LMEC,NAMEC,NSTEP ,MAXNST,IGNEXT,INWVOL,ISTOP,MAXMEC
     + ,IGAUTO,IEKBIN,ILOSL, IMULL,INGOTO,NLDOWN,NLEVIN,NLVSAV,ISTORY
     + ,MAXME1,NAMEC1
      REAL  VECT,GETOT,GEKIN,VOUT,DESTEP,DESTEL,SAFETY,SLENG ,STEP
     + ,SNEXT,SFIELD,TOFG  ,GEKRAT,UPWGHT
      REAL POLAR
      PARAMETER (MAXMEC=30)
      COMMON/GCTRAK/VECT(7),GETOT,GEKIN,VOUT(7),NMEC,LMEC(MAXMEC)
     + ,NAMEC(MAXMEC),NSTEP ,MAXNST,DESTEP,DESTEL,SAFETY,SLENG
     + ,STEP  ,SNEXT ,SFIELD,TOFG  ,GEKRAT,UPWGHT,IGNEXT,INWVOL
     + ,ISTOP ,IGAUTO,IEKBIN, ILOSL, IMULL,INGOTO,NLDOWN,NLEVIN
     + ,NLVSAV,ISTORY
      PARAMETER (MAXME1=30)
      COMMON/GCTPOL/POLAR(3), NAMEC1(MAXME1)
C
*KEEP,GCVOLU.
      COMMON/GCVOLU/NLEVEL,NAMES(15),NUMBER(15),
     +LVOLUM(15),LINDEX(15),INFROM,NLEVMX,NLDEV(15),LINMX(15),
     +GTRAN(3,15),GRMAT(10,15),GONLY(15),GLX(3)
C
      INTEGER NLEVEL,NAMES,NUMBER,LVOLUM,LINDEX,INFROM,NLEVMX,
     +        NLDEV,LINMX
      REAL GTRAN,GRMAT,GONLY,GLX
*KEEP,IO.
C Common block for GPH logical units etc.: file IO.CMN
c 13-Aug-96  R. V.       Added USRBATCHK for KUIP commands for UGCONS
c 21-Jul-96  R. V.       Remove unused record; note CARDDEF now obsolete
c 22-Feb-93  R. V.       Added USRMCNTU and IMCNTU for MC Ntuple
c 14-Nov-92  R. Verdier  Added UGCIN, IUGCIN, CARDDEF and ICARDDEF
c                        for UGC and EUCARD (obsolete) input units
C    Apr-92  C. Parsons  Replacement for original COMMON.CDT
      INTEGER USRGCIN, CARDDEF, USRIN, USROUT, USRMAP, USRNTU, USRZEB,
     *  USRMCNTU, USRBATKUMAC
      PARAMETER (USRIN = 4, USROUT = 16, USRMAP = 18)
      PARAMETER (USRNTU = 15, USRZEB = 17, USRGCIN = 19, CARDDEF = 20)
      PARAMETER (USRMCNTU = 21, USRBATKUMAC = 22)
      INTEGER ICARDDEF, IMCNTU, IUSRGCIN, IUSRIN, IUSROUT, IUSRMAP,
     *  IUSRNTU, IUSRZEB, IUSRAOU, IUSRBATKUMAC
      COMMON /IO/ IUSRIN, IUSROUT, IUSRMAP, IUSRNTU, IUSRZEB, IUSRGCIN,
     *  ICARDDEF, IMCNTU, IUSRAOU, IUSRBATKUMAC
*KEEP,MUPEP.
c MUPEP.CMN: PHOBOS MC multi-pass event processing variables
c 16-Mar-95 K. W.     NeSkip added
c  6-Jun-93 R. V.     Creation
c Multi-pass event processing breaks up each event into groups of
c tracks, in order to avoid overflowing Zebra track storage.
c Common for access to control variables:
      common /MUPEP/ EPState, NtConc, NtpE, NtPrim, NeSkip, NtSkip,
     1  MPind, NtPrev, NtTot, NtMax, NvPrev
c Parameters and data values are assigned in MUPEP.CDA.
c Event processing state: EPState = 0 for first pass,
c 1 for continuing pass, 2 for final pass
      integer EPState
c Current pass index:
      integer MPind
c Maximum number of tracks to process concurrently in a group:
      integer NtConc
c Maximum track number for this group:
      integer NtMax
c Maximum number of tracks per event:
      integer NtpE
c Number of primary and secondary tracks generated before this pass:
      integer NtPrev
c Number of primary tracks and vertices generated:
      integer NtPrim, NvPrev
c Number of input file events to skip initially:
      integer NeSkip
c Number of tracks to skip initially:
      integer NtSkip
c Total number of primary and secondary tracks generated:
      integer NtTot
*KEEP,UJKBUF.
C  PHOBOS JKINE user buffer definitions.          File: UJKBUF.INC
c 18-Jul-94  R. V.  Added NHitsAct
	integer Nwubuf, Nwubmax, NHitsAct
	parameter (Nwubmax = 11)
	common /UJKB/ Nwubuf, Ubuf(Nwubmax), NHitsAct
	real Ubuf
	integer IUBUF(Nwubmax)
	equivalence (IUBUF, UBUF)
c NHitsAct:   The number of hits on the track being propagated
c The user words at present are:
c UBUF(1):    Process in which particle was created (DCAY, BREM etc.)
c     (2):    Parent track number, negative is parent not stored
c             (possible in SECT=1 mode)
c     (3):    Volume number at origin, (4 characters in one integer word)
c             not definded for primaries and undefined for all tracks
c             in SECT=1 mode (integer value 0)
c     (4-6):  X, Y, Z at termination
c     (7-10): Px, Py, Pz, E at termination
c     (11):   Istop at track termination, positive when particle stopped
c             in a sensitive volume, negative otherwize
*KEEP,FLAG.
C  PHOBOS flags, run control, operating modes, misc.   File FLAG.CMN
C   7-Jul-98  K. W.      Added  ITRIG flag (initial numer in TRIG command)
C  27-Nov-97  K. W.      Added IAOUF flag
C  31-Mar-95  R.V.       Change STRA to SECT to avoid GEANT 3.21 conflict
C  22-Nov-92  D. McLeod
C   These are set in input cards
      INTEGER ISASC,ISHBO,ISHDO,ISNTU, ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     1        IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB, ITRIG
      REAL WPIXL,WLOSS
      COMMON/RHFLG/ISASC,ISHBO,ISHDO,ISNTU,
     1  ISMEM,ISAOU,IAOUF, ISECT,IDRAW,
     2  IGEN,IANAL,ISCAT,IPIXL,ISEQ,NTRK,IZEB,WPIXL,WLOSS, ITRIG
*KEND.
c The following, copied from ZEBRA.CAR, is used only for diagnostics
      COMMON /MZCC/  mzskip(58), LQSTA(21), LQEND(20), NQDMAX(20)
      integer mzskip, LQSTA, LQEND, NQDMAX
C
C     Local definitions:
C     ==================
C
c Local control.  Diagnostic level, Debuglev, should be externally
c controlled; LUD should be assigned to a separate unit

	integer Debuglev /1/ ! diagnostic printout level, none if 0
	integer LUD      /6/ ! diagnostic printout logical unit
	integer Newtl   /50/ ! write tracklist every newtl events
	                     ! if debugging
        integer SaveSec  /2/ ! see SECT above; this is the default

c Temporaries for GEANT calls:
	integer Itl, jk, jku, Nnew, Nt, Nvtx
c Variables for user buffer display and diagnostic print:
	integer ios, ipa, Iparent, ll, Nfw, nn, Ntact, NUBW
	real PhiPFi, PhiPIn, PIn(4), RIn(3), ThePFi, ThePIn
	real DegRad /57.2958/
c Initialization flag:
	integer IevPrev/-1/

c Local storage for temporarily saving the cumulative ID's of tracks
c until we decide to create permanent VERT and KINE banks for
c them.  Note the cumulative ID is used only for diagnostics:
	integer NltsLim
	parameter (NltsLim = 200)
	integer ItStack, Nlts /0/, NltsMax /0/
	integer IdCum, IdCums(NltsLim)
c Saved starting values of r and p for active track
	real Pinit(6)
c Saved parent track number, particle type, and # of active track
	integer IpaSave, IpSave, ITRASave
c Saved cumulative id of active track
	integer IdcAct
c Index of division containing KINE banks, for garbage collection
	integer Idiv_KINE /2/
c Save variables across calls
        save IdcAct, ios, IpaSave, IpSave, ITRASave

C     Executable Statements
C     =====================

c Initialize active track count and # final writes when the event
c sequence number changes.

	if (Ievent .ne. IevPrev) then

          if (ISECT.ge.0 .and. ISECT.le.2) SaveSec=ISECT
	  Nfw = 0  ! Clear # final writes
	  if (debuglev .ge. 2)
     1   write (LUD,*) ' USSTEP: starting event ', Ievent

c Initialize the user words for the primary tracks, which are
c already stored in the JKINE structure.

	  do Itl = 1, NTRACK

	    jk = LQ(JKINE-Itl)
	    if (jk .gt. 0) then
	      jku = LQ(jk-1)
	      if (jku .gt. 0) then
	        do nn = 1, Nwubmax
	          Q(jku+nn) = 0.
	        enddo
	        IQ(jku+3) = NAMES(NLEVEL)
	      endif ! jku
	    endif ! jk

	  enddo ! Itl

c If in SaveSec = 1 mode, force the division containing the KINE
c structure, which initially is allocated all unused ZEBRA space, to
c give up its unused space.  This keeps the 1st attempt to add a
c generated track to the tracking stack from aborting on insufficient
c space.  Apparently if the first expansion is done by GSVERT, as
c in SaveSec = 2, the garbage collection is performed automatically.

	  if (Debuglev .ge. 4)
     1   write (LUD,*) ' USSTEP: div limits: 1=', LQSTA(1), '-',
     2     LQEND(1), ', 2=', LQSTA(2), '-', LQEND(2), ', 19=',
     3     LQSTA(19),'-',LQEND(19), ', 20=', LQSTA(20),'-',LQEND(20)

	  if (Idiv_KINE .ne. 0 .and. SaveSec .eq. 1)
     1   CALL MZGARB (Idiv_KINE, 0)

       if (Debuglev .ge. 4)
     1   write (LUD,*) ' USSTEP: div limits: 1=', LQSTA(1), '-',
     2     LQEND(1), ', 2=', LQSTA(2), '-', LQEND(2), ', 19=',
     3     LQSTA(19),'-',LQEND(19), ', 20=', LQSTA(20),'-',LQEND(20)

	  Ntact = NTRACK
	  IdCum = NtPrev + Ntact
	  Nlts = 0
	  NltsMax = 0

	  IevPrev = Ievent

	endif ! Ievent

c Save the starting position, momentum and ID of each track in case
c we need to create VERT and KINE banks for it.  The track belongs to
c KINE bank # ITRA if ISTAK = 0, else to the ISTAKth entry (-1?) in
c the temporary stack

	if (NSTEP .eq. 0) then

c Update the active Id and # of entries in the local tracking storage.
c Clear the number of hits on the active track

	  NHitsAct = 0
	  do nn = 1, 3
	    Pinit(nn) = VECT(nn)
	    Pinit(nn+3) = VECT(nn+3) * VECT(7)
	  enddo

c Debug: write track starting information
	  if (Nlts .eq. 0) then
	    IdcAct = NtPrev + ITRA
	  else
	    IdcAct = IdCums(Nlts)
	    Nlts = Nlts - 1
	  endif
	  if (Debuglev .ge. 4) then
	    if (ISTAK .ne. 0) IpaSave = -IpaSave
	    IpaSave = ITRA
	    IpSave = IPART
	    write (LUD,'(2(a,i6),a,i3,4(a,f7.2))', iostat=ios)
     1     ' USSTEP: IDc=', IdcAct, ', ipa=', IpaSave,
     2     ', ip=', IpSave, ', r=(', VECT(1), ',', VECT(2),
     3     ',', VECT(3), '), p=', VECT(7)
	  endif
c TEMPORARY debug: check that particle type agrees with the currently
c active entry on the tracking stack
	  ItStack = IQ(JSTAK+1) + 1
	  if (IPART .ne. IQ(JSTAK + NWSTAK * ItStack - 7)) then
	    if (Debuglev .ge. 1) write (LUD,'(2(a,i3))', iostat=ios)
     1     ' USSTEP: error, IPART =', IPART,
     2     ', ip saved from tracking stack =',
     3     IQ(JSTAK + NWSTAK * ItStack - 7)
	  endif

	endif ! NSTEP

c If the current track is in a sensitive detector and is for a
c charged particle

	if (ISET .ge. 1 .and. CHARGE .ne. 0.) then

c On entry,

	  if (INWVOL .eq. 1) then

c Update its number of hits

	    NHitsAct = NHitsAct + 1

c Create new VERT and KINE banks if none existed and SaveSec = 1

	    if (ISTAK .ne. 0 .and. SaveSec .eq. 1
     1       .and. NHitsAct .eq. 1) then

	      call GSVERT (Pinit, 0, ITRA, 0, 0, Nvtx)

c Debug: error message if vertex creation failed
	      if (Nvtx .le. 0 .and. Debuglev .ge. 1)
     1       write (LUD,'(a,i6)', iostat=ios)
     2         '  USSTEP: error creating vertex for stack track #',
     3         ISTAK

c Init user buffer.  Negate parent track # to indicate that one or
c more secondary tracks were created since the primary track
	      call UZERO(UBuf, 1, Nwubmax)
	      IUBuf(1) = KCASE
	      UBuf(2) = -(ITRA + NtPrev)
	      IUBuf(3) = 0  !  origin volume of secondary is not the name
C                           !  of the current volume (detector hit)
C                           !  to avoid changes of GEANT volume tree
C                           !  a flag indicates that the name have to be
C                           !  retrieved later (if needed)

	      call GSKINE (Pinit(4), IPART, NVtx, UBuf, Nwubmax, Nt)

c Set ITRA to connect the current track with its new KINE bank
	      ITRA = Nt
c Clear ISTAK to identify this track as having a KINE bank
	      ISTAK = 0

c Debug: note created banks if level >= 3
	      if (Debuglev .ge. 3)
     1       write (LUD,'(4(a,i6))', iostat=ios)
     2         ' USSTEP: created vertex', Nvtx, ', KINE bank', Nt,
     3         ' for IDc = ', IdcAct, ', ip=', IPART

	    endif  !  ISTAK .ne. 0 ...

	  endif  !  INWVOL .eq. 1

c Call PH_HIT to create a HITS bank and store hit information in it.

	  call PH_HIT()

	endif  !  ISET .ge. 1 ...

c If current track interacted or stopped, and has a KINE bank, enter
c terminal information from GCTRAK and GCKINE into the user part

	if (Istop .ne. 0) then

	  if (ISTAK .eq. 0  .or. SaveSec .eq. 2) then
	    jk = LQ(JKINE-ITRA)
	    if (jk .gt. 0) then
	      jku = LQ(jk-1)
	      if (jku .gt. 0) then
c Bookkeeping check: does the stopped track have the correct
c particle type?
	        if (INT(Q(jk+5)) .ne. IPART .and. Debuglev .ge. 1)
     1         write (LUD,'(2(a,i3))',iostat=ios)
     2            ' USSTEP: error, IPART=', IPART, ', active ptype=',
     3            INT(Q(jk+5))
c Save stop flag, final position, momentum, and total energy in the
c user extension to the KINE bank
	        do nn = 1, 3
	          Q(jku+nn+3) = VECT(nn)
	          Q(jku+nn+6) = VECT(7) * VECT(nn+3)
	        enddo
	        Q(jku+10) = GETOT
	        IQ(jku+11) = ISTOP
                IF(ISET .LE. 0  .OR.  IHSET .EQ. 0)
     +                                        IQ(jku+11) = -ISTOP
	      endif ! jku
	    endif ! jk

	  endif ! ISTAK

	  Ntact = Ntact - 1

	endif ! Istop

c Process any secondary tracks produced in this step and update counts.
c Note the id's printed out here are not necessarily correct, since
c the new tracks may have kine banks created later, or be discarded.
c Option: bypass processing if SaveSec is 0

	if (SaveSec .eq. 0) go to 100

	Nnew = NGKINE

	if (Nnew .gt. 0) then

	  if (debuglev .ge. 4) then
	    write (LUD,'(a,2(i4,a),z8,a)', iostat=ios)
     1     ' USSTEP: step ', nstep,
     2     ' found', Nnew, ' secondary tracks produced by mech. ',
     3     KCASE, ':'
	    do nn = 1, Nnew
	      ipa = GKIN(5,nn)
	      write (LUD,'(2(a,i5),a,f8.3,a,3f8.3)',iostat=ios)
     1       ' Id=', NtTot+nn, ', ip=', ipa,
     2       ', E=', GKIN(4,nn), ', p=', (GKIN(ll,nn), ll = 1, 3)
	    end do
	  endif

	  if (SaveSec .eq. 1) then

c If SaveSec = 1, call GSKING to put the track on the tracking stack;
c we will create KINE and VERT banks later, if the track produces hits

c Limit saved tracks to available local storage
	    if (Nlts + Nnew .gt. NltsLim) then
	      if (debuglev .ge. 1) then
	        write (LUD, '(3(a,i6))',iostat=ios) ' USSTEP: error,',
     1            Nlts, ' tracks on stack, saved', NltsLim - Nlts,
     2            ' of requested', Nnew
	      endif
	      Nnew = MAX(NltsLim - Nlts, 0)
	    endif

	    do nn = 1, Nnew
	      IdCums(Nlts + nn) = NtTot + nn
c Set flag to 0 so GSKING will not create a new KINE bank
	      IFLGK(nn) = 0
	      call GSKING(nn)
	    end do

	    Nlts = Nlts + Nnew
	    NltsMax = MAX(Nlts, NltsMax)

	  else if (SaveSec .ge. 2) then

c If SaveSec = 2, create a new vertex for the secondaries at the
c current point; the vertex index, Nvtx, is returned by GSVERT.
c Create KINE banks for all the secondary tracks in the JKINE
c structure, adding space for user words to contain stop flag,
c parent id, and final kimematics.  Also put the new tracks on the
c tracking stack, where they will be picked up and propagated AFTER
c the current track has stopped or decayed.
c At this point we could set IFLGK < 0 for low energy tracks and
c neutrinos to save time by discarding them.

	    call GSVERT (VECT, 0, ITRA, 0, 0, Nvtx)

	    ITRASave = ITRA

	    do nn = 1, Nnew

	      call UZERO(UBuf, 1, Nwubmax)
	      IUBuf(1) = KCASE
	      UBuf(2) = ITRASave + NtPrev
	      IUBuf(3) = Names(NLevel)

c Create a KINE bank for the new track
	      call GSKINE (GKIN(1,nn), INT(GKIN(5,nn)), NVtx, UBuf,
     1       Nwubmax, Nt)

c Create a tracking stack entry for the new track
	      ITRA = Nt
	      IFLGK(nn) = 0
	      call GSKING(nn)

	    end do  ! nn

	    ITRA = ITRASave

	    if (debuglev .ge. 6) then
c Debug: print space points for all tracks and tracking and physics
c parameters after the current step
	      write (LUD,'(a)') ' USSTEP: GPJXYZ(0) output:'
	      call GPJXYZ(0)
	      write (LUD,'(a)') ' USSTEP: GPCXYZ output:'
	      call GPCXYZ
	    endif
	  endif  ! SaveSec

	  NtTot = NtTot + Nnew
	  Ntact = Ntact + Nnew

        endif ! Nnew .gt. 0

100	continue

c Debug: display current track at each step
	if (debuglev .ge. 5) then
	  if (mod(Nstep,10) .eq. 0) ! NB-- this restarts for each track
     1   write (LUD,'(12a)') ' St#', '  Ip', '      x', '      y',
     2   '      z', '     tx', '     ty', '     tz', '       p',
     3   '      dE', ' Stp', ' Mec'
	  write (LUD,'(2i4,3f7.2,3f7.3,2f8.3,2i4)', iostat=ios)
     1   Nstep, Ipart, (Vect(nn), nn=1,7), DeStep, Istop, Nmec
	endif

c Debug: write out final track list every Newtl events
c (this should be a user exit call)

	if (Debuglev .ge. 2 .and. NTRACK .gt. 0 .and. Nfw .eq. 0 .and.
     1   mod(Ievent,Newtl) .eq. 0 .and. Ntact .eq. 0) then

	  write (LUD,'(a,3(i6,a))') ' USSTEP: The track list for event',
     1   Ievent, ' now has', NTRACK, ' total entries,', Ntact,
     2   ' active.'
	  write (LUD,'(10a)')
     1   '    Id', '  Ip', '         ', 'Idpar', '       E',
     2   '      x', '      y', '      z', ' Thetap', '   Phip'
	  do Itl = 1, NTRACK
	    call GFKINE(Itl, RIn, PIn, ipa, Nvtx, UBuf, NUBW)
	    if (PIn(1) .ne. 0. .or. PIn(2) .ne. 0.
     1     .or. PIn(3) .ne. 0.) then
	      ThePIn = atan2(sqrt(PIn(1)**2 + PIn(2)**2), PIn(3))
	    else
	      ThePIn = 0.
	    endif
	    if (PIn(1) .ne. 0. .or. PIn(2) .ne. 0.) then
	      PhiPIn = atan2(PIn(2), PIn(1))
       	    else
	      PhiPIn = 0.
	    endif
	    if (UBuf(7) .ne. 0. .or. UBuf(8) .ne. 0.
     1     .or. UBuf(9) .ne. 0.) then
	      ThePFi = atan2(sqrt(UBuf(7)**2 + UBuf(8)**2), UBuf(9))
	    else
	      ThePFi = 0.
	    endif
	    if (UBuf(7) .ne. 0. .or. UBuf(8) .ne. 0.) then
	      PhiPFi = atan2(UBuf(8), UBuf(7))
       	    else
	      PhiPFi = 0.
	    endif
	    Iparent = UBuf(2)
	    write (LUD,'(1x,i6,i6,1x,2a4,i5,f8.3,3f7.2,2f7.1 /'//
     1     '7x,a8,i6,a1,5x,f8.3,3f7.2,2f7.1)', iostat=ios)
     2     Itl + NtTot, ipa, (IQ(LQ(JPART-ipa)+ll), ll=1,2),
     3     Iparent, PIn(4), (RIn(nn), nn=1,3),
     4     DegRad * ThePIn, DegRad * PhiPIn,
     5     'at stop', int(UBuf(1)), ':', UBuf(10),
     6     (UBuf(nn+3), nn=1,3), DegRad * ThePFi,
     7     DegRad * PhiPFi
	  enddo ! Itl

	endif ! Debuglev + ...

 	return
	end
