#ifndef _PhMerging_INCLUDED_
#define _PhMerging_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhMerging                                                     //
//                                                                   //
//    PHOBOS detector simulation class                               //
//                                                                   //
//    TPhMerging provides access to merged spectrometer hits         //
//                                                                   //
//    Author  : Krzysztof Wozniak (krzysztof.wozniak@ifj.edu.pl)     //
//    Created : 15/10/98                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// ROOT classes

#include "TObject.h"
#include "TObjArray.h"

// Phobos classes
#include "TPhRecHit.h"

//class TPhHit;
class TPhEvent;
class TPhTrack;
class TPhMCTrack;
class TPhRecTrack;
class TPhSensor;
class TPhDetector;

const Int_t kMaxSpecLayers=16;
const Int_t kMaxHitNumber=5000;
const Int_t knhmax=500;  

class TPhMerging : public TObject {

public:
    TPhMerging();
    TPhMerging(Char_t detside);

    virtual ~TPhMerging();

    virtual void         SetMergingType(Int_t i)   { fMergingType = i;     }
    virtual void         SetMaxMergedPads(Int_t i);

    virtual Char_t       GetDetSide()             { return fDetSide;       }
    virtual Int_t        GetMergingType()         { return fMergingType;   }
    virtual Int_t        GetMaxMergedPads()       { return fMaxMergedPads; }
    virtual Int_t        GetNSpecHits()           { return fNSpecHits;     }
    virtual Int_t        GetNTOFHits()            { return fNTOFHits;      }
    virtual Int_t        GetNMergedHits()         { return fNMergedHits;   }
    virtual Int_t        GetSpecHitIndexes(Int_t layer, Int_t *index1, Int_t *index2);
    virtual Int_t        GetSpecHitIndexes(Int_t layer, Float_t d45min, Float_t d45max, Int_t *index1, Int_t *index2);
    virtual Int_t        GetTOFHitIndexes(Int_t *index1, Int_t *index2);
    virtual Int_t        GetTOFHitIndexes(Float_t d45min, Float_t d45max, Int_t *index1, Int_t *index2);
    virtual TPhRecHit   *GetHit(Int_t index);

    virtual Int_t        MergeSpecHits();
    virtual Int_t        MergeSpecHits(Char_t detside);
    virtual Int_t        MergeTOFHits();
    virtual Char_t      *GetSpecArmName()  {  return  fSpecArmName;  }
    virtual Char_t      *GetTOFArmName()   {  return  fTOFArmName;   }

    virtual TPhMCTrack  *MergeMCTrack(TPhMCTrack *mctrack);
    virtual TPhMCTrack  *CloneMCTrack(TPhMCTrack *mctrack);

    virtual void         SetVerbose(Int_t i)      { fVerbose  = i;  }

    virtual void         Print(Option_t *option);
    virtual void         Reset();
    virtual void         Reset(Char_t detside);

private:

    virtual Int_t        AddMultipleHits(Int_t i1, Int_t i2);
    virtual void         MergeSensorHits(Int_t i1, Int_t i2);
    virtual void         MergeCluster(Int_t k1, Int_t k2);
    virtual void         RemoveNulls(Int_t *i1, Int_t *i2);

    Int_t       fMergingType;       // Merging algorithm type:
                                    //  0 - no merging
                                    //  1 - merge up to fMaxMergedPads
                                    //  other - to be implemented
    Int_t       fMaxMergedPads;     // maximal number of Pads to be merged

    Char_t      fDetSide;           // Detector side:  P  or  N
    Char_t      fSpecArmName[10];   // Subdetector (spectrometer arm) name
    Char_t      fTOFArmName[10];    // Subdetector (TOF arm) name
    Int_t       fNAllSpecHits;      // number of original (not merged) hits in spectrometer
    Int_t       fNSpecHits;         // number of merged hits in spectrometer
    Int_t       fNAllTOFHits;       // number of original (not merged) TOF hits
    Int_t       fNTOFHits;          // number of merged TOF hits
    Int_t       fNMergedHits;       // number of all merged hits
    Int_t       fLayerIndex1[kMaxSpecLayers];   // position of first hit in a layer
    Int_t       fLayerIndex2[kMaxSpecLayers];   // position of last hit in a layer
    Int_t       fTOFIndex1;         // position of first hit in TOF
    Int_t       fTOFIndex2;         // position of last hit in TOF

    TPhRecHit   *fHitArray[kMaxHitNumber];  // Array of hits 

    TPhDetector *fDetector;         //! PHOBOS Detector Object
    Bool_t      fVerbose;           //! Control debugging output

    TPhHit     *hittable[knhmax];   //! Working table for track hits

public:

    ClassDef(TPhMerging,1) // Stores the merged spectrometer hits

};

#endif
