#include "math.h"

#include "TPhMerging.h"
#include "TPhDetector.h"
#include "TPhSubDetector.h"
#include "TPhLayer.h"
#include "TPhHit.h"
#include "TPhMCHit.h"
#include "TPhRecMCHit.h"
#include "TPhTrack.h"
#include "TPhMCTrack.h"
#include "TPhRecTrack.h"
#include "TPhEvent.h"
#include "TPhStuff.h"
#include "TRandom.h"

#include "TStopwatch.h"
#include "TFile.h"
#include "TKey.h"

ClassImp(TPhMerging)

TPhMerging::TPhMerging()
{
// Default constructor of TPhMerging class.

    fVerbose        = 0;
    fNAllSpecHits   = 0;
    fNSpecHits      = 0;
    fNAllTOFHits    = 0;
    fNTOFHits       = 0;
    fNMergedHits    = 0;
    fMergingType    = 0;
    fMaxMergedPads  = 1;
    fDetSide        = '?';
}

TPhMerging::TPhMerging(Char_t detside)
{
// Default constructor of TPhMerging class.

    fVerbose        = 0;
    fNAllSpecHits   = 0;
    fNSpecHits      = 0;
    fNAllTOFHits    = 0;
    fNTOFHits       = 0;
    fNMergedHits    = 0;
    fMergingType    = 0;
    fMaxMergedPads  = 1;
    fDetSide        = detside;
    if(detside == 'P' ||  detside == 'N')
    {   
       if(detside == 'P')
       {
          strcpy(fSpecArmName, "SpectP");
          strcpy(fTOFArmName, "TOFP");
       }
       if(detside == 'N')
       {
          strcpy(fSpecArmName, "SpectN");
          strcpy(fTOFArmName, "TOFN");
       }
    }
    else
    {
       fDetSide = '?';
       printf("TPhMerging: unknown detector side  %c  (should be P or N)\n", detside);
    }
}

TPhMerging::~TPhMerging()
{
 // Default destructor of TPhMerging class.
    Reset();
}

void TPhMerging::Reset()
{
    Int_t i;

    if(fNMergedHits > 0)
       for(i=0; i< fNMergedHits; i++)
          if(fHitArray[i])  delete  fHitArray[i];
    fNAllSpecHits   = 0;
    fNSpecHits      = 0;
    fNAllTOFHits    = 0;
    fNTOFHits       = 0;
    fNMergedHits    = 0;
}

void TPhMerging::Reset(Char_t detside)
{
    Int_t i;

    if(fNMergedHits > 0)
       for(i=0; i< fNMergedHits; i++)
          if(fHitArray[i])  delete  fHitArray[i];
    fNAllSpecHits   = 0;
    fNSpecHits      = 0;
    fNAllTOFHits    = 0;
    fNTOFHits       = 0;
    fNMergedHits    = 0;
    fDetSide        = detside;
    if(detside == 'P' ||  detside == 'N')
    {
       if(detside == 'P')
       {
          strcpy(fSpecArmName, "SpectP");
          strcpy(fTOFArmName, "TOFP");
       }
       if(detside == 'N')
       {
          strcpy(fSpecArmName, "SpectN");
          strcpy(fTOFArmName, "TOFN");
       }
    }
    else
    {
       fDetSide = '?';
       printf("TPhMerging: unknown detector side  %c  (should be P or N)\n", detside);
    }
}

void TPhMerging::SetMaxMergedPads(Int_t i) 
 { 
   if(i < 1  ||  i > kNPadsMax)
      printf("TPhMerging::SetMaxMergedPads: the values allowed are: 1 to %2d\n", kNPadsMax);
   else
      fMaxMergedPads = i;   
}

TPhRecHit *TPhMerging::GetHit(Int_t index)
{
   // give pointer to the hit with pointed by the index

    if(index < 0 || index >= fNMergedHits)  return  NULL;

    return fHitArray[index];
}

Int_t TPhMerging::GetSpecHitIndexes(Int_t layerno, Int_t *index1, Int_t *index2)
{

    if(layerno >= 0  &&  layerno < kMaxSpecLayers)
    {
       *index1 = fLayerIndex1[layerno];
       *index2 = fLayerIndex2[layerno];
       return (fLayerIndex2[layerno] - fLayerIndex1[layerno] + 1);
    }
    else
    {
       fprintf(stderr, "TPhMerging::GetHitIndexes : wrong layer number: %d\n", layerno);
       *index1 = -1;
       *index2 = -2;
       return 0;
    }
}

Int_t TPhMerging::GetSpecHitIndexes(Int_t layerno, Float_t d45min, Float_t d45max, Int_t *index1, Int_t *index2)
{
   // give indexes of the hits in layerno with d45 in the specified range
    Int_t  i, i1, i2, im;

    if(layerno < 0  ||  layerno >= kMaxSpecLayers)
    {
       fprintf(stderr, "TPhMerging::GetHitIndexes : wrong layer number: %d\n", layerno);
       *index1 = -1;
       *index2 = -2;
       return 0;
    }

    i1 = fLayerIndex1[layerno];
    i2 = fLayerIndex2[layerno];
    if(i1 > i2)   return  0;

    if(fHitArray[i1]->CalcDist45() > d45max  || 
       fHitArray[i2]->CalcDist45() < d45min)
    {
       *index1 = -1;
       *index2 = -2;
       return  0;
    }

    if(i2-i1 > 6)   // reduce range by binary search
    {
       if(fHitArray[i1]->CalcDist45() < d45min)
       {  im = i2;
          while((im-i1) > 3)
          {  i = (im+i1) / 2;
             if(fHitArray[i]->CalcDist45() < d45min)
                i1 = i;
             else
                im = i;
          }
       }
       if(fHitArray[i2]->CalcDist45() > d45max)
       {  im = i1;
          while((i2-im) > 3)
          {  i= (i2+im) / 2;
             if(fHitArray[i]->CalcDist45() > d45max)
                i2 = i;
             else
                im = i;
          }
       }
    }    // if(i2-i1 > 6) 

    while(fHitArray[i1]->CalcDist45() < d45min)  i1++;
    while(fHitArray[i2]->CalcDist45() > d45max)  i2--;

    *index1 = i1;
    *index2 = i2;
    return  i2-i1+1;
}

Int_t TPhMerging::GetTOFHitIndexes(Int_t *index1, Int_t *index2)
{
  // Give indexes of the hits in TOF

    *index1 = fTOFIndex1;
    *index2 = fTOFIndex2;
    return fTOFIndex2-fTOFIndex1+1;
}

Int_t TPhMerging::GetTOFHitIndexes(Float_t d45min, Float_t d45max, Int_t *index1, Int_t *index2)
{
  // Find indexes of the hits in TOF with d45 in the specified range

    Int_t  i, i1, i2, im;

    i1 = fTOFIndex1;
    i2 = fTOFIndex2;
    if(i1 > i2)   return  0;

    if(fHitArray[i1]->CalcDist45() > d45max  ||
       fHitArray[i2]->CalcDist45() < d45min)
    {
       *index1 = -1;
       *index2 = -2;
       return  0;
    }

    if(i2-i1 > 6)   // reduce range by binary search
    {
       if(fHitArray[i1]->CalcDist45() < d45min)
       {  im = i2;
          while((im-i1) > 3)
          {  i = (im+i1) / 2;
             if(fHitArray[i]->CalcDist45() < d45min)
                i1 = i;
             else
                im = i;
          }
       }
       if(fHitArray[i2]->CalcDist45() > d45max)
       {  im = i1;
          while((i2-im) > 3)
          {  i= (i2+im) / 2;
             if(fHitArray[i]->CalcDist45() > d45max)
                i2 = i;
             else
                im = i;
          }
       }
    }    // if(i2-i1 > 6)

    while(fHitArray[i1]->CalcDist45() < d45min)  i1++;
    while(fHitArray[i2]->CalcDist45() > d45max)  i2--;

    *index1 = i1;
    *index2 = i2;
    return  i2-i1+1;
}


Int_t TPhMerging::MergeSpecHits(Char_t detside)
{
    Reset(detside);
    return MergeSpecHits();
}

Int_t TPhMerging::MergeSpecHits()
{
// Merging hits in Spectrometer
// Verbose:
//   0  -  no output
//   1  -  summary
//   2  -  layers level
//   3  -  sensor level
//   4  -  hits level

    Int_t i, i1, i2, k, iy, ky, layerno;
    Int_t index,index1, index2;
    Float_t d45;

    TPhSensor *sensor;
    TPhLayer  *layer;  
    TPhRecHit *hit;
    TPhHit    *rawhit;

    Reset();
    if(fDetSide == '?')
    {
       printf("TPhMerging::MergeSpecHits: Undefined detector side\n");
       return 0;
    }

    if(!fDetector)
    {
        fDetector = TPhDetector::GetInstance();
        if(!fDetector)
        {
            fprintf(stderr,"TPhMerging::MergeSpecHits : Can't access Detector\n");
            return 0;
        }
    }
    TPhSubDetector *SpectrometerArm;


    index = 0;

    SpectrometerArm = fDetector->GetSubDetector(fSpecArmName);
    TIter NextLayer(SpectrometerArm->GetLayerList());
    while( (layer = (TPhLayer *) NextLayer()) )
    {
       if(layer->GetLabel() < kMaxSpecLayers)
       {
          if(fVerbose > 1)
             printf("Merge hits in layer %d (%d sensors)\n",
                layer->GetLabel(),layer->GetSensorList()->GetEntries());
          layerno = layer->GetLabel();
          index1 = index;
          index2 = index-1;
          TIter NextSensor(layer->GetSensorList());
          while( (sensor = (TPhSensor *) NextSensor()) )
          {
             if(fVerbose > 2)
                 if(sensor->GetHitList())
                    printf("Found %d hits in sensor %s \n",
                           sensor->GetHitList()->GetEntries(),
                           sensor->GetChLabel());
             i1 = index; 
             TIter NextHit(sensor->GetHitList());
             while( (rawhit = (TPhHit *) NextHit()) )
             {
                if(index >= kMaxHitNumber)
                {
                    fprintf(stderr, "TPhMerging::MergeSpectrometerHits : too many hits in spectrometer (>%d) remainig ignored\n", fNAllSpecHits);
                    return fNAllSpecHits;
                }
                hit = new TPhRecHit(rawhit);
                fHitArray[index] = hit;
                fNAllSpecHits++;
                index++;
                if(fVerbose > 3)
                     printf("Add hit in layer %d: %g %g \n",
                     hit->GetLayer(),
                     hit->GetX(),  
                     hit->GetZ());
             }
//
// Merging of hits in the sensor
             i2 = index-1;
             if(i2 > i1)
             {
               // Sorting according to increasing PadY and PadX
                 for(i=i1; i<i2; i++)
                   for(k=i+1; k<=i2; k++)
                   {
                      iy = fHitArray[i]->GetPadY();
                      ky = fHitArray[k]->GetPadY();
                      if(iy >= ky)
                      {
                         if(iy > ky)
                         {
                            hit = fHitArray[i];
                            fHitArray[i] = fHitArray[k];
                            fHitArray[k] = hit;
                         }
                         else
                         {  if(fHitArray[i]->GetPadX() > fHitArray[k]->GetPadX())
                            {
                               hit = fHitArray[i];
                               fHitArray[i] = fHitArray[k];
                               fHitArray[k] = hit;
                             }
                         }
                      }
                   }     //  for(i   ...  for(k  ...

               // Merging
                 if(fMergingType > 0)
                 {
                    if(AddMultipleHits(i1,i2))  RemoveNulls(&i1, &i2);
                    MergeSensorHits(i1, i2);
                    RemoveNulls(&i1, &i2);
                 }
                 fNMergedHits += i2 - i1 +1;
                 index2 = i2;
             }
             else   //  not  i2 > i1
             {   
                if(i1 == i2) 
                {
                    fNMergedHits++;
                    index2++;
                }
             }

             index = index2 + 1;

             if(fVerbose > 3)
                for(i=i1; i<index2; i++)
                {
                    hit = fHitArray[i];
                    printf("Merged hit in layer %d: %g %g %d\n",
                      hit->GetLayer(),
                      hit->GetX(),
                      hit->GetZ(),
                      hit->GetNPads());
                }

          }     //  while(sensor = ...
          if(fVerbose > 2)
             printf("Layer %5d  is in the table at %6d  to %6d\n", layerno, index1, index2);
          fLayerIndex1[layerno] = index1;
          fLayerIndex2[layerno] = index2;

          if(index2 > index1)
          { // Sorting according to increasing dist45
             for(i1=index1; i1<index2; i1++)
             {
                d45 = fHitArray[i1]->CalcDist45();
                for(i2=i1+1; i2<=index2; i2++)
                {
                    if(d45 > fHitArray[i2]->CalcDist45())
                    {
                        hit = fHitArray[i1];
                        fHitArray[i1] = fHitArray[i2];
                        fHitArray[i2] = hit;
                    }
                }
             }  //  for(i1=index1; i1<index2; i1++)
          }  //  if(index2 > index1)

       }  //  if(layer->GetLabel() < kMaxSpecLayers)
    }  //  while(layer = (TPhLayer *) NextLayer())

  fNSpecHits = fNMergedHits;
  return fNMergedHits;
}

Int_t TPhMerging::AddMultipleHits(Int_t i1, Int_t i2)
{
   Int_t nadd, i, k, ix, iy;

   if(i2 <= i1)   return  0;

   i = i1;
   nadd = 0;
   while(i < i2)
   {
      ix = fHitArray[i]->GetPadX();
      iy = fHitArray[i]->GetPadY();
      k = i + 1;
      while(fHitArray[k]->GetPadY() == iy  &&
            fHitArray[k]->GetPadX() == ix     )
      {
         fHitArray[i]->AddHit(fHitArray[k]);
         delete fHitArray[k];
         fHitArray[k] = NULL;
         nadd++;
         k++;
         if(k > i2)  break;
      }
      i = k;
   }
   return nadd;
}

void TPhMerging::MergeSensorHits(Int_t i1, Int_t i2)
{
   Int_t ix, ix1, iy1, k1, k2, kk;

   if(i2 <= i1)   return;

   k1 = i1;
   while(k1 < i2)
   {
      ix1 = fHitArray[k1]->GetPadX();
      iy1 = fHitArray[k1]->GetPadY();
      ix = ix1;
      kk = k1 + 1;
      k2 = k1;
      while(fHitArray[kk]->GetPadY() == iy1  &&
            (fHitArray[kk]->GetPadX() - ix) == 1     )
      {
         k2 = kk;
         ix++;
         kk++;
         if(kk > i2)  break;
      }
      if(k2 > k1)   MergeCluster(k1, k2);
      k1 = kk;
   }
}

void TPhMerging::MergeCluster(Int_t k1, Int_t k2)
{
   Int_t j, j1, j2;

   if(k2 <= k1)   return;

   if(fMergingType == 1)
   {
      if(fMaxMergedPads < 2)   return;

      j1 = k1;
      while(j1 <= k2)
      {
         j2 = j1 + fMaxMergedPads - 1;
         j2 = j2 > k2 ?  k2  :  j2;
         for(j=j1+1; j <= j2; j++)
         {
            fHitArray[j1]->MergeHit(fHitArray[j]);
            delete fHitArray[j];
            fHitArray[j] = NULL;
         }
         j1 = j2 + 1;
      }
   }
}


void TPhMerging::RemoveNulls(Int_t *i1, Int_t *i2)
{
   Int_t i, k, ilast;

   if(*i2 < *i1)   return;

   ilast = *i1 - 1;
   for(i=*i1; i<=*i2; i++)
   {
      if(fHitArray[i])
      {
         ilast = i;
      }
      else
      {
         k = i +1;
         while(k <= *i2)
         {
            if(fHitArray[k])
            {
               fHitArray[i] = fHitArray[k];
               fHitArray[k] = NULL;
               ilast = i;
               k = *i2 + 1;
            }
            else    k++;
         }
      }  //  if(fHitArray[i])
   }  //  for(i=*i1; i<=*i2; i++)

   *i2 = ilast;
}


Int_t TPhMerging::MergeTOFHits()
{
// Adding hits from TOF
// Verbose:
//   0  -  no output
//   1  -  summary
//   2  -  TOF level
//   3  -  sensor level
//   4  -  hits level

    Int_t i, i1, i2, k, iy, ky, layerno;
    Int_t index,index1, index2;
    Float_t d45;

    TPhLayer  *layer;
    TPhSensor *sensor;
    TPhRecHit *hit;
    TPhHit    *rawhit;

    if(fDetSide == '?')
    {
       printf("TPhMerging::MergeTOFHits: Undefined detector side\n");
       return 0;
    }

    if(!fDetector)
    {
        fDetector = TPhDetector::GetInstance();
        if(!fDetector)
        {
            fprintf(stderr,"TPhMerging::MergeTOFHits : Can't access Detector\n");
            return 0;
        }
    }
    TPhSubDetector *TOFArm;

    if(fNTOFHits > 0)
       for(i=fTOFIndex1; i< fNMergedHits; i++)
          if(fHitArray[i])  delete  fHitArray[i];

    fNMergedHits    = fNSpecHits;
    index = fNMergedHits;
    fTOFIndex1 = index;
    fTOFIndex2 = index-1;
    fNAllTOFHits = 0;
    fNTOFHits = 0;

    TOFArm = fDetector->GetSubDetector(fTOFArmName);
    TIter NextLayer(TOFArm->GetLayerList());
    while( (layer = (TPhLayer *) NextLayer()) )
    {
       if(fVerbose > 1)
          printf("Merge hits in TOF layer %d (%d sensors)\n",
                layer->GetLabel(),layer->GetSensorList()->GetEntries());
       layerno = layer->GetLabel();
       index1 = index;
       index2 = index-1;
       TIter NextSensor(layer->GetSensorList());
       while( (sensor = (TPhSensor *) NextSensor()) )
       {
          if(fVerbose > 2)
              if(sensor->GetHitList())
                  printf("Found %d hits in sensor %s \n",
                           sensor->GetHitList()->GetEntries(),
                           sensor->GetChLabel());
          i1 = index; 
          TIter NextHit(sensor->GetHitList());
          while( (rawhit = (TPhHit *) NextHit()) )
          {
             if(index >= kMaxHitNumber)
             {
                 fprintf(stderr, "TPhMerging::MergeTOFHits : too many hits in TOF (>%d) remainig ignored\n", fNAllTOFHits);
                 return fNAllTOFHits;
             }
             hit = new TPhRecHit(rawhit);
             fHitArray[index] = hit;
             fNAllTOFHits++;
             index++;
             if(fVerbose > 3)
                  printf("Add hit in TOF layer %d: %g %g \n",
                     hit->GetLayer(),
                     hit->GetX(),  
                     hit->GetZ());
          }
//
// Merging of hits in the sensor
          i2 = index-1;
          if(i2 > i1)
          {
            // Sorting
              for(i=i1; i<i2; i++)
                for(k=i+1; k<=i2; k++)
                {
                   iy = fHitArray[i]->GetPadY();
                   ky = fHitArray[k]->GetPadY();
                   if(iy >= ky)
                   {
                      if(iy > ky)
                      {
                         hit = fHitArray[i];
                         fHitArray[i] = fHitArray[k];
                         fHitArray[k] = hit;
                      }
                      else
                      {  if(fHitArray[i]->GetPadX() > fHitArray[k]->GetPadX())
                         {
                            hit = fHitArray[i];
                            fHitArray[i] = fHitArray[k];
                            fHitArray[k] = hit;
                          }
                      }
                   }
                }     //  for(i   ...  for(k  ...

            // Counting of different hits and (in the case of MC events)
            // merging of hits in the same TOF element
              for(i=i1; i<=i2; i++)
              {     //temporary not implemented !!
                 fNMergedHits++;
                 fNTOFHits++;
                 index2++;
              }
          }
          else   //  not  i1 > i2
          {   if(i1 == i2)  
              {
                 fNMergedHits++;
                 fNTOFHits++;
                 index2++;
              }
          }

          if(fVerbose > 3)
             for(i=i1; i<index2; i++)
             {
                 hit = fHitArray[i];
                 printf("Merged hit in TOF layer %d: %g %g %d\n",
                   hit->GetLayer(),
                   hit->GetX(),
                   hit->GetZ(),
                   hit->GetNPads());
             }

       }     //  while(sensor = ...

       fTOFIndex2 = index2;

    }  //  while(layer = (TPhLayer *) NextLayer())

  if(fTOFIndex2 > fTOFIndex1)
  { // Sorting according to increasing dist45
      for(i1=fTOFIndex1; i1<fTOFIndex2; i1++)
      {
         d45 = fHitArray[i1]->CalcDist45();
         for(i2=i1+1; i2<=fTOFIndex2; i2++)
         {
             if(d45 > fHitArray[i2]->CalcDist45())
             {
                    hit = fHitArray[i1];
                    fHitArray[i1] = fHitArray[i2];
                    fHitArray[i2] = hit;
             }
         }
      }  //  for(i1=index1; i1<index2; i1++)
  }  //  if(index2 > index1)


  fNTOFHits = fTOFIndex2 - fTOFIndex1 + 1;
  return fNTOFHits;
}


void  TPhMerging::Print(Option_t *option)
{
// List all objects in the TPhMerging class
// Use "number" as an option to indicate how detailed output is requested

    Int_t i, nh;
    Int_t level = atoi(option);

    if(level >= 0)
    {
       printf("Registered hits in spectrometer: %7d\n", fNAllSpecHits);
       printf("  Merged   hits in spectrometer: %7d\n", fNSpecHits);
       printf("Registered hits in TOF:          %7d\n", fNAllTOFHits);
       printf("  Merged   hits in TOF:          %7d\n", fNTOFHits);
       printf(" All merged hits:                %7d\n", fNMergedHits);
    }
    if(level >0)
    {
       for(i=0; i<kMaxSpecLayers; i++)
       {
          nh = fLayerIndex2[i] - fLayerIndex1[i] + 1;
          printf("Layer %2d with %6d hits\n", i, nh);
       }
    }
    if(level > 2)
    {
       for(i=0; i<fNMergedHits; i++)  fHitArray[i]->Print(option);
    }
} 

TPhMCTrack*  TPhMerging::CloneMCTrack(TPhMCTrack *mctrack)
{
  // Substitution for MergeMCTrack in case we do not want 
  // to make real berging, but only transform TPhMCHit to TPhRecMCHit
  TPhMCTrack *mrgtrack;
  TPhMCHit *mchit;
  TPhRecMCHit *newhit;
  TObjArray *hitlist;

  Int_t  ih, inh;

  if(!mctrack)  return  NULL;

  mrgtrack = new TPhMCTrack();

  mrgtrack->SetXProd(mctrack->GetXProd());
  mrgtrack->SetYProd(mctrack->GetYProd());
  mrgtrack->SetZProd(mctrack->GetZProd());
  mrgtrack->SetGPid(mctrack->GetGPid());
  mrgtrack->SetMass(mctrack->GetMass());
  mrgtrack->SetPx(mctrack->GetPx());
  mrgtrack->SetPy(mctrack->GetPy());
  mrgtrack->SetPz(mctrack->GetPz());
  mrgtrack->SetCharge(mctrack->GetCharge());

  inh = mctrack->GetNHits();
  hitlist = mctrack->GetHitList();
  for(ih=0; ih<inh; ih++)
  {
     mchit = (TPhMCHit *) hitlist->At(ih);

     newhit = new TPhRecMCHit(mchit);
     newhit->SetNPads(1);
     mrgtrack->AddHit(newhit);
  }
  return mrgtrack;
}

TPhMCTrack*  TPhMerging::MergeMCTrack(TPhMCTrack *mctrack)
{
  // Merge hits in a MCTrack
  // The hits in a sensor are merged if they form a cluster:
  // in the sequence of hits each next hit has PadX and PadY value
  // different no more than +-1 from the values of previous hit
  // This way incorrectly are merged "true" hits only in the case
  // when a particle leaves the sensor to air and enters the same 
  // detector in the same or neighbour pad
  // Values for the cluster:
  //  - calculated as a mean:  
  //      X, Y, Z, PadX, PadY, Field, Psi, Length
  //  - calculated as a sum:
  //      De, Dx, Dy, Dz
  //  - copied from the first hit:
  //      all other including Layer, SensorLabel and Dt

  TPhMCTrack *mrgtrack;
  TPhMCHit *mchit;
  TPhMCHit *mchit1;
  TPhRecMCHit *newhit;
  TObjArray *hitlist;

  Int_t  i, ih, inh, nh;
  Float_t norm;

  if(!mctrack)  return  NULL;

  mrgtrack = new TPhMCTrack();
  
  mrgtrack->SetXProd(mctrack->GetXProd());
  mrgtrack->SetYProd(mctrack->GetYProd());
  mrgtrack->SetZProd(mctrack->GetZProd());
  mrgtrack->SetGPid(mctrack->GetGPid());
  mrgtrack->SetMass(mctrack->GetMass());
  mrgtrack->SetPx(mctrack->GetPx());
  mrgtrack->SetPy(mctrack->GetPy());
  mrgtrack->SetPz(mctrack->GetPz());
  mrgtrack->SetCharge(mctrack->GetCharge());

  inh = mctrack->GetNHits();
  hitlist = mctrack->GetHitList();
  nh = 0;
  for(ih=0; ih<inh; ih++)
  {
     mchit = (TPhMCHit *) hitlist->At(ih);
     if(nh == 0)
     {
        hittable[nh] = mchit;
        nh++;
     }
     else
     {
        if((hittable[0]->GetSensorLabel() == mchit->GetSensorLabel())  &&
           (abs(hittable[0]->GetPadX() - mchit->GetPadX()) < 2)   )
        {
           if(nh < knhmax)
           {
               hittable[nh] = mchit;
               nh++;
           }
           else
               printf("TPhRecTrack::RecFromMCTrack: too many hits in one sensor %d, remaining ignored\n", nh);
        }
        else
        {
           mchit1 = (TPhMCHit *) hittable[0];
           newhit = new TPhRecMCHit(mchit1);
           newhit->SetNPads(nh);
           if(nh > 1)
           {
              for(i=1; i<nh; i++)
              { // Calculate mean values -
                // start from 1 as first hit is already accounted in newhit
                 mchit1 = (TPhMCHit *) hittable[i];
                 newhit->SetX(newhit->GetX() + mchit1->GetX());
                 newhit->SetY(newhit->GetY() + mchit1->GetY());         
                 newhit->SetZ(newhit->GetZ() + mchit1->GetZ()); 
                 newhit->SetPadX(newhit->GetPadX() + mchit1->GetPadX()); 
                 newhit->SetPadY(newhit->GetPadY() + mchit1->GetPadY()); 
                 newhit->SetDe(newhit->GetDe() + mchit1->GetDe());         
                 newhit->SetField(newhit->GetField() + mchit1->GetField()); 
                 newhit->SetDx(newhit->GetDx() + mchit1->GetDx());         
                 newhit->SetDy(newhit->GetDy() + mchit1->GetDy());         
                 newhit->SetDz(newhit->GetDz() + mchit1->GetDz());
                 newhit->SetPsi(newhit->CalcPsi() + mchit1->CalcPsi());
                 newhit->SetLength(newhit->GetLength() + mchit1->GetLength()); 
              }
              norm = nh;
              newhit->SetX(newhit->GetX() / norm);
              newhit->SetY(newhit->GetY() / norm);
              newhit->SetZ(newhit->GetZ() / norm);
              newhit->SetPadX(newhit->GetPadX() / nh);
              newhit->SetPadY(newhit->GetPadY() / nh);
              newhit->SetField(newhit->GetField() / norm);
              newhit->SetPsi(newhit->CalcPsi() / norm);
              newhit->SetLength(newhit->GetLength() / norm);
           }  // if(nh > 1)

           mrgtrack->AddHit(newhit);
           hittable[0] = mchit;
           nh = 1;
           
        }  //  if((hitttable[0]->GetSensorLabel() == ...
     }  //  if(nh == 0)
  }  //  while(mchit = ...

  mchit1 = (TPhMCHit *) hittable[0];
  newhit = new TPhRecMCHit(mchit1);
  newhit->SetNPads(nh);
  if(nh > 1)
  {
     for(i=1; i<nh; i++)
     { // Calculate mean values -
       // start from 1 as first hit is already accounted in newhit
        mchit1 = (TPhMCHit *) hittable[i];
        newhit->SetX(newhit->GetX() + mchit1->GetX());
        newhit->SetY(newhit->GetY() + mchit1->GetY());
        newhit->SetZ(newhit->GetZ() + mchit1->GetZ());
        newhit->SetPadX(newhit->GetPadX() + mchit1->GetPadX());
        newhit->SetPadY(newhit->GetPadY() + mchit1->GetPadY());
        newhit->SetDe(newhit->GetDe() + mchit1->GetDe());
        newhit->SetField(newhit->GetField() + mchit1->GetField());
        newhit->SetDx(newhit->GetDx() + mchit1->GetDx());
        newhit->SetDy(newhit->GetDy() + mchit1->GetDy());
        newhit->SetDz(newhit->GetDz() + mchit1->GetDz());
        newhit->SetPsi(newhit->CalcPsi() + mchit1->CalcPsi());
        newhit->SetLength(newhit->GetLength() + mchit1->GetLength());
     }
     norm = nh;
     newhit->SetX(newhit->GetX() / norm);
     newhit->SetY(newhit->GetY() / norm);
     newhit->SetZ(newhit->GetZ() / norm);
     newhit->SetPadY(newhit->GetPadY() / nh);
     newhit->SetZ(newhit->GetZ() / norm);
     newhit->SetField(newhit->GetField() / norm);
     newhit->SetPsi(newhit->CalcPsi() / norm);
     newhit->SetLength(newhit->GetLength() / norm);
  }  // if(nh > 1)

  mrgtrack->AddHit(newhit);

  return mrgtrack;
}
