//
//
//
//
//
#include <math.h>
#include <stdio.h>
#include "TPhMulguisin.h"

// ClassImp(TPhMulguisin)

TPhMulguisin::TPhMulguisin()
{
  if (DebugMGS) printf("TPhMulguisin\n");
  fFrame	=1;
  fMethod	=0;
  fMode		=1;
  fResol	=0.10;
}

TPhMulguisin::TPhMulguisin(int frame,int method,int mode,float resol)
{
  if (DebugMGS) printf("TPhMulguisin\n");
  fFrame	=frame;
  fMethod	=method;
  fMode		=mode;
  fResol	=resol;
}

TPhMulguisin::TPhMulguisin(int frame,int method,int mode,float resol,int InumC,int* IdetC,float* RentC,float* RetaC,float* RphiC)
{
  if (DebugMGS) printf("TPhMulguisin\n");
  fFrame	=frame;
  fMode		=mode;
  fResol	=resol;
  Clustering(InumC,IdetC,RentC,RetaC,RphiC); 
}

TPhMulguisin::~TPhMulguisin()
{
  if (DebugMGS) printf("~TPhMulguisin\n");
  delete [] IdetG;
  delete [] RentG;
  delete [] RetaG;
  delete [] RphiG;
  delete [] RradG;
}  

void TPhMulguisin::Clustering(int InumC, int* IdetC, float* RentC, float* RetaC, float* RphiC)
{

  // ==========
  // initialise
  // ==========

  int *Iuse0 = new int[InumC];
  int *Iuse1 = new int[InumC];
  int *Iuse2 = new int[InumC];
  
  // =====================
  // initialize cell array
  // =====================
  
  int    InumS = 0;
  int   *IdetS = new int[InumC];  
  float *RentS = new float[InumC];
  float *RetaS = new float[InumC];
  float *RphiS = new float[InumC];

  for (int ic=0; ic<InumC; ic++) Iuse0[ic]=-1;
  
  // ==========
  // sort array
  // ==========
  
  int Idx_max;
  float Ent_max;
  for (int i=0; i<InumC; i++)
  {
    Ent_max=0;
    for (int j=0; j<InumC; j++)
    {
      if (Iuse0[j]==-1)
      {	
        if (RentC[j]>Ent_max)
        {
	  Ent_max=RentC[j];
	  Idx_max=j;
        }
      }	
    }
    Iuse0[Idx_max]=InumS;
    IdetS[InumS]=IdetC[Idx_max];
    if (fMethod==0) RentS[InumS]=1;
    else RentS[InumS]=RentC[Idx_max];
    RetaS[InumS]=RetaC[Idx_max];
    RphiS[InumS]=RphiC[Idx_max];
    InumS++;
  }

  // =========================
  // initialise cluster arrays
  // =========================
  
  InumG = 0;
  IdetG = new int[InumS];
  RentG = new float[InumS];
  RetaG = new float[InumS];
  RphiG = new float[InumS];
  RradG = new float[InumS];

  float *RentH = new float[InumS];
  float *RetaH = new float[InumS];
  float *RphiH = new float[InumS];

  // ==================
  // do clustering step
  // ==================
  
  int Iwg=0;
  float Rcut=fResol;
  float Dist,Dmin,Sent,Seta,Senx,Seny,Sphi;
  float dx,dy;
  for (int is=0; is<InumS; is++)
  { 
    //
    // decide the closest cluster to the cell
    //
    Dmin=100.0; 
    for (int ig=0;ig<InumG; ig++)
    {  
      if (fMode==0) Dist=Rdist(RetaS[is],RphiS[is],RetaH[ig],RphiH[ig]);
      else Dist=Rdist(RetaS[is],RphiS[is],RetaG[ig],RphiG[ig]);
      if (Dist<Dmin) {Dmin=Dist; Iwg=ig;}
    }  
    //
    // determine whether the cell is inside the cluster
    //
    int Inside=0;
    if (fFrame==0)
    {// rectangular
      if (fMode==0)
      {
        dx=Rdist(RetaS[is],0,RetaH[Iwg],0);
        dy=Rdist(0,RphiS[is],0,RphiH[Iwg]);
      }
      else
      {
        dx=Rdist(RetaS[is],0,RetaG[Iwg],0);
        dy=Rdist(0,RphiS[is],0,RphiG[Iwg]);	
      }
      if (dx<fRange[0] && dy<fRange[1]) Inside=1;    
    }
    else 
    {// circular area
      if (Iwg!=0) Rcut=RradG[Iwg];
      if (Dmin<Rcut) Inside=1;
    }
    //
    // if inside, the cell joins to the cluster. if not, create a new cluster 
    //
    if (Inside)
    {
      Iuse2[is]=Iwg; 
	
      Sent=RentG[Iwg]+RentS[is];
      Seta=RentG[Iwg]*RetaG[Iwg]+RentS[is]*RetaS[is];
      Sphi=RentG[Iwg]*RphiG[Iwg]+RentS[is]*RphiS[is];
      Senx=RentG[Iwg]*cos(RphiG[Iwg])+RentS[is]*cos(RphiS[is]);
      Seny=RentG[Iwg]*sin(RphiG[Iwg])+RentS[is]*sin(RphiS[is]);
      
      if (fMethod!=0) RentG[Iwg]=Sent;
      
      RetaG[Iwg]=Seta/Sent;
      if (fFrame==0) RphiG[Iwg]=Sphi/Sent;
      else RphiG[Iwg]=atan2(Seny,Senx);
      
      IdetG[Iwg]|=IdetS[is];
      
    }
    else
    {
      RetaH[InumG]=RetaS[is];
      RphiH[InumG]=RphiS[is];
      RentG[InumG]=RentS[is];
      RetaG[InumG]=RetaS[is];
      RphiG[InumG]=RphiS[is];
      RradG[InumG]=fResol;
      IdetG[InumG]=IdetS[is];
      Iuse2[is]=InumG;
      InumG++;
    }
  }

  //
  // release memory
  //
  
  delete [] Iuse0;
  delete [] Iuse1;
  delete [] Iuse2;
  
  delete [] IdetS;
  delete [] RentS;
  delete [] RetaS;
  delete [] RphiS;
  
  delete [] RentH;
  delete [] RetaH;
  delete [] RphiH;

}  

float TPhMulguisin::Rdist(float eta1,float phi1,float eta0,float phi0)
{
  const float pi =3.141592654;
  const float pi2=6.283185307;
  float deta=eta1-eta0;
  float dphi=phi1-phi0;
  if (fFrame==0)
  {
    return sqrt(deta*deta+dphi*dphi); 
  }
  else
  {
    //dphi=min(abs(dphi),pi2-abs(dphi));
    dphi=(dphi>0)?dphi:-dphi;
    dphi=(pi-dphi>0)?dphi:pi2-dphi;
    return sqrt(deta*deta+dphi*dphi);
  }
}
